/* Capstone Unified Disassembler Engine */
/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013> */

#include <string.h>

#include "mapping.h"
#include "X86DisassemblerDecoder.h"

#include "../../utils.h"
#include "../../include/x86.h"

static x86_reg sib_base_map[] = {
	X86_REG_INVALID,
#define ENTRY(x) X86_REG_##x,
	ALL_SIB_BASES
#undef ENTRY
};

// Fill-ins to make the compiler happy.  These constants are never actually
//   assigned; they are just filler to make an automatically-generated switch
//   statement work.
static enum {
	X86_REG_BX_SI = 500,
	X86_REG_BX_DI = 501,
	X86_REG_BP_SI = 502,
	X86_REG_BP_DI = 503,
	X86_REG_sib   = 504,
	X86_REG_sib64 = 505
} _dummy;

static x86_reg sib_index_map[] = {
	X86_REG_INVALID,
#define ENTRY(x) X86_REG_##x,
	ALL_EA_BASES
	REGS_XMM
	REGS_YMM
	REGS_ZMM
#undef ENTRY
};

static x86_reg segment_map[] = {
	X86_REG_INVALID,
	X86_REG_CS,
	X86_REG_SS,
	X86_REG_DS,
	X86_REG_ES,
	X86_REG_FS,
	X86_REG_GS,
};

x86_reg x86_map_sib_base(int r)
{
	return sib_base_map[r];
}

x86_reg x86_map_sib_index(int r)
{
	return sib_index_map[r];
}

x86_reg x86_map_segment(int r)
{
	return segment_map[r];
}

static name_map reg_name_maps[] = {
	{ X86_REG_INVALID, NULL },
	{ X86_REG_AH, "ah" },
	{ X86_REG_AL, "al" },
	{ X86_REG_AX, "ax" },
	{ X86_REG_BH, "bh" },
	{ X86_REG_BL, "bl" },
	{ X86_REG_BP, "bp" },
	{ X86_REG_BPL, "bpL" },
	{ X86_REG_BX, "bx" },
	{ X86_REG_CH, "ch" },
	{ X86_REG_CL, "cl" },
	{ X86_REG_CS, "cs" },
	{ X86_REG_CX, "cx" },
	{ X86_REG_DH, "dh" },
	{ X86_REG_DI, "di" },
	{ X86_REG_DIL, "dil" },
	{ X86_REG_DL, "dl" },
	{ X86_REG_DS, "ds" },
	{ X86_REG_DX, "dx" },
	{ X86_REG_EAX, "eax" },
	{ X86_REG_EBP, "ebp" },
	{ X86_REG_EBX, "ebx" },
	{ X86_REG_ECX, "ecx" },
	{ X86_REG_EDI, "edi" },
	{ X86_REG_EDX, "edx" },
	{ X86_REG_FLAGS, "flags" },
	{ X86_REG_EIP, "eip" },
	{ X86_REG_EIZ, "eiz" },
	{ X86_REG_ES, "es" },
	{ X86_REG_ESI, "esi" },
	{ X86_REG_ESP, "esp" },
	{ X86_REG_FPSW, "fpsw" },
	{ X86_REG_FS, "fs" },
	{ X86_REG_GS, "gs" },
	{ X86_REG_IP, "ip" },
	{ X86_REG_RAX, "rax" },
	{ X86_REG_RBP, "rbp" },
	{ X86_REG_RBX, "rbx" },
	{ X86_REG_RCX, "rcx" },
	{ X86_REG_RDI, "rdi" },
	{ X86_REG_RDX, "rdx" },
	{ X86_REG_RIP, "rip" },
	{ X86_REG_RIZ, "riz" },
	{ X86_REG_RSI, "rsi" },
	{ X86_REG_RSP, "rsp" },
	{ X86_REG_SI, "si" },
	{ X86_REG_SIL, "sil" },
	{ X86_REG_SP, "sp" },
	{ X86_REG_SPL, "spl" },
	{ X86_REG_SS, "ss" },
	{ X86_REG_CR0, "cr0" },
	{ X86_REG_CR1, "cr1" },
	{ X86_REG_CR2, "cr2" },
	{ X86_REG_CR3, "cr3" },
	{ X86_REG_CR4, "cr4" },
	{ X86_REG_CR5, "cr5" },
	{ X86_REG_CR6, "cr6" },
	{ X86_REG_CR7, "cr7" },
	{ X86_REG_CR8, "cr8" },
	{ X86_REG_CR9, "cr9" },
	{ X86_REG_CR10, "cr10" },
	{ X86_REG_CR11, "cr11" },
	{ X86_REG_CR12, "cr12" },
	{ X86_REG_CR13, "cr13" },
	{ X86_REG_CR14, "cr14" },
	{ X86_REG_CR15, "cr15" },
	{ X86_REG_DR0, "dr0" },
	{ X86_REG_DR1, "dr1" },
	{ X86_REG_DR2, "dr2" },
	{ X86_REG_DR3, "dr3" },
	{ X86_REG_DR4, "dr4" },
	{ X86_REG_DR5, "dr5" },
	{ X86_REG_DR6, "dr6" },
	{ X86_REG_DR7, "dr7" },
	{ X86_REG_FP0, "fp0" },
	{ X86_REG_FP1, "fp1" },
	{ X86_REG_FP2, "fp2" },
	{ X86_REG_FP3, "fp3" },
	{ X86_REG_FP4, "fp4" },
	{ X86_REG_FP5, "fp5" },
	{ X86_REG_FP6, "fp6" },
	{ X86_REG_K0, "k0" },
	{ X86_REG_K1, "k1" },
	{ X86_REG_K2, "k2" },
	{ X86_REG_K3, "k3" },
	{ X86_REG_K4, "k4" },
	{ X86_REG_K5, "k5" },
	{ X86_REG_K6, "k6" },
	{ X86_REG_K7, "k7" },
	{ X86_REG_MM0, "mm0" },
	{ X86_REG_MM1, "mm1" },
	{ X86_REG_MM2, "mm2" },
	{ X86_REG_MM3, "mm3" },
	{ X86_REG_MM4, "mm4" },
	{ X86_REG_MM5, "mm5" },
	{ X86_REG_MM6, "mm6" },
	{ X86_REG_MM7, "mm7" },
	{ X86_REG_R8, "r8" },
	{ X86_REG_R9, "r9" },
	{ X86_REG_R10, "r10" },
	{ X86_REG_R11, "r11" },
	{ X86_REG_R12, "r12" },
	{ X86_REG_R13, "r13" },
	{ X86_REG_R14, "r14" },
	{ X86_REG_R15, "r15" },
	{ X86_REG_ST0, "st0" },
	{ X86_REG_ST1, "st1" },
	{ X86_REG_ST2, "st2" },
	{ X86_REG_ST3, "st3" },
	{ X86_REG_ST4, "st4" },
	{ X86_REG_ST5, "st5" },
	{ X86_REG_ST6, "st6" },
	{ X86_REG_ST7, "st7" },
	{ X86_REG_XMM0, "xmm0" },
	{ X86_REG_XMM1, "xmm1" },
	{ X86_REG_XMM2, "xmm2" },
	{ X86_REG_XMM3, "xmm3" },
	{ X86_REG_XMM4, "xmm4" },
	{ X86_REG_XMM5, "xmm5" },
	{ X86_REG_XMM6, "xmm6" },
	{ X86_REG_XMM7, "xmm7" },
	{ X86_REG_XMM8, "xmm8" },
	{ X86_REG_XMM9, "xmm9" },
	{ X86_REG_XMM10, "xmm10" },
	{ X86_REG_XMM11, "xmm11" },
	{ X86_REG_XMM12, "xmm12" },
	{ X86_REG_XMM13, "xmm13" },
	{ X86_REG_XMM14, "xmm14" },
	{ X86_REG_XMM15, "xmm15" },
	{ X86_REG_XMM16, "xmm16" },
	{ X86_REG_XMM17, "xmm17" },
	{ X86_REG_XMM18, "xmm18" },
	{ X86_REG_XMM19, "xmm19" },
	{ X86_REG_XMM20, "xmm20" },
	{ X86_REG_XMM21, "xmm21" },
	{ X86_REG_XMM22, "xmm22" },
	{ X86_REG_XMM23, "xmm23" },
	{ X86_REG_XMM24, "xmm24" },
	{ X86_REG_XMM25, "xmm25" },
	{ X86_REG_XMM26, "xmm26" },
	{ X86_REG_XMM27, "xmm27" },
	{ X86_REG_XMM28, "xmm28" },
	{ X86_REG_XMM29, "xmm29" },
	{ X86_REG_XMM30, "xmm30" },
	{ X86_REG_XMM31, "xmm31" },
	{ X86_REG_YMM0, "ymm0" },
	{ X86_REG_YMM1, "ymm1" },
	{ X86_REG_YMM2, "ymm2" },
	{ X86_REG_YMM3, "ymm3" },
	{ X86_REG_YMM4, "ymm4" },
	{ X86_REG_YMM5, "ymm5" },
	{ X86_REG_YMM6, "ymm6" },
	{ X86_REG_YMM7, "ymm7" },
	{ X86_REG_YMM8, "ymm8" },
	{ X86_REG_YMM9, "ymm9" },
	{ X86_REG_YMM10, "ymm10" },
	{ X86_REG_YMM11, "ymm11" },
	{ X86_REG_YMM12, "ymm12" },
	{ X86_REG_YMM13, "ymm13" },
	{ X86_REG_YMM14, "ymm14" },
	{ X86_REG_YMM15, "ymm15" },
	{ X86_REG_YMM16, "ymm16" },
	{ X86_REG_YMM17, "ymm17" },
	{ X86_REG_YMM18, "ymm18" },
	{ X86_REG_YMM19, "ymm19" },
	{ X86_REG_YMM20, "ymm20" },
	{ X86_REG_YMM21, "ymm21" },
	{ X86_REG_YMM22, "ymm22" },
	{ X86_REG_YMM23, "ymm23" },
	{ X86_REG_YMM24, "ymm24" },
	{ X86_REG_YMM25, "ymm25" },
	{ X86_REG_YMM26, "ymm26" },
	{ X86_REG_YMM27, "ymm27" },
	{ X86_REG_YMM28, "ymm28" },
	{ X86_REG_YMM29, "ymm29" },
	{ X86_REG_YMM30, "ymm30" },
	{ X86_REG_YMM31, "ymm31" },
	{ X86_REG_ZMM0, "zmm0" },
	{ X86_REG_ZMM1, "zmm1" },
	{ X86_REG_ZMM2, "zmm2" },
	{ X86_REG_ZMM3, "zmm3" },
	{ X86_REG_ZMM4, "zmm4" },
	{ X86_REG_ZMM5, "zmm5" },
	{ X86_REG_ZMM6, "zmm6" },
	{ X86_REG_ZMM7, "zmm7" },
	{ X86_REG_ZMM8, "zmm8" },
	{ X86_REG_ZMM9, "zmm9" },
	{ X86_REG_ZMM10, "zmm10" },
	{ X86_REG_ZMM11, "zmm11" },
	{ X86_REG_ZMM12, "zmm12" },
	{ X86_REG_ZMM13, "zmm13" },
	{ X86_REG_ZMM14, "zmm14" },
	{ X86_REG_ZMM15, "zmm15" },
	{ X86_REG_ZMM16, "zmm16" },
	{ X86_REG_ZMM17, "zmm17" },
	{ X86_REG_ZMM18, "zmm18" },
	{ X86_REG_ZMM19, "zmm19" },
	{ X86_REG_ZMM20, "zmm20" },
	{ X86_REG_ZMM21, "zmm21" },
	{ X86_REG_ZMM22, "zmm22" },
	{ X86_REG_ZMM23, "zmm23" },
	{ X86_REG_ZMM24, "zmm24" },
	{ X86_REG_ZMM25, "zmm25" },
	{ X86_REG_ZMM26, "zmm26" },
	{ X86_REG_ZMM27, "zmm27" },
	{ X86_REG_ZMM28, "zmm28" },
	{ X86_REG_ZMM29, "zmm29" },
	{ X86_REG_ZMM30, "zmm30" },
	{ X86_REG_ZMM31, "zmm31" },
	{ X86_REG_R8B, "r8b" },
	{ X86_REG_R9B, "r9b" },
	{ X86_REG_R10B, "r10b" },
	{ X86_REG_R11B, "r11b" },
	{ X86_REG_R12B, "r12b" },
	{ X86_REG_R13B, "r13b" },
	{ X86_REG_R14B, "r14b" },
	{ X86_REG_R15B, "r15b" },
	{ X86_REG_R8D, "r8d" },
	{ X86_REG_R9D, "r9d" },
	{ X86_REG_R10D, "r10d" },
	{ X86_REG_R11D, "r11d" },
	{ X86_REG_R12D, "r12d" },
	{ X86_REG_R13D, "r13d" },
	{ X86_REG_R14D, "r14d" },
	{ X86_REG_R15D, "r15d" },
	{ X86_REG_R8W, "r8w" },
	{ X86_REG_R9W, "r9w" },
	{ X86_REG_R10W, "r10w" },
	{ X86_REG_R11W, "r11w" },
	{ X86_REG_R12W, "r12w" },
	{ X86_REG_R13W, "r13w" },
	{ X86_REG_R14W, "r14w" },
	{ X86_REG_R15W, "r15w" },
};

char *X86_reg_name(unsigned int reg)
{
	if (reg >= X86_REG_MAX)
		return NULL;

	return reg_name_maps[reg].name;
}

x86_reg x86_map_regname(char *reg)
{
	int i = name2id(&reg_name_maps[1], ARR_SIZE(reg_name_maps) - 1, reg);

	if (i != -1)
		return i;

	// nothing match
	return (i != -1)? i : X86_REG_INVALID;
}

static name_map insn_name_maps[] = {
	{ X86_INS_INVALID, NULL },
	{ X86_INS_AAA, "AAA" },
	{ X86_INS_AAD, "AAD" },
	{ X86_INS_AAM, "AAM" },
	{ X86_INS_AAS, "AAS" },
	{ X86_INS_FABS, "FABS" },
	{ X86_INS_ADC, "ADC" },
	{ X86_INS_ADCX, "ADCX" },
	{ X86_INS_ADD, "ADD" },
	{ X86_INS_ADDPD, "ADDPD" },
	{ X86_INS_ADDPS, "ADDPS" },
	{ X86_INS_ADDSD, "ADDSD" },
	{ X86_INS_ADDSS, "ADDSS" },
	{ X86_INS_ADDSUBPD, "ADDSUBPD" },
	{ X86_INS_ADDSUBPS, "ADDSUBPS" },
	{ X86_INS_FADD, "FADD" },
	{ X86_INS_FIADD, "FIADD" },
	{ X86_INS_FADDP, "FADDP" },
	{ X86_INS_ADOX, "ADOX" },
	{ X86_INS_AESDECLAST, "AESDECLAST" },
	{ X86_INS_AESDEC, "AESDEC" },
	{ X86_INS_AESENCLAST, "AESENCLAST" },
	{ X86_INS_AESENC, "AESENC" },
	{ X86_INS_AESIMC, "AESIMC" },
	{ X86_INS_AESKEYGENASSIST, "AESKEYGENASSIST" },
	{ X86_INS_AND, "AND" },
	{ X86_INS_ANDN, "ANDN" },
	{ X86_INS_ANDNPD, "ANDNPD" },
	{ X86_INS_ANDNPS, "ANDNPS" },
	{ X86_INS_ANDPD, "ANDPD" },
	{ X86_INS_ANDPS, "ANDPS" },
	{ X86_INS_ARPL, "ARPL" },
	{ X86_INS_BEXTR, "BEXTR" },
	{ X86_INS_BLENDPD, "BLENDPD" },
	{ X86_INS_BLENDPS, "BLENDPS" },
	{ X86_INS_BLENDVPD, "BLENDVPD" },
	{ X86_INS_BLENDVPS, "BLENDVPS" },
	{ X86_INS_BLSI, "BLSI" },
	{ X86_INS_BLSMSK, "BLSMSK" },
	{ X86_INS_BLSR, "BLSR" },
	{ X86_INS_BOUND, "BOUND" },
	{ X86_INS_BSF, "BSF" },
	{ X86_INS_BSR, "BSR" },
	{ X86_INS_BSWAP, "BSWAP" },
	{ X86_INS_BT, "BT" },
	{ X86_INS_BTC, "BTC" },
	{ X86_INS_BTR, "BTR" },
	{ X86_INS_BTS, "BTS" },
	{ X86_INS_BZHI, "BZHI" },
	{ X86_INS_CALL, "CALL" },
	{ X86_INS_CALLW, "CALLW" },
	{ X86_INS_CBW, "CBW" },
	{ X86_INS_CDQ, "CDQ" },
	{ X86_INS_CDQE, "CDQE" },
	{ X86_INS_FCHS, "FCHS" },
	{ X86_INS_CLAC, "CLAC" },
	{ X86_INS_CLC, "CLC" },
	{ X86_INS_CLD, "CLD" },
	{ X86_INS_CLFLUSH, "CLFLUSH" },
	{ X86_INS_CLGI, "CLGI" },
	{ X86_INS_CLI, "CLI" },
	{ X86_INS_CLTS, "CLTS" },
	{ X86_INS_CMC, "CMC" },
	{ X86_INS_CMOVA, "CMOVA" },
	{ X86_INS_CMOVAE, "CMOVAE" },
	{ X86_INS_CMOVB, "CMOVB" },
	{ X86_INS_CMOVBE, "CMOVBE" },
	{ X86_INS_FCMOVBE, "FCMOVBE" },
	{ X86_INS_FCMOVB, "FCMOVB" },
	{ X86_INS_CMOVE, "CMOVE" },
	{ X86_INS_FCMOVE, "FCMOVE" },
	{ X86_INS_CMOVG, "CMOVG" },
	{ X86_INS_CMOVGE, "CMOVGE" },
	{ X86_INS_CMOVL, "CMOVL" },
	{ X86_INS_CMOVLE, "CMOVLE" },
	{ X86_INS_FCMOVNBE, "FCMOVNBE" },
	{ X86_INS_FCMOVNB, "FCMOVNB" },
	{ X86_INS_CMOVNE, "CMOVNE" },
	{ X86_INS_FCMOVNE, "FCMOVNE" },
	{ X86_INS_CMOVNO, "CMOVNO" },
	{ X86_INS_CMOVNP, "CMOVNP" },
	{ X86_INS_FCMOVNU, "FCMOVNU" },
	{ X86_INS_CMOVNS, "CMOVNS" },
	{ X86_INS_CMOVO, "CMOVO" },
	{ X86_INS_CMOVP, "CMOVP" },
	{ X86_INS_FCMOVU, "FCMOVU" },
	{ X86_INS_CMOVS, "CMOVS" },
	{ X86_INS_CMP, "CMP" },
	{ X86_INS_CMPPD, "CMPPD" },
	{ X86_INS_CMPPS, "CMPPS" },
	{ X86_INS_CMPSW, "CMPSW" },
	{ X86_INS_CMPSD, "CMPSD" },
	{ X86_INS_CMPSQ, "CMPSQ" },
	{ X86_INS_CMPSB, "CMPSB" },
	{ X86_INS_CMPSS, "CMPSS" },
	{ X86_INS_CMPXCHG16B, "CMPXCHG16B" },
	{ X86_INS_CMPXCHG, "CMPXCHG" },
	{ X86_INS_CMPXCHG8B, "CMPXCHG8B" },
	{ X86_INS_COMISD, "COMISD" },
	{ X86_INS_COMISS, "COMISS" },
	{ X86_INS_FCOMP, "FCOMP" },
	{ X86_INS_FCOMPI, "FCOMPI" },
	{ X86_INS_FCOMI, "FCOMI" },
	{ X86_INS_FCOM, "FCOM" },
	{ X86_INS_FCOS, "FCOS" },
	{ X86_INS_CPUID, "CPUID" },
	{ X86_INS_CQO, "CQO" },
	{ X86_INS_CRC32, "CRC32" },
	{ X86_INS_CS, "CS" },
	{ X86_INS_CVTDQ2PD, "CVTDQ2PD" },
	{ X86_INS_CVTDQ2PS, "CVTDQ2PS" },
	{ X86_INS_CVTPD2DQ, "CVTPD2DQ" },
	{ X86_INS_CVTPD2PS, "CVTPD2PS" },
	{ X86_INS_CVTPS2DQ, "CVTPS2DQ" },
	{ X86_INS_CVTPS2PD, "CVTPS2PD" },
	{ X86_INS_CVTSD2SI, "CVTSD2SI" },
	{ X86_INS_CVTSD2SS, "CVTSD2SS" },
	{ X86_INS_CVTSI2SD, "CVTSI2SD" },
	{ X86_INS_CVTSI2SS, "CVTSI2SS" },
	{ X86_INS_CVTSS2SD, "CVTSS2SD" },
	{ X86_INS_CVTSS2SI, "CVTSS2SI" },
	{ X86_INS_CVTTPD2DQ, "CVTTPD2DQ" },
	{ X86_INS_CVTTPS2DQ, "CVTTPS2DQ" },
	{ X86_INS_CVTTSD2SI, "CVTTSD2SI" },
	{ X86_INS_CVTTSS2SI, "CVTTSS2SI" },
	{ X86_INS_CWD, "CWD" },
	{ X86_INS_CWDE, "CWDE" },
	{ X86_INS_DAA, "DAA" },
	{ X86_INS_DAS, "DAS" },
	{ X86_INS_DATA16, "DATA16" },
	{ X86_INS_DEC, "DEC" },
	{ X86_INS_DIV, "DIV" },
	{ X86_INS_DIVPD, "DIVPD" },
	{ X86_INS_DIVPS, "DIVPS" },
	{ X86_INS_FDIVR, "FDIVR" },
	{ X86_INS_FIDIVR, "FIDIVR" },
	{ X86_INS_FDIVRP, "FDIVRP" },
	{ X86_INS_DIVSD, "DIVSD" },
	{ X86_INS_DIVSS, "DIVSS" },
	{ X86_INS_FDIV, "FDIV" },
	{ X86_INS_FIDIV, "FIDIV" },
	{ X86_INS_FDIVP, "FDIVP" },
	{ X86_INS_DPPD, "DPPD" },
	{ X86_INS_DPPS, "DPPS" },
	{ X86_INS_DS, "DS" },
	{ X86_INS_RET, "RET" },
	{ X86_INS_ENTER, "ENTER" },
	{ X86_INS_ES, "ES" },
	{ X86_INS_EXTRACTPS, "EXTRACTPS" },
	{ X86_INS_EXTRQ, "EXTRQ" },
	{ X86_INS_F2XM1, "F2XM1" },
	{ X86_INS_LCALL, "LCALL" },
	{ X86_INS_LJMP, "LJMP" },
	{ X86_INS_FBLD, "FBLD" },
	{ X86_INS_FBSTP, "FBSTP" },
	{ X86_INS_FCOMPP, "FCOMPP" },
	{ X86_INS_FDECSTP, "FDECSTP" },
	{ X86_INS_FEMMS, "FEMMS" },
	{ X86_INS_FFREE, "FFREE" },
	{ X86_INS_FICOM, "FICOM" },
	{ X86_INS_FICOMP, "FICOMP" },
	{ X86_INS_FINCSTP, "FINCSTP" },
	{ X86_INS_FLDCW, "FLDCW" },
	{ X86_INS_FLDENV, "FLDENV" },
	{ X86_INS_FLDL2E, "FLDL2E" },
	{ X86_INS_FLDL2T, "FLDL2T" },
	{ X86_INS_FLDLG2, "FLDLG2" },
	{ X86_INS_FLDLN2, "FLDLN2" },
	{ X86_INS_FLDPI, "FLDPI" },
	{ X86_INS_FNCLEX, "FNCLEX" },
	{ X86_INS_FNINIT, "FNINIT" },
	{ X86_INS_FNOP, "FNOP" },
	{ X86_INS_FNSTCW, "FNSTCW" },
	{ X86_INS_FNSTSW, "FNSTSW" },
	{ X86_INS_FPATAN, "FPATAN" },
	{ X86_INS_FPREM, "FPREM" },
	{ X86_INS_FPREM1, "FPREM1" },
	{ X86_INS_FPTAN, "FPTAN" },
	{ X86_INS_FRNDINT, "FRNDINT" },
	{ X86_INS_FRSTOR, "FRSTOR" },
	{ X86_INS_FNSAVE, "FNSAVE" },
	{ X86_INS_FSCALE, "FSCALE" },
	{ X86_INS_FSINCOS, "FSINCOS" },
	{ X86_INS_FNSTENV, "FNSTENV" },
	{ X86_INS_FS, "FS" },
	{ X86_INS_FXAM, "FXAM" },
	{ X86_INS_FXRSTOR, "FXRSTOR" },
	{ X86_INS_FXRSTORQ, "FXRSTORQ" },
	{ X86_INS_FXSAVE, "FXSAVE" },
	{ X86_INS_FXSAVEQ, "FXSAVEQ" },
	{ X86_INS_FXTRACT, "FXTRACT" },
	{ X86_INS_FYL2X, "FYL2X" },
	{ X86_INS_FYL2XP1, "FYL2XP1" },
	{ X86_INS_MOVAPD, "MOVAPD" },
	{ X86_INS_MOVAPS, "MOVAPS" },
	{ X86_INS_ORPD, "ORPD" },
	{ X86_INS_ORPS, "ORPS" },
	{ X86_INS_VMOVAPD, "VMOVAPD" },
	{ X86_INS_VMOVAPS, "VMOVAPS" },
	{ X86_INS_XORPD, "XORPD" },
	{ X86_INS_XORPS, "XORPS" },
	{ X86_INS_GS, "GS" },
	{ X86_INS_HADDPD, "HADDPD" },
	{ X86_INS_HADDPS, "HADDPS" },
	{ X86_INS_HLT, "HLT" },
	{ X86_INS_HSUBPD, "HSUBPD" },
	{ X86_INS_HSUBPS, "HSUBPS" },
	{ X86_INS_IDIV, "IDIV" },
	{ X86_INS_FILD, "FILD" },
	{ X86_INS_IMUL, "IMUL" },
	{ X86_INS_INS, "INS" },
	{ X86_INS_IN, "IN" },
	{ X86_INS_INC, "INC" },
	{ X86_INS_INSERTPS, "INSERTPS" },
	{ X86_INS_INSERTQ, "INSERTQ" },
	{ X86_INS_INT, "INT" },
	{ X86_INS_INT3, "INT3" },
	{ X86_INS_INTO, "INTO" },
	{ X86_INS_INVD, "INVD" },
	{ X86_INS_INVEPT, "INVEPT" },
	{ X86_INS_INVLPG, "INVLPG" },
	{ X86_INS_INVLPGA, "INVLPGA" },
	{ X86_INS_INVPCID, "INVPCID" },
	{ X86_INS_INVVPID, "INVVPID" },
	{ X86_INS_IRET, "IRET" },
	{ X86_INS_IRETD, "IRETD" },
	{ X86_INS_IRETQ, "IRETQ" },
	{ X86_INS_FISTTP, "FISTTP" },
	{ X86_INS_FIST, "FIST" },
	{ X86_INS_FISTP, "FISTP" },
	{ X86_INS_UCOMISD, "UCOMISD" },
	{ X86_INS_UCOMISS, "UCOMISS" },
	{ X86_INS_VCMPSD, "VCMPSD" },
	{ X86_INS_VCMPSS, "VCMPSS" },
	{ X86_INS_VCOMISD, "VCOMISD" },
	{ X86_INS_VCOMISS, "VCOMISS" },
	{ X86_INS_VCVTSD2SS, "VCVTSD2SS" },
	{ X86_INS_VCVTSI2SD, "VCVTSI2SD" },
	{ X86_INS_VCVTSI2SS, "VCVTSI2SS" },
	{ X86_INS_VCVTSS2SD, "VCVTSS2SD" },
	{ X86_INS_VCVTTSD2SI, "VCVTTSD2SI" },
	{ X86_INS_VCVTTSS2SI, "VCVTTSS2SI" },
	{ X86_INS_VUCOMISD, "VUCOMISD" },
	{ X86_INS_VUCOMISS, "VUCOMISS" },
	{ X86_INS_JAE, "JAE" },
	{ X86_INS_JA, "JA" },
	{ X86_INS_JBE, "JBE" },
	{ X86_INS_JB, "JB" },
	{ X86_INS_JCXZ, "JCXZ" },
	{ X86_INS_JECXZ, "JECXZ" },
	{ X86_INS_JE, "JE" },
	{ X86_INS_JGE, "JGE" },
	{ X86_INS_JG, "JG" },
	{ X86_INS_JLE, "JLE" },
	{ X86_INS_JL, "JL" },
	{ X86_INS_JMP, "JMP" },
	{ X86_INS_JMPQ, "JMPQ" },
	{ X86_INS_JNE, "JNE" },
	{ X86_INS_JNO, "JNO" },
	{ X86_INS_JNP, "JNP" },
	{ X86_INS_JNS, "JNS" },
	{ X86_INS_JO, "JO" },
	{ X86_INS_JP, "JP" },
	{ X86_INS_JRCXZ, "JRCXZ" },
	{ X86_INS_JS, "JS" },
	{ X86_INS_KADDW, "KADDW" },
	{ X86_INS_KANDNW, "KANDNW" },
	{ X86_INS_KANDW, "KANDW" },
	{ X86_INS_KMOVW, "KMOVW" },
	{ X86_INS_KNOTW, "KNOTW" },
	{ X86_INS_KORTESTW, "KORTESTW" },
	{ X86_INS_KORW, "KORW" },
	{ X86_INS_KSHIFTLW, "KSHIFTLW" },
	{ X86_INS_KSHIFTRW, "KSHIFTRW" },
	{ X86_INS_KTESTW, "KTESTW" },
	{ X86_INS_KUNPCKBW, "KUNPCKBW" },
	{ X86_INS_KXNORW, "KXNORW" },
	{ X86_INS_KXORW, "KXORW" },
	{ X86_INS_LAHF, "LAHF" },
	{ X86_INS_LAR, "LAR" },
	{ X86_INS_LDDQU, "LDDQU" },
	{ X86_INS_LDMXCSR, "LDMXCSR" },
	{ X86_INS_LDS, "LDS" },
	{ X86_INS_FLDZ, "FLDZ" },
	{ X86_INS_FLD1, "FLD1" },
	{ X86_INS_FLD, "FLD" },
	{ X86_INS_LEA, "LEA" },
	{ X86_INS_LEAVE, "LEAVE" },
	{ X86_INS_LES, "LES" },
	{ X86_INS_LFENCE, "LFENCE" },
	{ X86_INS_LFS, "LFS" },
	{ X86_INS_LGDT, "LGDT" },
	{ X86_INS_LGS, "LGS" },
	{ X86_INS_LIDT, "LIDT" },
	{ X86_INS_LLDT, "LLDT" },
	{ X86_INS_LMSW, "LMSW" },
	{ X86_INS_OR, "OR" },
	{ X86_INS_LOCK, "LOCK" },
	{ X86_INS_SUB, "SUB" },
	{ X86_INS_XOR, "XOR" },
	{ X86_INS_LODSB, "LODSB" },
	{ X86_INS_LODSD, "LODSD" },
	{ X86_INS_LODSQ, "LODSQ" },
	{ X86_INS_LODSW, "LODSW" },
	{ X86_INS_LOOP, "LOOP" },
	{ X86_INS_LOOPE, "LOOPE" },
	{ X86_INS_LOOPNE, "LOOPNE" },
	{ X86_INS_RETF, "RETF" },
	{ X86_INS_LSL, "LSL" },
	{ X86_INS_LSS, "LSS" },
	{ X86_INS_LTR, "LTR" },
	{ X86_INS_XADD, "XADD" },
	{ X86_INS_LZCNT, "LZCNT" },
	{ X86_INS_MASKMOVDQU, "MASKMOVDQU" },
	{ X86_INS_MAXPD, "MAXPD" },
	{ X86_INS_MAXPS, "MAXPS" },
	{ X86_INS_MAXSD, "MAXSD" },
	{ X86_INS_MAXSS, "MAXSS" },
	{ X86_INS_MFENCE, "MFENCE" },
	{ X86_INS_MINPD, "MINPD" },
	{ X86_INS_MINPS, "MINPS" },
	{ X86_INS_MINSD, "MINSD" },
	{ X86_INS_MINSS, "MINSS" },
	{ X86_INS_CVTPD2PI, "CVTPD2PI" },
	{ X86_INS_CVTPI2PD, "CVTPI2PD" },
	{ X86_INS_CVTPI2PS, "CVTPI2PS" },
	{ X86_INS_CVTPS2PI, "CVTPS2PI" },
	{ X86_INS_CVTTPD2PI, "CVTTPD2PI" },
	{ X86_INS_CVTTPS2PI, "CVTTPS2PI" },
	{ X86_INS_EMMS, "EMMS" },
	{ X86_INS_MASKMOVQ, "MASKMOVQ" },
	{ X86_INS_MOVD, "MOVD" },
	{ X86_INS_MOVDQ2Q, "MOVDQ2Q" },
	{ X86_INS_MOVNTQ, "MOVNTQ" },
	{ X86_INS_MOVQ2DQ, "MOVQ2DQ" },
	{ X86_INS_MOVQ, "MOVQ" },
	{ X86_INS_PABSB, "PABSB" },
	{ X86_INS_PABSD, "PABSD" },
	{ X86_INS_PABSW, "PABSW" },
	{ X86_INS_PACKSSDW, "PACKSSDW" },
	{ X86_INS_PACKSSWB, "PACKSSWB" },
	{ X86_INS_PACKUSWB, "PACKUSWB" },
	{ X86_INS_PADDB, "PADDB" },
	{ X86_INS_PADDD, "PADDD" },
	{ X86_INS_PADDQ, "PADDQ" },
	{ X86_INS_PADDSB, "PADDSB" },
	{ X86_INS_PADDSW, "PADDSW" },
	{ X86_INS_PADDUSB, "PADDUSB" },
	{ X86_INS_PADDUSW, "PADDUSW" },
	{ X86_INS_PADDW, "PADDW" },
	{ X86_INS_PALIGNR, "PALIGNR" },
	{ X86_INS_PANDN, "PANDN" },
	{ X86_INS_PAND, "PAND" },
	{ X86_INS_PAVGB, "PAVGB" },
	{ X86_INS_PAVGW, "PAVGW" },
	{ X86_INS_PCMPEQB, "PCMPEQB" },
	{ X86_INS_PCMPEQD, "PCMPEQD" },
	{ X86_INS_PCMPEQW, "PCMPEQW" },
	{ X86_INS_PCMPGTB, "PCMPGTB" },
	{ X86_INS_PCMPGTD, "PCMPGTD" },
	{ X86_INS_PCMPGTW, "PCMPGTW" },
	{ X86_INS_PEXTRW, "PEXTRW" },
	{ X86_INS_PHADDSW, "PHADDSW" },
	{ X86_INS_PHADDW, "PHADDW" },
	{ X86_INS_PHADDD, "PHADDD" },
	{ X86_INS_PHSUBD, "PHSUBD" },
	{ X86_INS_PHSUBSW, "PHSUBSW" },
	{ X86_INS_PHSUBW, "PHSUBW" },
	{ X86_INS_PINSRW, "PINSRW" },
	{ X86_INS_PMADDUBSW, "PMADDUBSW" },
	{ X86_INS_PMADDWD, "PMADDWD" },
	{ X86_INS_PMAXSW, "PMAXSW" },
	{ X86_INS_PMAXUB, "PMAXUB" },
	{ X86_INS_PMINSW, "PMINSW" },
	{ X86_INS_PMINUB, "PMINUB" },
	{ X86_INS_PMOVMSKB, "PMOVMSKB" },
	{ X86_INS_PMULHRSW, "PMULHRSW" },
	{ X86_INS_PMULHUW, "PMULHUW" },
	{ X86_INS_PMULHW, "PMULHW" },
	{ X86_INS_PMULLW, "PMULLW" },
	{ X86_INS_PMULUDQ, "PMULUDQ" },
	{ X86_INS_POR, "POR" },
	{ X86_INS_PSADBW, "PSADBW" },
	{ X86_INS_PSHUFB, "PSHUFB" },
	{ X86_INS_PSHUFW, "PSHUFW" },
	{ X86_INS_PSIGNB, "PSIGNB" },
	{ X86_INS_PSIGND, "PSIGND" },
	{ X86_INS_PSIGNW, "PSIGNW" },
	{ X86_INS_PSLLD, "PSLLD" },
	{ X86_INS_PSLLQ, "PSLLQ" },
	{ X86_INS_PSLLW, "PSLLW" },
	{ X86_INS_PSRAD, "PSRAD" },
	{ X86_INS_PSRAW, "PSRAW" },
	{ X86_INS_PSRLD, "PSRLD" },
	{ X86_INS_PSRLQ, "PSRLQ" },
	{ X86_INS_PSRLW, "PSRLW" },
	{ X86_INS_PSUBB, "PSUBB" },
	{ X86_INS_PSUBD, "PSUBD" },
	{ X86_INS_PSUBQ, "PSUBQ" },
	{ X86_INS_PSUBSB, "PSUBSB" },
	{ X86_INS_PSUBSW, "PSUBSW" },
	{ X86_INS_PSUBUSB, "PSUBUSB" },
	{ X86_INS_PSUBUSW, "PSUBUSW" },
	{ X86_INS_PSUBW, "PSUBW" },
	{ X86_INS_PUNPCKHBW, "PUNPCKHBW" },
	{ X86_INS_PUNPCKHDQ, "PUNPCKHDQ" },
	{ X86_INS_PUNPCKHWD, "PUNPCKHWD" },
	{ X86_INS_PUNPCKLBW, "PUNPCKLBW" },
	{ X86_INS_PUNPCKLDQ, "PUNPCKLDQ" },
	{ X86_INS_PUNPCKLWD, "PUNPCKLWD" },
	{ X86_INS_PXOR, "PXOR" },
	{ X86_INS_MONITOR, "MONITOR" },
	{ X86_INS_MONTMUL, "MONTMUL" },
	{ X86_INS_MOV, "MOV" },
	{ X86_INS_MOVABS, "MOVABS" },
	{ X86_INS_MOVBE, "MOVBE" },
	{ X86_INS_MOVDDUP, "MOVDDUP" },
	{ X86_INS_MOVDQA, "MOVDQA" },
	{ X86_INS_MOVDQU, "MOVDQU" },
	{ X86_INS_MOVHLPS, "MOVHLPS" },
	{ X86_INS_MOVHPD, "MOVHPD" },
	{ X86_INS_MOVHPS, "MOVHPS" },
	{ X86_INS_MOVLHPS, "MOVLHPS" },
	{ X86_INS_MOVLPD, "MOVLPD" },
	{ X86_INS_MOVLPS, "MOVLPS" },
	{ X86_INS_MOVMSKPD, "MOVMSKPD" },
	{ X86_INS_MOVMSKPS, "MOVMSKPS" },
	{ X86_INS_MOVNTDQA, "MOVNTDQA" },
	{ X86_INS_MOVNTDQ, "MOVNTDQ" },
	{ X86_INS_MOVNTI, "MOVNTI" },
	{ X86_INS_MOVNTPD, "MOVNTPD" },
	{ X86_INS_MOVNTPS, "MOVNTPS" },
	{ X86_INS_MOVNTSD, "MOVNTSD" },
	{ X86_INS_MOVNTSS, "MOVNTSS" },
	{ X86_INS_MOVSB, "MOVSB" },
	{ X86_INS_MOVSD, "MOVSD" },
	{ X86_INS_MOVSHDUP, "MOVSHDUP" },
	{ X86_INS_MOVSLDUP, "MOVSLDUP" },
	{ X86_INS_MOVSQ, "MOVSQ" },
	{ X86_INS_MOVSS, "MOVSS" },
	{ X86_INS_MOVSW, "MOVSW" },
	{ X86_INS_MOVSX, "MOVSX" },
	{ X86_INS_MOVSXD, "MOVSXD" },
	{ X86_INS_MOVUPD, "MOVUPD" },
	{ X86_INS_MOVUPS, "MOVUPS" },
	{ X86_INS_MOVZX, "MOVZX" },
	{ X86_INS_MPSADBW, "MPSADBW" },
	{ X86_INS_MUL, "MUL" },
	{ X86_INS_MULPD, "MULPD" },
	{ X86_INS_MULPS, "MULPS" },
	{ X86_INS_MULSD, "MULSD" },
	{ X86_INS_MULSS, "MULSS" },
	{ X86_INS_MULX, "MULX" },
	{ X86_INS_FMUL, "FMUL" },
	{ X86_INS_FIMUL, "FIMUL" },
	{ X86_INS_FMULP, "FMULP" },
	{ X86_INS_MWAIT, "MWAIT" },
	{ X86_INS_NEG, "NEG" },
	{ X86_INS_NOP, "NOP" },
	{ X86_INS_NOT, "NOT" },
	{ X86_INS_OUT, "OUT" },
	{ X86_INS_OUTSB, "OUTSB" },
	{ X86_INS_OUTSD, "OUTSD" },
	{ X86_INS_OUTSW, "OUTSW" },
	{ X86_INS_PACKUSDW, "PACKUSDW" },
	{ X86_INS_PAUSE, "PAUSE" },
	{ X86_INS_PAVGUSB, "PAVGUSB" },
	{ X86_INS_PBLENDVB, "PBLENDVB" },
	{ X86_INS_PBLENDW, "PBLENDW" },
	{ X86_INS_PCLMULQDQ, "PCLMULQDQ" },
	{ X86_INS_PCMPEQQ, "PCMPEQQ" },
	{ X86_INS_PCMPESTRI, "PCMPESTRI" },
	{ X86_INS_PCMPESTRM, "PCMPESTRM" },
	{ X86_INS_PCMPGTQ, "PCMPGTQ" },
	{ X86_INS_PCMPISTRI, "PCMPISTRI" },
	{ X86_INS_PCMPISTRM, "PCMPISTRM" },
	{ X86_INS_PDEP, "PDEP" },
	{ X86_INS_PEXT, "PEXT" },
	{ X86_INS_PEXTRB, "PEXTRB" },
	{ X86_INS_PEXTRD, "PEXTRD" },
	{ X86_INS_PEXTRQ, "PEXTRQ" },
	{ X86_INS_PF2ID, "PF2ID" },
	{ X86_INS_PF2IW, "PF2IW" },
	{ X86_INS_PFACC, "PFACC" },
	{ X86_INS_PFADD, "PFADD" },
	{ X86_INS_PFCMPEQ, "PFCMPEQ" },
	{ X86_INS_PFCMPGE, "PFCMPGE" },
	{ X86_INS_PFCMPGT, "PFCMPGT" },
	{ X86_INS_PFMAX, "PFMAX" },
	{ X86_INS_PFMIN, "PFMIN" },
	{ X86_INS_PFMUL, "PFMUL" },
	{ X86_INS_PFNACC, "PFNACC" },
	{ X86_INS_PFPNACC, "PFPNACC" },
	{ X86_INS_PFRCPIT1, "PFRCPIT1" },
	{ X86_INS_PFRCPIT2, "PFRCPIT2" },
	{ X86_INS_PFRCP, "PFRCP" },
	{ X86_INS_PFRSQIT1, "PFRSQIT1" },
	{ X86_INS_PFRSQRT, "PFRSQRT" },
	{ X86_INS_PFSUBR, "PFSUBR" },
	{ X86_INS_PFSUB, "PFSUB" },
	{ X86_INS_PHMINPOSUW, "PHMINPOSUW" },
	{ X86_INS_PI2FD, "PI2FD" },
	{ X86_INS_PI2FW, "PI2FW" },
	{ X86_INS_PINSRB, "PINSRB" },
	{ X86_INS_PINSRD, "PINSRD" },
	{ X86_INS_PINSRQ, "PINSRQ" },
	{ X86_INS_PMAXSB, "PMAXSB" },
	{ X86_INS_PMAXSD, "PMAXSD" },
	{ X86_INS_PMAXUD, "PMAXUD" },
	{ X86_INS_PMAXUW, "PMAXUW" },
	{ X86_INS_PMINSB, "PMINSB" },
	{ X86_INS_PMINSD, "PMINSD" },
	{ X86_INS_PMINUD, "PMINUD" },
	{ X86_INS_PMINUW, "PMINUW" },
	{ X86_INS_PMOVSXBD, "PMOVSXBD" },
	{ X86_INS_PMOVSXBQ, "PMOVSXBQ" },
	{ X86_INS_PMOVSXBW, "PMOVSXBW" },
	{ X86_INS_PMOVSXDQ, "PMOVSXDQ" },
	{ X86_INS_PMOVSXWD, "PMOVSXWD" },
	{ X86_INS_PMOVSXWQ, "PMOVSXWQ" },
	{ X86_INS_PMOVZXBD, "PMOVZXBD" },
	{ X86_INS_PMOVZXBQ, "PMOVZXBQ" },
	{ X86_INS_PMOVZXBW, "PMOVZXBW" },
	{ X86_INS_PMOVZXDQ, "PMOVZXDQ" },
	{ X86_INS_PMOVZXWD, "PMOVZXWD" },
	{ X86_INS_PMOVZXWQ, "PMOVZXWQ" },
	{ X86_INS_PMULDQ, "PMULDQ" },
	{ X86_INS_PMULHRW, "PMULHRW" },
	{ X86_INS_PMULLD, "PMULLD" },
	{ X86_INS_POP, "POP" },
	{ X86_INS_POPA, "POPA" },
	{ X86_INS_POPCNT, "POPCNT" },
	{ X86_INS_POPF, "POPF" },
	{ X86_INS_POPFD, "POPFD" },
	{ X86_INS_POPFQ, "POPFQ" },
	{ X86_INS_PREFETCH, "PREFETCH" },
	{ X86_INS_PREFETCHNTA, "PREFETCHNTA" },
	{ X86_INS_PREFETCHT0, "PREFETCHT0" },
	{ X86_INS_PREFETCHT1, "PREFETCHT1" },
	{ X86_INS_PREFETCHT2, "PREFETCHT2" },
	{ X86_INS_PREFETCHW, "PREFETCHW" },
	{ X86_INS_PSHUFD, "PSHUFD" },
	{ X86_INS_PSHUFHW, "PSHUFHW" },
	{ X86_INS_PSHUFLW, "PSHUFLW" },
	{ X86_INS_PSLLDQ, "PSLLDQ" },
	{ X86_INS_PSRLDQ, "PSRLDQ" },
	{ X86_INS_PSWAPD, "PSWAPD" },
	{ X86_INS_PTEST, "PTEST" },
	{ X86_INS_PUNPCKHQDQ, "PUNPCKHQDQ" },
	{ X86_INS_PUNPCKLQDQ, "PUNPCKLQDQ" },
	{ X86_INS_PUSH, "PUSH" },
	{ X86_INS_PUSHA, "PUSHA" },
	{ X86_INS_PUSHF, "PUSHF" },
	{ X86_INS_PUSHFD, "PUSHFD" },
	{ X86_INS_PUSHFQ, "PUSHFQ" },
	{ X86_INS_RCL, "RCL" },
	{ X86_INS_RCPPS, "RCPPS" },
	{ X86_INS_RCPSS, "RCPSS" },
	{ X86_INS_RCR, "RCR" },
	{ X86_INS_RDFSBASE, "RDFSBASE" },
	{ X86_INS_RDGSBASE, "RDGSBASE" },
	{ X86_INS_RDMSR, "RDMSR" },
	{ X86_INS_RDPMC, "RDPMC" },
	{ X86_INS_RDRAND, "RDRAND" },
	{ X86_INS_RDSEED, "RDSEED" },
	{ X86_INS_RDTSC, "RDTSC" },
	{ X86_INS_RDTSCP, "RDTSCP" },
	{ X86_INS_REPNE, "REPNE" },
	{ X86_INS_REP, "REP" },
	{ X86_INS_REX64, "REX64" },
	{ X86_INS_ROL, "ROL" },
	{ X86_INS_ROR, "ROR" },
	{ X86_INS_RORX, "RORX" },
	{ X86_INS_ROUNDPD, "ROUNDPD" },
	{ X86_INS_ROUNDPS, "ROUNDPS" },
	{ X86_INS_ROUNDSD, "ROUNDSD" },
	{ X86_INS_ROUNDSS, "ROUNDSS" },
	{ X86_INS_RSM, "RSM" },
	{ X86_INS_RSQRTPS, "RSQRTPS" },
	{ X86_INS_RSQRTSS, "RSQRTSS" },
	{ X86_INS_SAHF, "SAHF" },
	{ X86_INS_SAR, "SAR" },
	{ X86_INS_SARX, "SARX" },
	{ X86_INS_SBB, "SBB" },
	{ X86_INS_SCASW, "SCASW" },
	{ X86_INS_SCASD, "SCASD" },
	{ X86_INS_SCASQ, "SCASQ" },
	{ X86_INS_SCASB, "SCASB" },
	{ X86_INS_SETAE, "SETAE" },
	{ X86_INS_SETA, "SETA" },
	{ X86_INS_SETBE, "SETBE" },
	{ X86_INS_SETB, "SETB" },
	{ X86_INS_SETE, "SETE" },
	{ X86_INS_SETGE, "SETGE" },
	{ X86_INS_SETG, "SETG" },
	{ X86_INS_SETLE, "SETLE" },
	{ X86_INS_SETL, "SETL" },
	{ X86_INS_SETNE, "SETNE" },
	{ X86_INS_SETNO, "SETNO" },
	{ X86_INS_SETNP, "SETNP" },
	{ X86_INS_SETNS, "SETNS" },
	{ X86_INS_SETO, "SETO" },
	{ X86_INS_SETP, "SETP" },
	{ X86_INS_SETS, "SETS" },
	{ X86_INS_SFENCE, "SFENCE" },
	{ X86_INS_SGDT, "SGDT" },
	{ X86_INS_SHA1MSG1, "SHA1MSG1" },
	{ X86_INS_SHA1MSG2, "SHA1MSG2" },
	{ X86_INS_SHA1NEXTE, "SHA1NEXTE" },
	{ X86_INS_SHA1RNDS4, "SHA1RNDS4" },
	{ X86_INS_SHA256MSG1, "SHA256MSG1" },
	{ X86_INS_SHA256MSG2, "SHA256MSG2" },
	{ X86_INS_SHA256RNDS2, "SHA256RNDS2" },
	{ X86_INS_SHL, "SHL" },
	{ X86_INS_SHLD, "SHLD" },
	{ X86_INS_SHLX, "SHLX" },
	{ X86_INS_SHR, "SHR" },
	{ X86_INS_SHRD, "SHRD" },
	{ X86_INS_SHRX, "SHRX" },
	{ X86_INS_SHUFPD, "SHUFPD" },
	{ X86_INS_SHUFPS, "SHUFPS" },
	{ X86_INS_SIDT, "SIDT" },
	{ X86_INS_FSIN, "FSIN" },
	{ X86_INS_SKINIT, "SKINIT" },
	{ X86_INS_SLDT, "SLDT" },
	{ X86_INS_SMSW, "SMSW" },
	{ X86_INS_SQRTPD, "SQRTPD" },
	{ X86_INS_SQRTPS, "SQRTPS" },
	{ X86_INS_SQRTSD, "SQRTSD" },
	{ X86_INS_SQRTSS, "SQRTSS" },
	{ X86_INS_FSQRT, "FSQRT" },
	{ X86_INS_SS, "SS" },
	{ X86_INS_STAC, "STAC" },
	{ X86_INS_STC, "STC" },
	{ X86_INS_STD, "STD" },
	{ X86_INS_STGI, "STGI" },
	{ X86_INS_STI, "STI" },
	{ X86_INS_STMXCSR, "STMXCSR" },
	{ X86_INS_STOSB, "STOSB" },
	{ X86_INS_STOSD, "STOSD" },
	{ X86_INS_STOSQ, "STOSQ" },
	{ X86_INS_STOSW, "STOSW" },
	{ X86_INS_STR, "STR" },
	{ X86_INS_FST, "FST" },
	{ X86_INS_FSTP, "FSTP" },
	{ X86_INS_SUBPD, "SUBPD" },
	{ X86_INS_SUBPS, "SUBPS" },
	{ X86_INS_FSUBR, "FSUBR" },
	{ X86_INS_FISUBR, "FISUBR" },
	{ X86_INS_FSUBRP, "FSUBRP" },
	{ X86_INS_SUBSD, "SUBSD" },
	{ X86_INS_SUBSS, "SUBSS" },
	{ X86_INS_FSUB, "FSUB" },
	{ X86_INS_FISUB, "FISUB" },
	{ X86_INS_FSUBP, "FSUBP" },
	{ X86_INS_SWAPGS, "SWAPGS" },
	{ X86_INS_SYSCALL, "SYSCALL" },
	{ X86_INS_SYSENTER, "SYSENTER" },
	{ X86_INS_SYSEXIT, "SYSEXIT" },
	{ X86_INS_SYSRET, "SYSRET" },
	{ X86_INS_TEST, "TEST" },
	{ X86_INS_UD2, "UD2" },
	{ X86_INS_FTST, "FTST" },
	{ X86_INS_TZCNT, "TZCNT" },
	{ X86_INS_FUCOMPI, "FUCOMPI" },
	{ X86_INS_FUCOMI, "FUCOMI" },
	{ X86_INS_FUCOMPP, "FUCOMPP" },
	{ X86_INS_FUCOMP, "FUCOMP" },
	{ X86_INS_FUCOM, "FUCOM" },
	{ X86_INS_UD2B, "UD2B" },
	{ X86_INS_UNPCKHPD, "UNPCKHPD" },
	{ X86_INS_UNPCKHPS, "UNPCKHPS" },
	{ X86_INS_UNPCKLPD, "UNPCKLPD" },
	{ X86_INS_UNPCKLPS, "UNPCKLPS" },
	{ X86_INS_VADDPD, "VADDPD" },
	{ X86_INS_VADDPS, "VADDPS" },
	{ X86_INS_VADDSD, "VADDSD" },
	{ X86_INS_VADDSS, "VADDSS" },
	{ X86_INS_VADDSUBPD, "VADDSUBPD" },
	{ X86_INS_VADDSUBPS, "VADDSUBPS" },
	{ X86_INS_VAESDECLAST, "VAESDECLAST" },
	{ X86_INS_VAESDEC, "VAESDEC" },
	{ X86_INS_VAESENCLAST, "VAESENCLAST" },
	{ X86_INS_VAESENC, "VAESENC" },
	{ X86_INS_VAESIMC, "VAESIMC" },
	{ X86_INS_VAESKEYGENASSIST, "VAESKEYGENASSIST" },
	{ X86_INS_VALIGND, "VALIGND" },
	{ X86_INS_VALIGNQ, "VALIGNQ" },
	{ X86_INS_VANDNPD, "VANDNPD" },
	{ X86_INS_VANDNPS, "VANDNPS" },
	{ X86_INS_VANDPD, "VANDPD" },
	{ X86_INS_VANDPS, "VANDPS" },
	{ X86_INS_VBLENDMPD, "VBLENDMPD" },
	{ X86_INS_VBLENDMPS, "VBLENDMPS" },
	{ X86_INS_VBLENDPD, "VBLENDPD" },
	{ X86_INS_VBLENDPS, "VBLENDPS" },
	{ X86_INS_VBLENDVPD, "VBLENDVPD" },
	{ X86_INS_VBLENDVPS, "VBLENDVPS" },
	{ X86_INS_VBROADCASTF128, "VBROADCASTF128" },
	{ X86_INS_VBROADCASTI128, "VBROADCASTI128" },
	{ X86_INS_VBROADCASTSD, "VBROADCASTSD" },
	{ X86_INS_VBROADCASTSS, "VBROADCASTSS" },
	{ X86_INS_VCMPPD, "VCMPPD" },
	{ X86_INS_VVCMPPD, "VVCMPPD" },
	{ X86_INS_VCMPPS, "VCMPPS" },
	{ X86_INS_VVCMPPS, "VVCMPPS" },
	{ X86_INS_VCVTDQ2PD, "VCVTDQ2PD" },
	{ X86_INS_VCVTDQ2PS, "VCVTDQ2PS" },
	{ X86_INS_VCVTPD2DQX, "VCVTPD2DQX" },
	{ X86_INS_VCVTPD2DQ, "VCVTPD2DQ" },
	{ X86_INS_VCVTPD2PSX, "VCVTPD2PSX" },
	{ X86_INS_VCVTPD2PS, "VCVTPD2PS" },
	{ X86_INS_VCVTPH2PS, "VCVTPH2PS" },
	{ X86_INS_VCVTPS2DQ, "VCVTPS2DQ" },
	{ X86_INS_VCVTPS2PD, "VCVTPS2PD" },
	{ X86_INS_VCVTPS2PH, "VCVTPS2PH" },
	{ X86_INS_VCVTSD2SI, "VCVTSD2SI" },
	{ X86_INS_VCVTSS2SI, "VCVTSS2SI" },
	{ X86_INS_VCVTTPD2DQX, "VCVTTPD2DQX" },
	{ X86_INS_VCVTTPD2DQ, "VCVTTPD2DQ" },
	{ X86_INS_VCVTTPD2UDQ, "VCVTTPD2UDQ" },
	{ X86_INS_VCVTTPS2DQ, "VCVTTPS2DQ" },
	{ X86_INS_VCVTTPS2UDQ, "VCVTTPS2UDQ" },
	{ X86_INS_VCVTUDQ2PD, "VCVTUDQ2PD" },
	{ X86_INS_VCVTUDQ2PS, "VCVTUDQ2PS" },
	{ X86_INS_VDIVPD, "VDIVPD" },
	{ X86_INS_VDIVPS, "VDIVPS" },
	{ X86_INS_VDIVSD, "VDIVSD" },
	{ X86_INS_VDIVSS, "VDIVSS" },
	{ X86_INS_VDPPD, "VDPPD" },
	{ X86_INS_VDPPS, "VDPPS" },
	{ X86_INS_VERR, "VERR" },
	{ X86_INS_VERW, "VERW" },
	{ X86_INS_VEXTRACTF128, "VEXTRACTF128" },
	{ X86_INS_VEXTRACTF32X4, "VEXTRACTF32X4" },
	{ X86_INS_VEXTRACTF64X4, "VEXTRACTF64X4" },
	{ X86_INS_VEXTRACTI128, "VEXTRACTI128" },
	{ X86_INS_VEXTRACTI32X4, "VEXTRACTI32X4" },
	{ X86_INS_VEXTRACTI64X4, "VEXTRACTI64X4" },
	{ X86_INS_VEXTRACTPS, "VEXTRACTPS" },
	{ X86_INS_VFMADD132PD, "VFMADD132PD" },
	{ X86_INS_VFMADD132PS, "VFMADD132PS" },
	{ X86_INS_VFMADD213PD, "VFMADD213PD" },
	{ X86_INS_VFMADD213PS, "VFMADD213PS" },
	{ X86_INS_VFMADDPD, "VFMADDPD" },
	{ X86_INS_VFMADD231PD, "VFMADD231PD" },
	{ X86_INS_VFMADDPS, "VFMADDPS" },
	{ X86_INS_VFMADD231PS, "VFMADD231PS" },
	{ X86_INS_VFMADDSD, "VFMADDSD" },
	{ X86_INS_VFMADD213SD, "VFMADD213SD" },
	{ X86_INS_VFMADD132SD, "VFMADD132SD" },
	{ X86_INS_VFMADD231SD, "VFMADD231SD" },
	{ X86_INS_VFMADDSS, "VFMADDSS" },
	{ X86_INS_VFMADD213SS, "VFMADD213SS" },
	{ X86_INS_VFMADD132SS, "VFMADD132SS" },
	{ X86_INS_VFMADD231SS, "VFMADD231SS" },
	{ X86_INS_VFMADDSUB132PD, "VFMADDSUB132PD" },
	{ X86_INS_VFMADDSUB132PS, "VFMADDSUB132PS" },
	{ X86_INS_VFMADDSUB213PD, "VFMADDSUB213PD" },
	{ X86_INS_VFMADDSUB213PS, "VFMADDSUB213PS" },
	{ X86_INS_VFMADDSUBPD, "VFMADDSUBPD" },
	{ X86_INS_VFMADDSUB231PD, "VFMADDSUB231PD" },
	{ X86_INS_VFMADDSUBPS, "VFMADDSUBPS" },
	{ X86_INS_VFMADDSUB231PS, "VFMADDSUB231PS" },
	{ X86_INS_VFMSUB132PD, "VFMSUB132PD" },
	{ X86_INS_VFMSUB132PS, "VFMSUB132PS" },
	{ X86_INS_VFMSUB213PD, "VFMSUB213PD" },
	{ X86_INS_VFMSUB213PS, "VFMSUB213PS" },
	{ X86_INS_VFMSUBADD132PD, "VFMSUBADD132PD" },
	{ X86_INS_VFMSUBADD132PS, "VFMSUBADD132PS" },
	{ X86_INS_VFMSUBADD213PD, "VFMSUBADD213PD" },
	{ X86_INS_VFMSUBADD213PS, "VFMSUBADD213PS" },
	{ X86_INS_VFMSUBADDPD, "VFMSUBADDPD" },
	{ X86_INS_VFMSUBADD231PD, "VFMSUBADD231PD" },
	{ X86_INS_VFMSUBADDPS, "VFMSUBADDPS" },
	{ X86_INS_VFMSUBADD231PS, "VFMSUBADD231PS" },
	{ X86_INS_VFMSUBPD, "VFMSUBPD" },
	{ X86_INS_VFMSUB231PD, "VFMSUB231PD" },
	{ X86_INS_VFMSUBPS, "VFMSUBPS" },
	{ X86_INS_VFMSUB231PS, "VFMSUB231PS" },
	{ X86_INS_VFMSUBSD, "VFMSUBSD" },
	{ X86_INS_VFMSUB213SD, "VFMSUB213SD" },
	{ X86_INS_VFMSUB132SD, "VFMSUB132SD" },
	{ X86_INS_VFMSUB231SD, "VFMSUB231SD" },
	{ X86_INS_VFMSUBSS, "VFMSUBSS" },
	{ X86_INS_VFMSUB213SS, "VFMSUB213SS" },
	{ X86_INS_VFMSUB132SS, "VFMSUB132SS" },
	{ X86_INS_VFMSUB231SS, "VFMSUB231SS" },
	{ X86_INS_VFNMADD132PD, "VFNMADD132PD" },
	{ X86_INS_VFNMADD132PS, "VFNMADD132PS" },
	{ X86_INS_VFNMADD213PD, "VFNMADD213PD" },
	{ X86_INS_VFNMADD213PS, "VFNMADD213PS" },
	{ X86_INS_VFNMADDPD, "VFNMADDPD" },
	{ X86_INS_VFNMADD231PD, "VFNMADD231PD" },
	{ X86_INS_VFNMADDPS, "VFNMADDPS" },
	{ X86_INS_VFNMADD231PS, "VFNMADD231PS" },
	{ X86_INS_VFNMADDSD, "VFNMADDSD" },
	{ X86_INS_VFNMADD213SD, "VFNMADD213SD" },
	{ X86_INS_VFNMADD132SD, "VFNMADD132SD" },
	{ X86_INS_VFNMADD231SD, "VFNMADD231SD" },
	{ X86_INS_VFNMADDSS, "VFNMADDSS" },
	{ X86_INS_VFNMADD213SS, "VFNMADD213SS" },
	{ X86_INS_VFNMADD132SS, "VFNMADD132SS" },
	{ X86_INS_VFNMADD231SS, "VFNMADD231SS" },
	{ X86_INS_VFNMSUB132PD, "VFNMSUB132PD" },
	{ X86_INS_VFNMSUB132PS, "VFNMSUB132PS" },
	{ X86_INS_VFNMSUB213PD, "VFNMSUB213PD" },
	{ X86_INS_VFNMSUB213PS, "VFNMSUB213PS" },
	{ X86_INS_VFNMSUBPD, "VFNMSUBPD" },
	{ X86_INS_VFNMSUB231PD, "VFNMSUB231PD" },
	{ X86_INS_VFNMSUBPS, "VFNMSUBPS" },
	{ X86_INS_VFNMSUB231PS, "VFNMSUB231PS" },
	{ X86_INS_VFNMSUBSD, "VFNMSUBSD" },
	{ X86_INS_VFNMSUB213SD, "VFNMSUB213SD" },
	{ X86_INS_VFNMSUB132SD, "VFNMSUB132SD" },
	{ X86_INS_VFNMSUB231SD, "VFNMSUB231SD" },
	{ X86_INS_VFNMSUBSS, "VFNMSUBSS" },
	{ X86_INS_VFNMSUB213SS, "VFNMSUB213SS" },
	{ X86_INS_VFNMSUB132SS, "VFNMSUB132SS" },
	{ X86_INS_VFNMSUB231SS, "VFNMSUB231SS" },
	{ X86_INS_VFRCZPD, "VFRCZPD" },
	{ X86_INS_VFRCZPS, "VFRCZPS" },
	{ X86_INS_VFRCZSD, "VFRCZSD" },
	{ X86_INS_VFRCZSS, "VFRCZSS" },
	{ X86_INS_VORPD, "VORPD" },
	{ X86_INS_VORPS, "VORPS" },
	{ X86_INS_VXORPD, "VXORPD" },
	{ X86_INS_VXORPS, "VXORPS" },
	{ X86_INS_VGATHERDPD, "VGATHERDPD" },
	{ X86_INS_VGATHERDPS, "VGATHERDPS" },
	{ X86_INS_VGATHERQPD, "VGATHERQPD" },
	{ X86_INS_VGATHERQPS, "VGATHERQPS" },
	{ X86_INS_VHADDPD, "VHADDPD" },
	{ X86_INS_VHADDPS, "VHADDPS" },
	{ X86_INS_VHSUBPD, "VHSUBPD" },
	{ X86_INS_VHSUBPS, "VHSUBPS" },
	{ X86_INS_VINSERTF128, "VINSERTF128" },
	{ X86_INS_VINSERTF32X4, "VINSERTF32X4" },
	{ X86_INS_VINSERTF64X4, "VINSERTF64X4" },
	{ X86_INS_VINSERTI128, "VINSERTI128" },
	{ X86_INS_VINSERTI32X4, "VINSERTI32X4" },
	{ X86_INS_VINSERTI64X4, "VINSERTI64X4" },
	{ X86_INS_VINSERTPS, "VINSERTPS" },
	{ X86_INS_VLDDQU, "VLDDQU" },
	{ X86_INS_VLDMXCSR, "VLDMXCSR" },
	{ X86_INS_VMASKMOVDQU, "VMASKMOVDQU" },
	{ X86_INS_VMASKMOVPD, "VMASKMOVPD" },
	{ X86_INS_VMASKMOVPS, "VMASKMOVPS" },
	{ X86_INS_VMAXPD, "VMAXPD" },
	{ X86_INS_VMAXPS, "VMAXPS" },
	{ X86_INS_VMAXSD, "VMAXSD" },
	{ X86_INS_VMAXSS, "VMAXSS" },
	{ X86_INS_VMCALL, "VMCALL" },
	{ X86_INS_VMCLEAR, "VMCLEAR" },
	{ X86_INS_VMFUNC, "VMFUNC" },
	{ X86_INS_VMINPD, "VMINPD" },
	{ X86_INS_VMINPS, "VMINPS" },
	{ X86_INS_VMINSD, "VMINSD" },
	{ X86_INS_VMINSS, "VMINSS" },
	{ X86_INS_VMLAUNCH, "VMLAUNCH" },
	{ X86_INS_VMLOAD, "VMLOAD" },
	{ X86_INS_VMMCALL, "VMMCALL" },
	{ X86_INS_VMOVQ, "VMOVQ" },
	{ X86_INS_VVMOVQ, "VVMOVQ" },
	{ X86_INS_VMOVDDUP, "VMOVDDUP" },
	{ X86_INS_VMOVD, "VMOVD" },
	{ X86_INS_VMOVDQA32, "VMOVDQA32" },
	{ X86_INS_VMOVDQA64, "VMOVDQA64" },
	{ X86_INS_VMOVDQA, "VMOVDQA" },
	{ X86_INS_VMOVDQU32, "VMOVDQU32" },
	{ X86_INS_VMOVDQU64, "VMOVDQU64" },
	{ X86_INS_VMOVDQU, "VMOVDQU" },
	{ X86_INS_VMOVHLPS, "VMOVHLPS" },
	{ X86_INS_VMOVHPD, "VMOVHPD" },
	{ X86_INS_VMOVHPS, "VMOVHPS" },
	{ X86_INS_VMOVLHPS, "VMOVLHPS" },
	{ X86_INS_VMOVLPD, "VMOVLPD" },
	{ X86_INS_VMOVLPS, "VMOVLPS" },
	{ X86_INS_VMOVMSKPD, "VMOVMSKPD" },
	{ X86_INS_VMOVMSKPS, "VMOVMSKPS" },
	{ X86_INS_VMOVNTDQA, "VMOVNTDQA" },
	{ X86_INS_VMOVNTDQ, "VMOVNTDQ" },
	{ X86_INS_VMOVNTPD, "VMOVNTPD" },
	{ X86_INS_VMOVNTPS, "VMOVNTPS" },
	{ X86_INS_VMOVSD, "VMOVSD" },
	{ X86_INS_VMOVSHDUP, "VMOVSHDUP" },
	{ X86_INS_VMOVSLDUP, "VMOVSLDUP" },
	{ X86_INS_VMOVSS, "VMOVSS" },
	{ X86_INS_VMOVUPD, "VMOVUPD" },
	{ X86_INS_VMOVUPS, "VMOVUPS" },
	{ X86_INS_VMPSADBW, "VMPSADBW" },
	{ X86_INS_VMPTRLD, "VMPTRLD" },
	{ X86_INS_VMPTRST, "VMPTRST" },
	{ X86_INS_VMREAD, "VMREAD" },
	{ X86_INS_VMRESUME, "VMRESUME" },
	{ X86_INS_VMRUN, "VMRUN" },
	{ X86_INS_VMSAVE, "VMSAVE" },
	{ X86_INS_VMULPD, "VMULPD" },
	{ X86_INS_VMULPS, "VMULPS" },
	{ X86_INS_VMULSD, "VMULSD" },
	{ X86_INS_VMULSS, "VMULSS" },
	{ X86_INS_VMWRITE, "VMWRITE" },
	{ X86_INS_VMXOFF, "VMXOFF" },
	{ X86_INS_VMXON, "VMXON" },
	{ X86_INS_VPABSB, "VPABSB" },
	{ X86_INS_VPABSD, "VPABSD" },
	{ X86_INS_VPABSQ, "VPABSQ" },
	{ X86_INS_VPABSW, "VPABSW" },
	{ X86_INS_VPACKSSDW, "VPACKSSDW" },
	{ X86_INS_VPACKSSWB, "VPACKSSWB" },
	{ X86_INS_VPACKUSDW, "VPACKUSDW" },
	{ X86_INS_VPACKUSWB, "VPACKUSWB" },
	{ X86_INS_VPADDB, "VPADDB" },
	{ X86_INS_VPADDD, "VPADDD" },
	{ X86_INS_VPADDQ, "VPADDQ" },
	{ X86_INS_VPADDSB, "VPADDSB" },
	{ X86_INS_VPADDSW, "VPADDSW" },
	{ X86_INS_VPADDUSB, "VPADDUSB" },
	{ X86_INS_VPADDUSW, "VPADDUSW" },
	{ X86_INS_VPADDW, "VPADDW" },
	{ X86_INS_VPALIGNR, "VPALIGNR" },
	{ X86_INS_VPANDD, "VPANDD" },
	{ X86_INS_VPANDND, "VPANDND" },
	{ X86_INS_VPANDNQ, "VPANDNQ" },
	{ X86_INS_VPANDN, "VPANDN" },
	{ X86_INS_VPANDQ, "VPANDQ" },
	{ X86_INS_VPAND, "VPAND" },
	{ X86_INS_VPAVGB, "VPAVGB" },
	{ X86_INS_VPAVGW, "VPAVGW" },
	{ X86_INS_VPBLENDD, "VPBLENDD" },
	{ X86_INS_VPBLENDMD, "VPBLENDMD" },
	{ X86_INS_VPBLENDMQ, "VPBLENDMQ" },
	{ X86_INS_VPBLENDVB, "VPBLENDVB" },
	{ X86_INS_VPBLENDW, "VPBLENDW" },
	{ X86_INS_VPBROADCASTB, "VPBROADCASTB" },
	{ X86_INS_VPBROADCASTD, "VPBROADCASTD" },
	{ X86_INS_VPBROADCASTMB2Q, "VPBROADCASTMB2Q" },
	{ X86_INS_VPBROADCASTMW2D, "VPBROADCASTMW2D" },
	{ X86_INS_VPBROADCASTQ, "VPBROADCASTQ" },
	{ X86_INS_VPBROADCASTW, "VPBROADCASTW" },
	{ X86_INS_VPCLMULQDQ, "VPCLMULQDQ" },
	{ X86_INS_VPCMOV, "VPCMOV" },
	{ X86_INS_VPCMPD, "VPCMPD" },
	{ X86_INS_VPCMPEQB, "VPCMPEQB" },
	{ X86_INS_VPCMPEQD, "VPCMPEQD" },
	{ X86_INS_VPCMPEQQ, "VPCMPEQQ" },
	{ X86_INS_VPCMPEQW, "VPCMPEQW" },
	{ X86_INS_VPCMPESTRI, "VPCMPESTRI" },
	{ X86_INS_VPCMPESTRM, "VPCMPESTRM" },
	{ X86_INS_VPCMPGTB, "VPCMPGTB" },
	{ X86_INS_VPCMPGTD, "VPCMPGTD" },
	{ X86_INS_VPCMPGTQ, "VPCMPGTQ" },
	{ X86_INS_VPCMPGTW, "VPCMPGTW" },
	{ X86_INS_VPCMPISTRI, "VPCMPISTRI" },
	{ X86_INS_VPCMPISTRM, "VPCMPISTRM" },
	{ X86_INS_VPCMPQ, "VPCMPQ" },
	{ X86_INS_VPCMPUD, "VPCMPUD" },
	{ X86_INS_VPCMPUQ, "VPCMPUQ" },
	{ X86_INS_VPCOMB, "VPCOMB" },
	{ X86_INS_VPCOMD, "VPCOMD" },
	{ X86_INS_VPCOMQ, "VPCOMQ" },
	{ X86_INS_VPCOMUB, "VPCOMUB" },
	{ X86_INS_VPCOMUD, "VPCOMUD" },
	{ X86_INS_VPCOMUQ, "VPCOMUQ" },
	{ X86_INS_VPCOMUW, "VPCOMUW" },
	{ X86_INS_VPCOMW, "VPCOMW" },
	{ X86_INS_VPERM2F128, "VPERM2F128" },
	{ X86_INS_VPERM2I128, "VPERM2I128" },
	{ X86_INS_VPERMD, "VPERMD" },
	{ X86_INS_VPERMI2D, "VPERMI2D" },
	{ X86_INS_VPERMI2PD, "VPERMI2PD" },
	{ X86_INS_VPERMI2PS, "VPERMI2PS" },
	{ X86_INS_VPERMI2Q, "VPERMI2Q" },
	{ X86_INS_VPERMIL2PD, "VPERMIL2PD" },
	{ X86_INS_VPERMIL2PS, "VPERMIL2PS" },
	{ X86_INS_VPERMILPD, "VPERMILPD" },
	{ X86_INS_VPERMILPS, "VPERMILPS" },
	{ X86_INS_VPERMPD, "VPERMPD" },
	{ X86_INS_VPERMPS, "VPERMPS" },
	{ X86_INS_VPERMQ, "VPERMQ" },
	{ X86_INS_VPEXTRB, "VPEXTRB" },
	{ X86_INS_VPEXTRD, "VPEXTRD" },
	{ X86_INS_VPEXTRQ, "VPEXTRQ" },
	{ X86_INS_VPEXTRW, "VPEXTRW" },
	{ X86_INS_VPGATHERDD, "VPGATHERDD" },
	{ X86_INS_VPGATHERDQ, "VPGATHERDQ" },
	{ X86_INS_VPGATHERQD, "VPGATHERQD" },
	{ X86_INS_VPGATHERQQ, "VPGATHERQQ" },
	{ X86_INS_VPHADDBD, "VPHADDBD" },
	{ X86_INS_VPHADDBQ, "VPHADDBQ" },
	{ X86_INS_VPHADDBW, "VPHADDBW" },
	{ X86_INS_VPHADDDQ, "VPHADDDQ" },
	{ X86_INS_VPHADDD, "VPHADDD" },
	{ X86_INS_VPHADDSW, "VPHADDSW" },
	{ X86_INS_VPHADDUBD, "VPHADDUBD" },
	{ X86_INS_VPHADDUBQ, "VPHADDUBQ" },
	{ X86_INS_VPHADDUBW, "VPHADDUBW" },
	{ X86_INS_VPHADDUDQ, "VPHADDUDQ" },
	{ X86_INS_VPHADDUWD, "VPHADDUWD" },
	{ X86_INS_VPHADDUWQ, "VPHADDUWQ" },
	{ X86_INS_VPHADDWD, "VPHADDWD" },
	{ X86_INS_VPHADDWQ, "VPHADDWQ" },
	{ X86_INS_VPHADDW, "VPHADDW" },
	{ X86_INS_VPHMINPOSUW, "VPHMINPOSUW" },
	{ X86_INS_VPHSUBBW, "VPHSUBBW" },
	{ X86_INS_VPHSUBDQ, "VPHSUBDQ" },
	{ X86_INS_VPHSUBD, "VPHSUBD" },
	{ X86_INS_VPHSUBSW, "VPHSUBSW" },
	{ X86_INS_VPHSUBWD, "VPHSUBWD" },
	{ X86_INS_VPHSUBW, "VPHSUBW" },
	{ X86_INS_VPINSRB, "VPINSRB" },
	{ X86_INS_VPINSRD, "VPINSRD" },
	{ X86_INS_VPINSRQ, "VPINSRQ" },
	{ X86_INS_VPINSRW, "VPINSRW" },
	{ X86_INS_VPMACSDD, "VPMACSDD" },
	{ X86_INS_VPMACSDQH, "VPMACSDQH" },
	{ X86_INS_VPMACSDQL, "VPMACSDQL" },
	{ X86_INS_VPMACSSDD, "VPMACSSDD" },
	{ X86_INS_VPMACSSDQH, "VPMACSSDQH" },
	{ X86_INS_VPMACSSDQL, "VPMACSSDQL" },
	{ X86_INS_VPMACSSWD, "VPMACSSWD" },
	{ X86_INS_VPMACSSWW, "VPMACSSWW" },
	{ X86_INS_VPMACSWD, "VPMACSWD" },
	{ X86_INS_VPMACSWW, "VPMACSWW" },
	{ X86_INS_VPMADCSSWD, "VPMADCSSWD" },
	{ X86_INS_VPMADCSWD, "VPMADCSWD" },
	{ X86_INS_VPMADDUBSW, "VPMADDUBSW" },
	{ X86_INS_VPMADDWD, "VPMADDWD" },
	{ X86_INS_VPMASKMOVD, "VPMASKMOVD" },
	{ X86_INS_VPMASKMOVQ, "VPMASKMOVQ" },
	{ X86_INS_VPMAXSB, "VPMAXSB" },
	{ X86_INS_VPMAXSD, "VPMAXSD" },
	{ X86_INS_VPMAXSW, "VPMAXSW" },
	{ X86_INS_VPMAXUB, "VPMAXUB" },
	{ X86_INS_VPMAXUD, "VPMAXUD" },
	{ X86_INS_VPMAXUW, "VPMAXUW" },
	{ X86_INS_VPMINSB, "VPMINSB" },
	{ X86_INS_VPMINSD, "VPMINSD" },
	{ X86_INS_VPMINSW, "VPMINSW" },
	{ X86_INS_VPMINUB, "VPMINUB" },
	{ X86_INS_VPMINUD, "VPMINUD" },
	{ X86_INS_VPMINUW, "VPMINUW" },
	{ X86_INS_VPMOVDB, "VPMOVDB" },
	{ X86_INS_VPMOVDW, "VPMOVDW" },
	{ X86_INS_VPMOVMSKB, "VPMOVMSKB" },
	{ X86_INS_VPMOVQB, "VPMOVQB" },
	{ X86_INS_VPMOVQD, "VPMOVQD" },
	{ X86_INS_VPMOVQW, "VPMOVQW" },
	{ X86_INS_VPMOVSDB, "VPMOVSDB" },
	{ X86_INS_VPMOVSDW, "VPMOVSDW" },
	{ X86_INS_VPMOVSQB, "VPMOVSQB" },
	{ X86_INS_VPMOVSQD, "VPMOVSQD" },
	{ X86_INS_VPMOVSQW, "VPMOVSQW" },
	{ X86_INS_VPMOVSXBD, "VPMOVSXBD" },
	{ X86_INS_VPMOVSXBQ, "VPMOVSXBQ" },
	{ X86_INS_VPMOVSXBW, "VPMOVSXBW" },
	{ X86_INS_VPMOVSXDQ, "VPMOVSXDQ" },
	{ X86_INS_VPMOVSXWD, "VPMOVSXWD" },
	{ X86_INS_VPMOVSXWQ, "VPMOVSXWQ" },
	{ X86_INS_VPMOVUSDB, "VPMOVUSDB" },
	{ X86_INS_VPMOVUSDW, "VPMOVUSDW" },
	{ X86_INS_VPMOVUSQB, "VPMOVUSQB" },
	{ X86_INS_VPMOVUSQD, "VPMOVUSQD" },
	{ X86_INS_VPMOVUSQW, "VPMOVUSQW" },
	{ X86_INS_VPMOVZXBD, "VPMOVZXBD" },
	{ X86_INS_VPMOVZXBQ, "VPMOVZXBQ" },
	{ X86_INS_VPMOVZXBW, "VPMOVZXBW" },
	{ X86_INS_VPMOVZXDQ, "VPMOVZXDQ" },
	{ X86_INS_VPMOVZXWD, "VPMOVZXWD" },
	{ X86_INS_VPMOVZXWQ, "VPMOVZXWQ" },
	{ X86_INS_VPMULDQ, "VPMULDQ" },
	{ X86_INS_VPMULHRSW, "VPMULHRSW" },
	{ X86_INS_VPMULHUW, "VPMULHUW" },
	{ X86_INS_VPMULHW, "VPMULHW" },
	{ X86_INS_VPMULLD, "VPMULLD" },
	{ X86_INS_VPMULLW, "VPMULLW" },
	{ X86_INS_VPMULUDQ, "VPMULUDQ" },
	{ X86_INS_VPORD, "VPORD" },
	{ X86_INS_VPORQ, "VPORQ" },
	{ X86_INS_VPOR, "VPOR" },
	{ X86_INS_VPPERM, "VPPERM" },
	{ X86_INS_VPROTB, "VPROTB" },
	{ X86_INS_VPROTD, "VPROTD" },
	{ X86_INS_VPROTQ, "VPROTQ" },
	{ X86_INS_VPROTW, "VPROTW" },
	{ X86_INS_VPSADBW, "VPSADBW" },
	{ X86_INS_VPSCATTERDD, "VPSCATTERDD" },
	{ X86_INS_VPSCATTERDQ, "VPSCATTERDQ" },
	{ X86_INS_VPSCATTERQD, "VPSCATTERQD" },
	{ X86_INS_VPSCATTERQQ, "VPSCATTERQQ" },
	{ X86_INS_VPSHAB, "VPSHAB" },
	{ X86_INS_VPSHAD, "VPSHAD" },
	{ X86_INS_VPSHAQ, "VPSHAQ" },
	{ X86_INS_VPSHAW, "VPSHAW" },
	{ X86_INS_VPSHLB, "VPSHLB" },
	{ X86_INS_VPSHLD, "VPSHLD" },
	{ X86_INS_VPSHLQ, "VPSHLQ" },
	{ X86_INS_VPSHLW, "VPSHLW" },
	{ X86_INS_VPSHUFB, "VPSHUFB" },
	{ X86_INS_VPSHUFD, "VPSHUFD" },
	{ X86_INS_VPSHUFHW, "VPSHUFHW" },
	{ X86_INS_VPSHUFLW, "VPSHUFLW" },
	{ X86_INS_VPSIGNB, "VPSIGNB" },
	{ X86_INS_VPSIGND, "VPSIGND" },
	{ X86_INS_VPSIGNW, "VPSIGNW" },
	{ X86_INS_VPSLLDQ, "VPSLLDQ" },
	{ X86_INS_VPSLLD, "VPSLLD" },
	{ X86_INS_VPSLLQ, "VPSLLQ" },
	{ X86_INS_VPSLLVD, "VPSLLVD" },
	{ X86_INS_VPSLLVQ, "VPSLLVQ" },
	{ X86_INS_VPSLLW, "VPSLLW" },
	{ X86_INS_VPSRAD, "VPSRAD" },
	{ X86_INS_VPSRAQ, "VPSRAQ" },
	{ X86_INS_VPSRAVD, "VPSRAVD" },
	{ X86_INS_VPSRAVQ, "VPSRAVQ" },
	{ X86_INS_VPSRAW, "VPSRAW" },
	{ X86_INS_VPSRLDQ, "VPSRLDQ" },
	{ X86_INS_VPSRLD, "VPSRLD" },
	{ X86_INS_VPSRLQ, "VPSRLQ" },
	{ X86_INS_VPSRLVD, "VPSRLVD" },
	{ X86_INS_VPSRLVQ, "VPSRLVQ" },
	{ X86_INS_VPSRLW, "VPSRLW" },
	{ X86_INS_VPSUBB, "VPSUBB" },
	{ X86_INS_VPSUBD, "VPSUBD" },
	{ X86_INS_VPSUBQ, "VPSUBQ" },
	{ X86_INS_VPSUBSB, "VPSUBSB" },
	{ X86_INS_VPSUBSW, "VPSUBSW" },
	{ X86_INS_VPSUBUSB, "VPSUBUSB" },
	{ X86_INS_VPSUBUSW, "VPSUBUSW" },
	{ X86_INS_VPSUBW, "VPSUBW" },
	{ X86_INS_VPTESTMD, "VPTESTMD" },
	{ X86_INS_VPTESTMQ, "VPTESTMQ" },
	{ X86_INS_VPTEST, "VPTEST" },
	{ X86_INS_VPUNPCKHBW, "VPUNPCKHBW" },
	{ X86_INS_VPUNPCKHDQ, "VPUNPCKHDQ" },
	{ X86_INS_VPUNPCKHQDQ, "VPUNPCKHQDQ" },
	{ X86_INS_VPUNPCKHWD, "VPUNPCKHWD" },
	{ X86_INS_VPUNPCKLBW, "VPUNPCKLBW" },
	{ X86_INS_VPUNPCKLDQ, "VPUNPCKLDQ" },
	{ X86_INS_VPUNPCKLQDQ, "VPUNPCKLQDQ" },
	{ X86_INS_VPUNPCKLWD, "VPUNPCKLWD" },
	{ X86_INS_VPXORD, "VPXORD" },
	{ X86_INS_VPXORQ, "VPXORQ" },
	{ X86_INS_VPXOR, "VPXOR" },
	{ X86_INS_VRCP14PD, "VRCP14PD" },
	{ X86_INS_VRCP14PS, "VRCP14PS" },
	{ X86_INS_VRCP14SD, "VRCP14SD" },
	{ X86_INS_VRCP14SS, "VRCP14SS" },
	{ X86_INS_VRCPPS, "VRCPPS" },
	{ X86_INS_VRCPSS, "VRCPSS" },
	{ X86_INS_VRNDSCALESD, "VRNDSCALESD" },
	{ X86_INS_VRNDSCALESS, "VRNDSCALESS" },
	{ X86_INS_VRNDSCALEPD, "VRNDSCALEPD" },
	{ X86_INS_VRNDSCALEPS, "VRNDSCALEPS" },
	{ X86_INS_VROUNDPD, "VROUNDPD" },
	{ X86_INS_VROUNDPS, "VROUNDPS" },
	{ X86_INS_VROUNDSD, "VROUNDSD" },
	{ X86_INS_VROUNDSS, "VROUNDSS" },
	{ X86_INS_VRSQRT14PD, "VRSQRT14PD" },
	{ X86_INS_VRSQRT14PS, "VRSQRT14PS" },
	{ X86_INS_VRSQRT14SD, "VRSQRT14SD" },
	{ X86_INS_VRSQRT14SS, "VRSQRT14SS" },
	{ X86_INS_VRSQRTPS, "VRSQRTPS" },
	{ X86_INS_VRSQRTSS, "VRSQRTSS" },
	{ X86_INS_VSCATTERDPD, "VSCATTERDPD" },
	{ X86_INS_VSCATTERDPS, "VSCATTERDPS" },
	{ X86_INS_VSCATTERQPD, "VSCATTERQPD" },
	{ X86_INS_VSCATTERQPS, "VSCATTERQPS" },
	{ X86_INS_VSHUFPD, "VSHUFPD" },
	{ X86_INS_VSHUFPS, "VSHUFPS" },
	{ X86_INS_VSQRTPD, "VSQRTPD" },
	{ X86_INS_VSQRT, "VSQRT" },
	{ X86_INS_VSQRTPS, "VSQRTPS" },
	{ X86_INS_VSQRTSD, "VSQRTSD" },
	{ X86_INS_VSQRTSS, "VSQRTSS" },
	{ X86_INS_VSTMXCSR, "VSTMXCSR" },
	{ X86_INS_VSUBPD, "VSUBPD" },
	{ X86_INS_VSUBPS, "VSUBPS" },
	{ X86_INS_VSUBSD, "VSUBSD" },
	{ X86_INS_VSUBSS, "VSUBSS" },
	{ X86_INS_VTESTPD, "VTESTPD" },
	{ X86_INS_VTESTPS, "VTESTPS" },
	{ X86_INS_VUNPCKHPD, "VUNPCKHPD" },
	{ X86_INS_VUNPCKHPS, "VUNPCKHPS" },
	{ X86_INS_VUNPCKLPD, "VUNPCKLPD" },
	{ X86_INS_VUNPCKLPS, "VUNPCKLPS" },
	{ X86_INS_VZEROALL, "VZEROALL" },
	{ X86_INS_VZEROUPPER, "VZEROUPPER" },
	{ X86_INS_WAIT, "WAIT" },
	{ X86_INS_WBINVD, "WBINVD" },
	{ X86_INS_WRFSBASE, "WRFSBASE" },
	{ X86_INS_WRGSBASE, "WRGSBASE" },
	{ X86_INS_WRMSR, "WRMSR" },
	{ X86_INS_XABORT, "XABORT" },
	{ X86_INS_XACQUIRE, "XACQUIRE" },
	{ X86_INS_XBEGIN, "XBEGIN" },
	{ X86_INS_XCHG, "XCHG" },
	{ X86_INS_FXCH, "FXCH" },
	{ X86_INS_XCRYPTCBC, "XCRYPTCBC" },
	{ X86_INS_XCRYPTCFB, "XCRYPTCFB" },
	{ X86_INS_XCRYPTCTR, "XCRYPTCTR" },
	{ X86_INS_XCRYPTECB, "XCRYPTECB" },
	{ X86_INS_XCRYPTOFB, "XCRYPTOFB" },
	{ X86_INS_XEND, "XEND" },
	{ X86_INS_XGETBV, "XGETBV" },
	{ X86_INS_XLATB, "XLATB" },
	{ X86_INS_XRELEASE, "XRELEASE" },
	{ X86_INS_XRSTOR, "XRSTOR" },
	{ X86_INS_XRSTOR64, "XRSTOR64" },
	{ X86_INS_XSAVE, "XSAVE" },
	{ X86_INS_XSAVE64, "XSAVE64" },
	{ X86_INS_XSAVEOPT, "XSAVEOPT" },
	{ X86_INS_XSAVEOPT64, "XSAVEOPT64" },
	{ X86_INS_XSETBV, "XSETBV" },
	{ X86_INS_XSHA1, "XSHA1" },
	{ X86_INS_XSHA256, "XSHA256" },
	{ X86_INS_XSTORE, "XSTORE" },
	{ X86_INS_XTEST, "XTEST" },
};

char *X86_insn_name(unsigned int id)
{
	if (id >= X86_INS_MAX)
		return NULL;

	return insn_name_maps[id].name;
}

x86_reg X86_map_insn(char *name)
{
	int i = name2id(&insn_name_maps[1], ARR_SIZE(insn_name_maps) - 1, name);

	return (i != -1)? i : X86_REG_INVALID;
}

#define GET_INSTRINFO_ENUM
#include "X86GenInstrInfo.inc"

static insn_map insns[] = {
	{ X86_AAA, X86_INS_AAA, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_AAA, X86_INS_AAA, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_AAD8i8, X86_INS_AAD, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_AAM8i8, X86_INS_AAM, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_AAS, X86_INS_AAS, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_ABS_F, X86_INS_FABS, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_ADC16i16, X86_INS_ADC, { X86_REG_AX, X86_REG_FLAGS, 0 }, { X86_REG_AX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC16mi, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC16mi8, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC16mr, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC16ri, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC16ri8, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC16rm, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC16rr, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC16rr_REV, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC32i32, X86_INS_ADC, { X86_REG_EAX, X86_REG_FLAGS, 0 }, { X86_REG_EAX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC32mi, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC32mi8, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC32mr, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC32ri, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC32ri8, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC32rm, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC32rr, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC32rr_REV, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC64i32, X86_INS_ADC, { X86_REG_RAX, X86_REG_FLAGS, 0 }, { X86_REG_RAX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC64mi32, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC64mi8, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC64mr, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC64ri32, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC64ri8, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC64rm, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC64rr, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC64rr_REV, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC8i8, X86_INS_ADC, { X86_REG_AL, X86_REG_FLAGS, 0 }, { X86_REG_AL, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC8mi, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC8mr, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC8ri, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC8rm, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC8rr, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADC8rr_REV, X86_INS_ADC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADCX32rm, X86_INS_ADCX, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_ADX, 0 } },
	{ X86_ADCX32rr, X86_INS_ADCX, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_ADX, 0 } },
	{ X86_ADCX64rm, X86_INS_ADCX, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_ADX, 0 } },
	{ X86_ADCX64rr, X86_INS_ADCX, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_ADX, 0 } },
	{ X86_ADD16i16, X86_INS_ADD, { X86_REG_AX, 0 }, { X86_REG_AX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD16mi, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD16mi8, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD16mr, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD16ri, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD16ri8, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD16rm, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD16rr, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD16rr_REV, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD32i32, X86_INS_ADD, { X86_REG_EAX, 0 }, { X86_REG_EAX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD32mi, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD32mi8, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD32mr, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD32ri, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD32ri8, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD32rm, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD32rr, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD32rr_REV, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD64i32, X86_INS_ADD, { X86_REG_RAX, 0 }, { X86_REG_RAX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD64mi32, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD64mi8, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD64mr, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD64ri32, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD64ri8, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD64rm, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD64rr, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD64rr_REV, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD8i8, X86_INS_ADD, { X86_REG_AL, 0 }, { X86_REG_AL, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD8mi, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD8mr, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD8ri, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD8rm, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD8rr, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADD8rr_REV, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ADDPDrm, X86_INS_ADDPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_ADDPDrr, X86_INS_ADDPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_ADDPSrm, X86_INS_ADDPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_ADDPSrr, X86_INS_ADDPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_ADDSDrm, X86_INS_ADDSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_ADDSDrm_Int, X86_INS_ADDSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_ADDSDrr, X86_INS_ADDSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_ADDSDrr_Int, X86_INS_ADDSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_ADDSSrm, X86_INS_ADDSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_ADDSSrm_Int, X86_INS_ADDSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_ADDSSrr, X86_INS_ADDSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_ADDSSrr_Int, X86_INS_ADDSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_ADDSUBPDrm, X86_INS_ADDSUBPD, { 0 }, { 0 }, { X86_GRP_SSE3, 0 } },
	{ X86_ADDSUBPDrr, X86_INS_ADDSUBPD, { 0 }, { 0 }, { X86_GRP_SSE3, 0 } },
	{ X86_ADDSUBPSrm, X86_INS_ADDSUBPS, { 0 }, { 0 }, { X86_GRP_SSE3, 0 } },
	{ X86_ADDSUBPSrr, X86_INS_ADDSUBPS, { 0 }, { 0 }, { X86_GRP_SSE3, 0 } },
	{ X86_ADD_F32m, X86_INS_FADD, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_ADD_F64m, X86_INS_FADD, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_ADD_FI16m, X86_INS_FIADD, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_ADD_FI32m, X86_INS_FIADD, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_ADD_FPrST0, X86_INS_FADDP, { 0 }, { 0 }, { 0 } },
	{ X86_ADD_FST0r, X86_INS_FADD, { 0 }, { 0 }, { 0 } },
	{ X86_ADD_FrST0, X86_INS_FADD, { 0 }, { 0 }, { 0 } },
	{ X86_ADOX32rm, X86_INS_ADOX, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_ADX, 0 } },
	{ X86_ADOX32rr, X86_INS_ADOX, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_ADX, 0 } },
	{ X86_ADOX64rm, X86_INS_ADOX, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_ADX, 0 } },
	{ X86_ADOX64rr, X86_INS_ADOX, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_ADX, 0 } },
	{ X86_AESDECLASTrm, X86_INS_AESDECLAST, { 0 }, { 0 }, { X86_GRP_AES, 0 } },
	{ X86_AESDECLASTrr, X86_INS_AESDECLAST, { 0 }, { 0 }, { X86_GRP_AES, 0 } },
	{ X86_AESDECrm, X86_INS_AESDEC, { 0 }, { 0 }, { X86_GRP_AES, 0 } },
	{ X86_AESDECrr, X86_INS_AESDEC, { 0 }, { 0 }, { X86_GRP_AES, 0 } },
	{ X86_AESENCLASTrm, X86_INS_AESENCLAST, { 0 }, { 0 }, { X86_GRP_AES, 0 } },
	{ X86_AESENCLASTrr, X86_INS_AESENCLAST, { 0 }, { 0 }, { X86_GRP_AES, 0 } },
	{ X86_AESENCrm, X86_INS_AESENC, { 0 }, { 0 }, { X86_GRP_AES, 0 } },
	{ X86_AESENCrr, X86_INS_AESENC, { 0 }, { 0 }, { X86_GRP_AES, 0 } },
	{ X86_AESIMCrm, X86_INS_AESIMC, { 0 }, { 0 }, { X86_GRP_AES, 0 } },
	{ X86_AESIMCrr, X86_INS_AESIMC, { 0 }, { 0 }, { X86_GRP_AES, 0 } },
	{ X86_AESKEYGENASSIST128rm, X86_INS_AESKEYGENASSIST, { 0 }, { 0 }, { X86_GRP_AES, 0 } },
	{ X86_AESKEYGENASSIST128rr, X86_INS_AESKEYGENASSIST, { 0 }, { 0 }, { X86_GRP_AES, 0 } },
	{ X86_AND16i16, X86_INS_AND, { X86_REG_AX, 0 }, { X86_REG_AX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND16mi, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND16mi8, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND16mr, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND16ri, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND16ri8, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND16rm, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND16rr, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND16rr_REV, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND32i32, X86_INS_AND, { X86_REG_EAX, 0 }, { X86_REG_EAX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND32mi, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND32mi8, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND32mr, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND32ri, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND32ri8, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND32rm, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND32rr, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND32rr_REV, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND64i32, X86_INS_AND, { X86_REG_RAX, 0 }, { X86_REG_RAX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND64mi32, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND64mi8, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND64mr, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND64ri32, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND64ri8, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND64rm, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND64rr, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND64rr_REV, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND8i8, X86_INS_AND, { X86_REG_AL, 0 }, { X86_REG_AL, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND8mi, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND8mr, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND8ri, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND8rm, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND8rr, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_AND8rr_REV, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ANDN32rm, X86_INS_ANDN, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_ANDN32rr, X86_INS_ANDN, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_ANDN64rm, X86_INS_ANDN, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_ANDN64rr, X86_INS_ANDN, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_ANDNPDrm, X86_INS_ANDNPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_ANDNPDrr, X86_INS_ANDNPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_ANDNPSrm, X86_INS_ANDNPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_ANDNPSrr, X86_INS_ANDNPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_ANDPDrm, X86_INS_ANDPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_ANDPDrr, X86_INS_ANDPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_ANDPSrm, X86_INS_ANDPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_ANDPSrr, X86_INS_ANDPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_ARPL16mr, X86_INS_ARPL, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_ARPL16rr, X86_INS_ARPL, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_BEXTR32rm, X86_INS_BEXTR, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_BEXTR32rr, X86_INS_BEXTR, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_BEXTR64rm, X86_INS_BEXTR, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_BEXTR64rr, X86_INS_BEXTR, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_BLENDPDrmi, X86_INS_BLENDPD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_BLENDPDrri, X86_INS_BLENDPD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_BLENDPSrmi, X86_INS_BLENDPS, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_BLENDPSrri, X86_INS_BLENDPS, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_BLENDVPDrm0, X86_INS_BLENDVPD, { X86_REG_XMM0, 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_BLENDVPDrr0, X86_INS_BLENDVPD, { X86_REG_XMM0, 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_BLENDVPSrm0, X86_INS_BLENDVPS, { X86_REG_XMM0, 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_BLENDVPSrr0, X86_INS_BLENDVPS, { X86_REG_XMM0, 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_BLSI32rm, X86_INS_BLSI, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_BLSI32rr, X86_INS_BLSI, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_BLSI64rm, X86_INS_BLSI, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_BLSI64rr, X86_INS_BLSI, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_BLSMSK32rm, X86_INS_BLSMSK, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_BLSMSK32rr, X86_INS_BLSMSK, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_BLSMSK64rm, X86_INS_BLSMSK, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_BLSMSK64rr, X86_INS_BLSMSK, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_BLSR32rm, X86_INS_BLSR, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_BLSR32rr, X86_INS_BLSR, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_BLSR64rm, X86_INS_BLSR, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_BLSR64rr, X86_INS_BLSR, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_BOUNDS16rm, X86_INS_BOUND, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_BOUNDS32rm, X86_INS_BOUND, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_BSF16rm, X86_INS_BSF, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BSF16rr, X86_INS_BSF, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BSF32rm, X86_INS_BSF, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BSF32rr, X86_INS_BSF, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BSF64rm, X86_INS_BSF, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BSF64rr, X86_INS_BSF, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BSR16rm, X86_INS_BSR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BSR16rr, X86_INS_BSR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BSR32rm, X86_INS_BSR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BSR32rr, X86_INS_BSR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BSR64rm, X86_INS_BSR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BSR64rr, X86_INS_BSR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BSWAP32r, X86_INS_BSWAP, { 0 }, { 0 }, { 0 } },
	{ X86_BSWAP64r, X86_INS_BSWAP, { 0 }, { 0 }, { 0 } },
	{ X86_BT16mi8, X86_INS_BT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BT16mr, X86_INS_BT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BT16ri8, X86_INS_BT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BT16rr, X86_INS_BT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BT32mi8, X86_INS_BT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BT32mr, X86_INS_BT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BT32ri8, X86_INS_BT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BT32rr, X86_INS_BT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BT64mi8, X86_INS_BT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BT64mr, X86_INS_BT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BT64ri8, X86_INS_BT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BT64rr, X86_INS_BT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTC16mi8, X86_INS_BTC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTC16mr, X86_INS_BTC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTC16ri8, X86_INS_BTC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTC16rr, X86_INS_BTC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTC32mi8, X86_INS_BTC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTC32mr, X86_INS_BTC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTC32ri8, X86_INS_BTC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTC32rr, X86_INS_BTC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTC64mi8, X86_INS_BTC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTC64mr, X86_INS_BTC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTC64ri8, X86_INS_BTC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTC64rr, X86_INS_BTC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTR16mi8, X86_INS_BTR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTR16mr, X86_INS_BTR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTR16ri8, X86_INS_BTR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTR16rr, X86_INS_BTR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTR32mi8, X86_INS_BTR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTR32mr, X86_INS_BTR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTR32ri8, X86_INS_BTR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTR32rr, X86_INS_BTR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTR64mi8, X86_INS_BTR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTR64mr, X86_INS_BTR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTR64ri8, X86_INS_BTR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTR64rr, X86_INS_BTR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTS16mi8, X86_INS_BTS, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTS16mr, X86_INS_BTS, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTS16ri8, X86_INS_BTS, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTS16rr, X86_INS_BTS, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTS32mi8, X86_INS_BTS, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTS32mr, X86_INS_BTS, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTS32ri8, X86_INS_BTS, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTS32rr, X86_INS_BTS, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTS64mi8, X86_INS_BTS, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTS64mr, X86_INS_BTS, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTS64ri8, X86_INS_BTS, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BTS64rr, X86_INS_BTS, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_BZHI32rm, X86_INS_BZHI, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_BZHI32rr, X86_INS_BZHI, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_BZHI64rm, X86_INS_BZHI, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_BZHI64rr, X86_INS_BZHI, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_CALL32m, X86_INS_CALL, { X86_REG_ESP, 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_CALL32r, X86_INS_CALL, { X86_REG_ESP, 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_CALL64m, X86_INS_CALL, { X86_REG_RSP, 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_CALL64pcrel32, X86_INS_CALL, { X86_REG_RSP, 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_CALL64r, X86_INS_CALL, { X86_REG_RSP, 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_CALLpcrel16, X86_INS_CALLW, { X86_REG_ESP, 0 }, { 0 }, { 0 } },
	{ X86_CALLpcrel32, X86_INS_CALL, { X86_REG_ESP, 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_CBW, X86_INS_CBW, { X86_REG_AL, 0 }, { X86_REG_AX, 0 }, { 0 } },
	{ X86_CDQ, X86_INS_CDQ, { X86_REG_EAX, 0 }, { X86_REG_EAX, X86_REG_EDX, 0 }, { 0 } },
	{ X86_CDQE, X86_INS_CDQE, { X86_REG_EAX, 0 }, { X86_REG_RAX, 0 }, { 0 } },
	{ X86_CHS_F, X86_INS_FCHS, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_CLAC, X86_INS_CLAC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CLC, X86_INS_CLC, { 0 }, { 0 }, { 0 } },
	{ X86_CLD, X86_INS_CLD, { 0 }, { 0 }, { 0 } },
	{ X86_CLFLUSH, X86_INS_CLFLUSH, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CLGI, X86_INS_CLGI, { 0 }, { 0 }, { 0 } },
	{ X86_CLI, X86_INS_CLI, { 0 }, { 0 }, { 0 } },
	{ X86_CLTS, X86_INS_CLTS, { 0 }, { 0 }, { 0 } },
	{ X86_CMC, X86_INS_CMC, { 0 }, { 0 }, { 0 } },
	{ X86_CMOVA16rm, X86_INS_CMOVA, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVA16rr, X86_INS_CMOVA, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVA32rm, X86_INS_CMOVA, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVA32rr, X86_INS_CMOVA, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVA64rm, X86_INS_CMOVA, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVA64rr, X86_INS_CMOVA, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVAE16rm, X86_INS_CMOVAE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVAE16rr, X86_INS_CMOVAE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVAE32rm, X86_INS_CMOVAE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVAE32rr, X86_INS_CMOVAE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVAE64rm, X86_INS_CMOVAE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVAE64rr, X86_INS_CMOVAE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVB16rm, X86_INS_CMOVB, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVB16rr, X86_INS_CMOVB, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVB32rm, X86_INS_CMOVB, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVB32rr, X86_INS_CMOVB, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVB64rm, X86_INS_CMOVB, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVB64rr, X86_INS_CMOVB, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVBE16rm, X86_INS_CMOVBE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVBE16rr, X86_INS_CMOVBE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVBE32rm, X86_INS_CMOVBE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVBE32rr, X86_INS_CMOVBE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVBE64rm, X86_INS_CMOVBE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVBE64rr, X86_INS_CMOVBE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVBE_F, X86_INS_FCMOVBE, { 0 }, { X86_REG_FPSW, 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVB_F, X86_INS_FCMOVB, { 0 }, { X86_REG_FPSW, 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVE16rm, X86_INS_CMOVE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVE16rr, X86_INS_CMOVE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVE32rm, X86_INS_CMOVE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVE32rr, X86_INS_CMOVE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVE64rm, X86_INS_CMOVE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVE64rr, X86_INS_CMOVE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVE_F, X86_INS_FCMOVE, { 0 }, { X86_REG_FPSW, 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVG16rm, X86_INS_CMOVG, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVG16rr, X86_INS_CMOVG, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVG32rm, X86_INS_CMOVG, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVG32rr, X86_INS_CMOVG, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVG64rm, X86_INS_CMOVG, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVG64rr, X86_INS_CMOVG, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVGE16rm, X86_INS_CMOVGE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVGE16rr, X86_INS_CMOVGE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVGE32rm, X86_INS_CMOVGE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVGE32rr, X86_INS_CMOVGE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVGE64rm, X86_INS_CMOVGE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVGE64rr, X86_INS_CMOVGE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVL16rm, X86_INS_CMOVL, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVL16rr, X86_INS_CMOVL, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVL32rm, X86_INS_CMOVL, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVL32rr, X86_INS_CMOVL, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVL64rm, X86_INS_CMOVL, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVL64rr, X86_INS_CMOVL, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVLE16rm, X86_INS_CMOVLE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVLE16rr, X86_INS_CMOVLE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVLE32rm, X86_INS_CMOVLE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVLE32rr, X86_INS_CMOVLE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVLE64rm, X86_INS_CMOVLE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVLE64rr, X86_INS_CMOVLE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNBE_F, X86_INS_FCMOVNBE, { 0 }, { X86_REG_FPSW, 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNB_F, X86_INS_FCMOVNB, { 0 }, { X86_REG_FPSW, 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNE16rm, X86_INS_CMOVNE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNE16rr, X86_INS_CMOVNE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNE32rm, X86_INS_CMOVNE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNE32rr, X86_INS_CMOVNE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNE64rm, X86_INS_CMOVNE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNE64rr, X86_INS_CMOVNE, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNE_F, X86_INS_FCMOVNE, { 0 }, { X86_REG_FPSW, 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNO16rm, X86_INS_CMOVNO, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNO16rr, X86_INS_CMOVNO, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNO32rm, X86_INS_CMOVNO, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNO32rr, X86_INS_CMOVNO, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNO64rm, X86_INS_CMOVNO, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNO64rr, X86_INS_CMOVNO, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNP16rm, X86_INS_CMOVNP, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNP16rr, X86_INS_CMOVNP, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNP32rm, X86_INS_CMOVNP, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNP32rr, X86_INS_CMOVNP, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNP64rm, X86_INS_CMOVNP, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNP64rr, X86_INS_CMOVNP, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNP_F, X86_INS_FCMOVNU, { 0 }, { X86_REG_FPSW, 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNS16rm, X86_INS_CMOVNS, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNS16rr, X86_INS_CMOVNS, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNS32rm, X86_INS_CMOVNS, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNS32rr, X86_INS_CMOVNS, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNS64rm, X86_INS_CMOVNS, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVNS64rr, X86_INS_CMOVNS, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVO16rm, X86_INS_CMOVO, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVO16rr, X86_INS_CMOVO, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVO32rm, X86_INS_CMOVO, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVO32rr, X86_INS_CMOVO, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVO64rm, X86_INS_CMOVO, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVO64rr, X86_INS_CMOVO, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVP16rm, X86_INS_CMOVP, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVP16rr, X86_INS_CMOVP, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVP32rm, X86_INS_CMOVP, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVP32rr, X86_INS_CMOVP, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVP64rm, X86_INS_CMOVP, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVP64rr, X86_INS_CMOVP, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVP_F, X86_INS_FCMOVU, { 0 }, { X86_REG_FPSW, 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVS16rm, X86_INS_CMOVS, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVS16rr, X86_INS_CMOVS, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVS32rm, X86_INS_CMOVS, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVS32rr, X86_INS_CMOVS, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVS64rm, X86_INS_CMOVS, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMOVS64rr, X86_INS_CMOVS, { X86_REG_FLAGS, 0 }, { 0 }, { X86_GRP_CMOV, 0 } },
	{ X86_CMP16i16, X86_INS_CMP, { X86_REG_AX, 0 }, { X86_REG_AX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP16mi, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP16mi8, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP16mr, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP16ri, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP16ri8, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP16rm, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP16rr, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP16rr_REV, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP32i32, X86_INS_CMP, { X86_REG_EAX, 0 }, { X86_REG_EAX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP32mi, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP32mi8, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP32mr, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP32ri, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP32ri8, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP32rm, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP32rr, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP32rr_REV, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP64i32, X86_INS_CMP, { X86_REG_RAX, 0 }, { X86_REG_RAX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP64mi32, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP64mi8, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP64mr, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP64ri32, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP64ri8, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP64rm, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP64rr, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP64rr_REV, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP8i8, X86_INS_CMP, { X86_REG_AL, 0 }, { X86_REG_AL, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP8mi, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP8mr, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP8ri, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP8rm, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP8rr, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMP8rr_REV, X86_INS_CMP, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMPPDrmi, X86_INS_CMP, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CMPPDrmi_alt, X86_INS_CMPPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CMPPDrri, X86_INS_CMP, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CMPPDrri_alt, X86_INS_CMPPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CMPPSrmi, X86_INS_CMP, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_CMPPSrmi_alt, X86_INS_CMPPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_CMPPSrri, X86_INS_CMP, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_CMPPSrri_alt, X86_INS_CMPPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_CMPS16, X86_INS_CMPSW, { 0 }, { 0 }, { 0 } },
	{ X86_CMPS32, X86_INS_CMPSD, { 0 }, { 0 }, { 0 } },
	{ X86_CMPS64, X86_INS_CMPSQ, { 0 }, { 0 }, { 0 } },
	{ X86_CMPS8, X86_INS_CMPSB, { 0 }, { 0 }, { 0 } },
	{ X86_CMPSDrm, X86_INS_CMP, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CMPSDrm_alt, X86_INS_CMPSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CMPSDrr, X86_INS_CMP, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CMPSDrr_alt, X86_INS_CMPSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CMPSSrm, X86_INS_CMP, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_CMPSSrm_alt, X86_INS_CMPSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_CMPSSrr, X86_INS_CMP, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_CMPSSrr_alt, X86_INS_CMPSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_CMPXCHG16B, X86_INS_CMPXCHG16B, { X86_REG_RAX, X86_REG_RBX, X86_REG_RCX, X86_REG_RDX, 0 }, { X86_REG_RAX, X86_REG_RDX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMPXCHG16rm, X86_INS_CMPXCHG, { 0 }, { 0 }, { 0 } },
	{ X86_CMPXCHG16rr, X86_INS_CMPXCHG, { 0 }, { 0 }, { 0 } },
	{ X86_CMPXCHG32rm, X86_INS_CMPXCHG, { 0 }, { 0 }, { 0 } },
	{ X86_CMPXCHG32rr, X86_INS_CMPXCHG, { 0 }, { 0 }, { 0 } },
	{ X86_CMPXCHG64rm, X86_INS_CMPXCHG, { 0 }, { 0 }, { 0 } },
	{ X86_CMPXCHG64rr, X86_INS_CMPXCHG, { 0 }, { 0 }, { 0 } },
	{ X86_CMPXCHG8B, X86_INS_CMPXCHG8B, { X86_REG_EAX, X86_REG_EBX, X86_REG_ECX, X86_REG_EDX, 0 }, { X86_REG_EAX, X86_REG_EDX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_CMPXCHG8rm, X86_INS_CMPXCHG, { 0 }, { 0 }, { 0 } },
	{ X86_CMPXCHG8rr, X86_INS_CMPXCHG, { 0 }, { 0 }, { 0 } },
	{ X86_COMISDrm, X86_INS_COMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_COMISDrr, X86_INS_COMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_COMISSrm, X86_INS_COMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_COMISSrr, X86_INS_COMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_COMP_FST0r, X86_INS_FCOMP, { 0 }, { 0 }, { 0 } },
	{ X86_COM_FIPr, X86_INS_FCOMPI, { 0 }, { X86_REG_FLAGS, X86_REG_FPSW, 0 }, { 0 } },
	{ X86_COM_FIr, X86_INS_FCOMI, { 0 }, { X86_REG_FLAGS, X86_REG_FPSW, 0 }, { 0 } },
	{ X86_COM_FST0r, X86_INS_FCOM, { 0 }, { 0 }, { 0 } },
	{ X86_COS_F, X86_INS_FCOS, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_CPUID, X86_INS_CPUID, { 0 }, { 0 }, { 0 } },
	{ X86_CQO, X86_INS_CQO, { X86_REG_RAX, 0 }, { X86_REG_RAX, X86_REG_RDX, 0 }, { 0 } },
	{ X86_CRC32r32m16, X86_INS_CRC32, { 0 }, { 0 }, { X86_GRP_SSE42, 0 } },
	{ X86_CRC32r32m32, X86_INS_CRC32, { 0 }, { 0 }, { X86_GRP_SSE42, 0 } },
	{ X86_CRC32r32m8, X86_INS_CRC32, { 0 }, { 0 }, { X86_GRP_SSE42, 0 } },
	{ X86_CRC32r32r16, X86_INS_CRC32, { 0 }, { 0 }, { X86_GRP_SSE42, 0 } },
	{ X86_CRC32r32r32, X86_INS_CRC32, { 0 }, { 0 }, { X86_GRP_SSE42, 0 } },
	{ X86_CRC32r32r8, X86_INS_CRC32, { 0 }, { 0 }, { X86_GRP_SSE42, 0 } },
	{ X86_CRC32r64m64, X86_INS_CRC32, { 0 }, { 0 }, { X86_GRP_SSE42, 0 } },
	{ X86_CRC32r64m8, X86_INS_CRC32, { 0 }, { 0 }, { X86_GRP_SSE42, 0 } },
	{ X86_CRC32r64r64, X86_INS_CRC32, { 0 }, { 0 }, { X86_GRP_SSE42, 0 } },
	{ X86_CRC32r64r8, X86_INS_CRC32, { 0 }, { 0 }, { X86_GRP_SSE42, 0 } },
	{ X86_CS_PREFIX, X86_INS_CS, { 0 }, { 0 }, { 0 } },
	{ X86_CVTDQ2PDrm, X86_INS_CVTDQ2PD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTDQ2PDrr, X86_INS_CVTDQ2PD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTDQ2PSrm, X86_INS_CVTDQ2PS, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTDQ2PSrr, X86_INS_CVTDQ2PS, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTPD2DQrm, X86_INS_CVTPD2DQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTPD2DQrr, X86_INS_CVTPD2DQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTPD2PSrm, X86_INS_CVTPD2PS, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTPD2PSrr, X86_INS_CVTPD2PS, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTPS2DQrm, X86_INS_CVTPS2DQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTPS2DQrr, X86_INS_CVTPS2DQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTPS2PDrm, X86_INS_CVTPS2PD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTPS2PDrr, X86_INS_CVTPS2PD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTSD2SI64rm, X86_INS_CVTSD2SI, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTSD2SI64rr, X86_INS_CVTSD2SI, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTSD2SIrm, X86_INS_CVTSD2SI, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTSD2SIrr, X86_INS_CVTSD2SI, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTSD2SSrm, X86_INS_CVTSD2SS, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTSD2SSrr, X86_INS_CVTSD2SS, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTSI2SD64rm, X86_INS_CVTSI2SD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTSI2SD64rr, X86_INS_CVTSI2SD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTSI2SDrm, X86_INS_CVTSI2SD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTSI2SDrr, X86_INS_CVTSI2SD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTSI2SS64rm, X86_INS_CVTSI2SS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_CVTSI2SS64rr, X86_INS_CVTSI2SS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_CVTSI2SSrm, X86_INS_CVTSI2SS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_CVTSI2SSrr, X86_INS_CVTSI2SS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_CVTSS2SDrm, X86_INS_CVTSS2SD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTSS2SDrr, X86_INS_CVTSS2SD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTSS2SI64rm, X86_INS_CVTSS2SI, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_CVTSS2SI64rr, X86_INS_CVTSS2SI, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_CVTSS2SIrm, X86_INS_CVTSS2SI, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_CVTSS2SIrr, X86_INS_CVTSS2SI, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_CVTTPD2DQrm, X86_INS_CVTTPD2DQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTTPD2DQrr, X86_INS_CVTTPD2DQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTTPS2DQrm, X86_INS_CVTTPS2DQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTTPS2DQrr, X86_INS_CVTTPS2DQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTTSD2SI64rm, X86_INS_CVTTSD2SI, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTTSD2SI64rr, X86_INS_CVTTSD2SI, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTTSD2SIrm, X86_INS_CVTTSD2SI, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTTSD2SIrr, X86_INS_CVTTSD2SI, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_CVTTSS2SI64rm, X86_INS_CVTTSS2SI, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_CVTTSS2SI64rr, X86_INS_CVTTSS2SI, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_CVTTSS2SIrm, X86_INS_CVTTSS2SI, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_CVTTSS2SIrr, X86_INS_CVTTSS2SI, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_CWD, X86_INS_CWD, { X86_REG_AX, 0 }, { X86_REG_AX, X86_REG_DX, 0 }, { 0 } },
	{ X86_CWDE, X86_INS_CWDE, { X86_REG_AX, 0 }, { X86_REG_EAX, 0 }, { 0 } },
	{ X86_DAA, X86_INS_DAA, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_DAS, X86_INS_DAS, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_DATA16_PREFIX, X86_INS_DATA16, { 0 }, { 0 }, { 0 } },
	{ X86_DEC16m, X86_INS_DEC, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_DEC16r, X86_INS_DEC, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_DEC32m, X86_INS_DEC, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_DEC32r, X86_INS_DEC, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_DEC64_16m, X86_INS_DEC, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_DEC64_16r, X86_INS_DEC, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_DEC64_32m, X86_INS_DEC, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_DEC64_32r, X86_INS_DEC, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_DEC64m, X86_INS_DEC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_DEC64r, X86_INS_DEC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_DEC8m, X86_INS_DEC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_DEC8r, X86_INS_DEC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_DIV16m, X86_INS_DIV, { X86_REG_AX, X86_REG_DX, 0 }, { X86_REG_AX, X86_REG_DX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_DIV16r, X86_INS_DIV, { X86_REG_AX, X86_REG_DX, 0 }, { X86_REG_AX, X86_REG_DX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_DIV32m, X86_INS_DIV, { X86_REG_EAX, X86_REG_EDX, 0 }, { X86_REG_EAX, X86_REG_EDX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_DIV32r, X86_INS_DIV, { X86_REG_EAX, X86_REG_EDX, 0 }, { X86_REG_EAX, X86_REG_EDX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_DIV64m, X86_INS_DIV, { X86_REG_RAX, X86_REG_RDX, 0 }, { X86_REG_RAX, X86_REG_RDX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_DIV64r, X86_INS_DIV, { X86_REG_RAX, X86_REG_RDX, 0 }, { X86_REG_RAX, X86_REG_RDX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_DIV8m, X86_INS_DIV, { X86_REG_AX, 0 }, { X86_REG_AL, X86_REG_AH, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_DIV8r, X86_INS_DIV, { X86_REG_AX, 0 }, { X86_REG_AL, X86_REG_AH, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_DIVPDrm, X86_INS_DIVPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_DIVPDrr, X86_INS_DIVPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_DIVPSrm, X86_INS_DIVPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_DIVPSrr, X86_INS_DIVPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_DIVR_F32m, X86_INS_FDIVR, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_DIVR_F64m, X86_INS_FDIVR, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_DIVR_FI16m, X86_INS_FIDIVR, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_DIVR_FI32m, X86_INS_FIDIVR, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_DIVR_FPrST0, X86_INS_FDIVRP, { 0 }, { 0 }, { 0 } },
	{ X86_DIVR_FST0r, X86_INS_FDIVR, { 0 }, { 0 }, { 0 } },
	{ X86_DIVR_FrST0, X86_INS_FDIVR, { 0 }, { 0 }, { 0 } },
	{ X86_DIVSDrm, X86_INS_DIVSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_DIVSDrm_Int, X86_INS_DIVSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_DIVSDrr, X86_INS_DIVSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_DIVSDrr_Int, X86_INS_DIVSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_DIVSSrm, X86_INS_DIVSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_DIVSSrm_Int, X86_INS_DIVSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_DIVSSrr, X86_INS_DIVSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_DIVSSrr_Int, X86_INS_DIVSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_DIV_F32m, X86_INS_FDIV, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_DIV_F64m, X86_INS_FDIV, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_DIV_FI16m, X86_INS_FIDIV, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_DIV_FI32m, X86_INS_FIDIV, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_DIV_FPrST0, X86_INS_FDIVP, { 0 }, { 0 }, { 0 } },
	{ X86_DIV_FST0r, X86_INS_FDIV, { 0 }, { 0 }, { 0 } },
	{ X86_DIV_FrST0, X86_INS_FDIV, { 0 }, { 0 }, { 0 } },
	{ X86_DPPDrmi, X86_INS_DPPD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_DPPDrri, X86_INS_DPPD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_DPPSrmi, X86_INS_DPPS, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_DPPSrri, X86_INS_DPPS, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_DS_PREFIX, X86_INS_DS, { 0 }, { 0 }, { 0 } },
	{ X86_ENTER, X86_INS_ENTER, { 0 }, { 0 }, { 0 } },
	{ X86_ES_PREFIX, X86_INS_ES, { 0 }, { 0 }, { 0 } },
	{ X86_EXTRACTPSmr, X86_INS_EXTRACTPS, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_EXTRACTPSrr, X86_INS_EXTRACTPS, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_EXTRQ, X86_INS_EXTRQ, { 0 }, { 0 }, { X86_GRP_SSE4A, 0 } },
	{ X86_EXTRQI, X86_INS_EXTRQ, { 0 }, { 0 }, { X86_GRP_SSE4A, 0 } },
	{ X86_F2XM1, X86_INS_F2XM1, { 0 }, { 0 }, { 0 } },
	{ X86_FARCALL16i, X86_INS_LCALL, { X86_REG_ESP, 0 }, { 0 }, { 0 } },
	{ X86_FARCALL16m, X86_INS_LCALL, { X86_REG_ESP, 0 }, { 0 }, { 0 } },
	{ X86_FARCALL32i, X86_INS_LCALL, { X86_REG_ESP, 0 }, { 0 }, { 0 } },
	{ X86_FARCALL32m, X86_INS_LCALL, { X86_REG_ESP, 0 }, { 0 }, { 0 } },
	{ X86_FARCALL64, X86_INS_LCALL, { X86_REG_RSP, 0 }, { 0 }, { 0 } },
	{ X86_FARJMP16i, X86_INS_LJMP, { 0 }, { 0 }, { 0 } },
	{ X86_FARJMP16m, X86_INS_LJMP, { 0 }, { 0 }, { 0 } },
	{ X86_FARJMP32i, X86_INS_LJMP, { 0 }, { 0 }, { 0 } },
	{ X86_FARJMP32m, X86_INS_LJMP, { 0 }, { 0 }, { 0 } },
	{ X86_FARJMP64, X86_INS_LJMP, { 0 }, { 0 }, { 0 } },
	{ X86_FBLDm, X86_INS_FBLD, { 0 }, { 0 }, { 0 } },
	{ X86_FBSTPm, X86_INS_FBSTP, { 0 }, { 0 }, { 0 } },
	{ X86_FCOM32m, X86_INS_FCOM, { 0 }, { 0 }, { 0 } },
	{ X86_FCOM64m, X86_INS_FCOM, { 0 }, { 0 }, { 0 } },
	{ X86_FCOMP32m, X86_INS_FCOMP, { 0 }, { 0 }, { 0 } },
	{ X86_FCOMP64m, X86_INS_FCOMP, { 0 }, { 0 }, { 0 } },
	{ X86_FCOMPP, X86_INS_FCOMPP, { 0 }, { 0 }, { 0 } },
	{ X86_FDECSTP, X86_INS_FDECSTP, { 0 }, { 0 }, { 0 } },
	{ X86_FEMMS, X86_INS_FEMMS, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_FFREE, X86_INS_FFREE, { 0 }, { 0 }, { 0 } },
	{ X86_FICOM16m, X86_INS_FICOM, { 0 }, { 0 }, { 0 } },
	{ X86_FICOM32m, X86_INS_FICOM, { 0 }, { 0 }, { 0 } },
	{ X86_FICOMP16m, X86_INS_FICOMP, { 0 }, { 0 }, { 0 } },
	{ X86_FICOMP32m, X86_INS_FICOMP, { 0 }, { 0 }, { 0 } },
	{ X86_FINCSTP, X86_INS_FINCSTP, { 0 }, { 0 }, { 0 } },
	{ X86_FLDCW16m, X86_INS_FLDCW, { 0 }, { 0 }, { 0 } },
	{ X86_FLDENVm, X86_INS_FLDENV, { 0 }, { 0 }, { 0 } },
	{ X86_FLDL2E, X86_INS_FLDL2E, { 0 }, { 0 }, { 0 } },
	{ X86_FLDL2T, X86_INS_FLDL2T, { 0 }, { 0 }, { 0 } },
	{ X86_FLDLG2, X86_INS_FLDLG2, { 0 }, { 0 }, { 0 } },
	{ X86_FLDLN2, X86_INS_FLDLN2, { 0 }, { 0 }, { 0 } },
	{ X86_FLDPI, X86_INS_FLDPI, { 0 }, { 0 }, { 0 } },
	{ X86_FNCLEX, X86_INS_FNCLEX, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_FNINIT, X86_INS_FNINIT, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_FNOP, X86_INS_FNOP, { 0 }, { 0 }, { 0 } },
	{ X86_FNSTCW16m, X86_INS_FNSTCW, { 0 }, { 0 }, { 0 } },
	{ X86_FNSTSW16r, X86_INS_FNSTSW, { X86_REG_FPSW, 0 }, { X86_REG_AX, 0 }, { 0 } },
	{ X86_FNSTSWm, X86_INS_FNSTSW, { 0 }, { 0 }, { 0 } },
	{ X86_FPATAN, X86_INS_FPATAN, { 0 }, { 0 }, { 0 } },
	{ X86_FPREM, X86_INS_FPREM, { 0 }, { 0 }, { 0 } },
	{ X86_FPREM1, X86_INS_FPREM1, { 0 }, { 0 }, { 0 } },
	{ X86_FPTAN, X86_INS_FPTAN, { 0 }, { 0 }, { 0 } },
	{ X86_FRNDINT, X86_INS_FRNDINT, { 0 }, { 0 }, { 0 } },
	{ X86_FRSTORm, X86_INS_FRSTOR, { 0 }, { 0 }, { 0 } },
	{ X86_FSAVEm, X86_INS_FNSAVE, { 0 }, { 0 }, { 0 } },
	{ X86_FSCALE, X86_INS_FSCALE, { 0 }, { 0 }, { 0 } },
	{ X86_FSINCOS, X86_INS_FSINCOS, { 0 }, { 0 }, { 0 } },
	{ X86_FSTENVm, X86_INS_FNSTENV, { 0 }, { 0 }, { 0 } },
	{ X86_FS_PREFIX, X86_INS_FS, { 0 }, { 0 }, { 0 } },
	{ X86_FXAM, X86_INS_FXAM, { 0 }, { 0 }, { 0 } },
	{ X86_FXRSTOR, X86_INS_FXRSTOR, { 0 }, { 0 }, { 0 } },
	{ X86_FXRSTOR64, X86_INS_FXRSTORQ, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_FXSAVE, X86_INS_FXSAVE, { 0 }, { 0 }, { 0 } },
	{ X86_FXSAVE64, X86_INS_FXSAVEQ, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_FXTRACT, X86_INS_FXTRACT, { 0 }, { 0 }, { 0 } },
	{ X86_FYL2X, X86_INS_FYL2X, { 0 }, { 0 }, { 0 } },
	{ X86_FYL2XP1, X86_INS_FYL2XP1, { 0 }, { 0 }, { 0 } },
	{ X86_FsANDNPDrm, X86_INS_ANDNPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_FsANDNPDrr, X86_INS_ANDNPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_FsANDNPSrm, X86_INS_ANDNPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_FsANDNPSrr, X86_INS_ANDNPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_FsANDPDrm, X86_INS_ANDPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_FsANDPDrr, X86_INS_ANDPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_FsANDPSrm, X86_INS_ANDPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_FsANDPSrr, X86_INS_ANDPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_FsMOVAPDrm, X86_INS_MOVAPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_FsMOVAPDrr, X86_INS_MOVAPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_FsMOVAPSrm, X86_INS_MOVAPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_FsMOVAPSrr, X86_INS_MOVAPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_FsORPDrm, X86_INS_ORPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_FsORPDrr, X86_INS_ORPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_FsORPSrm, X86_INS_ORPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_FsORPSrr, X86_INS_ORPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_FsVMOVAPDrm, X86_INS_VMOVAPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_FsVMOVAPDrr, X86_INS_VMOVAPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_FsVMOVAPSrm, X86_INS_VMOVAPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_FsVMOVAPSrr, X86_INS_VMOVAPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_FsXORPDrm, X86_INS_XORPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_FsXORPDrr, X86_INS_XORPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_FsXORPSrm, X86_INS_XORPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_FsXORPSrr, X86_INS_XORPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_GS_PREFIX, X86_INS_GS, { 0 }, { 0 }, { 0 } },
	{ X86_HADDPDrm, X86_INS_HADDPD, { 0 }, { 0 }, { X86_GRP_SSE3, 0 } },
	{ X86_HADDPDrr, X86_INS_HADDPD, { 0 }, { 0 }, { X86_GRP_SSE3, 0 } },
	{ X86_HADDPSrm, X86_INS_HADDPS, { 0 }, { 0 }, { X86_GRP_SSE3, 0 } },
	{ X86_HADDPSrr, X86_INS_HADDPS, { 0 }, { 0 }, { X86_GRP_SSE3, 0 } },
	{ X86_HLT, X86_INS_HLT, { 0 }, { 0 }, { 0 } },
	{ X86_HSUBPDrm, X86_INS_HSUBPD, { 0 }, { 0 }, { X86_GRP_SSE3, 0 } },
	{ X86_HSUBPDrr, X86_INS_HSUBPD, { 0 }, { 0 }, { X86_GRP_SSE3, 0 } },
	{ X86_HSUBPSrm, X86_INS_HSUBPS, { 0 }, { 0 }, { X86_GRP_SSE3, 0 } },
	{ X86_HSUBPSrr, X86_INS_HSUBPS, { 0 }, { 0 }, { X86_GRP_SSE3, 0 } },
	{ X86_IDIV16m, X86_INS_IDIV, { X86_REG_AX, X86_REG_DX, 0 }, { X86_REG_AX, X86_REG_DX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IDIV16r, X86_INS_IDIV, { X86_REG_AX, X86_REG_DX, 0 }, { X86_REG_AX, X86_REG_DX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IDIV32m, X86_INS_IDIV, { X86_REG_EAX, X86_REG_EDX, 0 }, { X86_REG_EAX, X86_REG_EDX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IDIV32r, X86_INS_IDIV, { X86_REG_EAX, X86_REG_EDX, 0 }, { X86_REG_EAX, X86_REG_EDX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IDIV64m, X86_INS_IDIV, { X86_REG_RAX, X86_REG_RDX, 0 }, { X86_REG_RAX, X86_REG_RDX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IDIV64r, X86_INS_IDIV, { X86_REG_RAX, X86_REG_RDX, 0 }, { X86_REG_RAX, X86_REG_RDX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IDIV8m, X86_INS_IDIV, { X86_REG_AX, 0 }, { X86_REG_AL, X86_REG_AH, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IDIV8r, X86_INS_IDIV, { X86_REG_AX, 0 }, { X86_REG_AL, X86_REG_AH, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ILD_F16m, X86_INS_FILD, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_ILD_F32m, X86_INS_FILD, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_ILD_F64m, X86_INS_FILD, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_IMUL16m, X86_INS_IMUL, { X86_REG_AX, 0 }, { X86_REG_AX, X86_REG_DX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL16r, X86_INS_IMUL, { X86_REG_AX, 0 }, { X86_REG_AX, X86_REG_DX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL16rm, X86_INS_IMUL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL16rmi, X86_INS_IMUL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL16rmi8, X86_INS_IMUL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL16rr, X86_INS_IMUL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL16rri, X86_INS_IMUL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL16rri8, X86_INS_IMUL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL32m, X86_INS_IMUL, { X86_REG_EAX, 0 }, { X86_REG_EAX, X86_REG_EDX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL32r, X86_INS_IMUL, { X86_REG_EAX, 0 }, { X86_REG_EAX, X86_REG_EDX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL32rm, X86_INS_IMUL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL32rmi, X86_INS_IMUL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL32rmi8, X86_INS_IMUL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL32rr, X86_INS_IMUL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL32rri, X86_INS_IMUL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL32rri8, X86_INS_IMUL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL64m, X86_INS_IMUL, { X86_REG_RAX, 0 }, { X86_REG_RAX, X86_REG_RDX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL64r, X86_INS_IMUL, { X86_REG_RAX, 0 }, { X86_REG_RAX, X86_REG_RDX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL64rm, X86_INS_IMUL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL64rmi32, X86_INS_IMUL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL64rmi8, X86_INS_IMUL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL64rr, X86_INS_IMUL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL64rri32, X86_INS_IMUL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL64rri8, X86_INS_IMUL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_IMUL8m, X86_INS_IMUL, { X86_REG_AL, 0 }, { X86_REG_AL, X86_REG_FLAGS, X86_REG_AX, 0 }, { 0 } },
	{ X86_IMUL8r, X86_INS_IMUL, { X86_REG_AL, 0 }, { X86_REG_AL, X86_REG_FLAGS, X86_REG_AX, 0 }, { 0 } },
	{ X86_IN16, X86_INS_INS, { 0 }, { 0 }, { 0 } },
	{ X86_IN16ri, X86_INS_IN, { 0 }, { X86_REG_AX, 0 }, { 0 } },
	{ X86_IN16rr, X86_INS_IN, { X86_REG_DX, 0 }, { X86_REG_AX, 0 }, { 0 } },
	{ X86_IN32, X86_INS_INS, { 0 }, { 0 }, { 0 } },
	{ X86_IN32ri, X86_INS_IN, { 0 }, { X86_REG_EAX, 0 }, { 0 } },
	{ X86_IN32rr, X86_INS_IN, { X86_REG_DX, 0 }, { X86_REG_EAX, 0 }, { 0 } },
	{ X86_IN8, X86_INS_INS, { 0 }, { 0 }, { 0 } },
	{ X86_IN8ri, X86_INS_IN, { 0 }, { X86_REG_AL, 0 }, { 0 } },
	{ X86_IN8rr, X86_INS_IN, { X86_REG_DX, 0 }, { X86_REG_AL, 0 }, { 0 } },
	{ X86_INC16m, X86_INS_INC, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_INC16r, X86_INS_INC, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_INC32m, X86_INS_INC, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_INC32r, X86_INS_INC, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_INC64_16m, X86_INS_INC, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_INC64_16r, X86_INS_INC, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_INC64_32m, X86_INS_INC, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_INC64_32r, X86_INS_INC, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_INC64m, X86_INS_INC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_INC64r, X86_INS_INC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_INC8m, X86_INS_INC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_INC8r, X86_INS_INC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_INSERTPSrm, X86_INS_INSERTPS, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_INSERTPSrr, X86_INS_INSERTPS, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_INSERTQ, X86_INS_INSERTQ, { 0 }, { 0 }, { X86_GRP_SSE4A, 0 } },
	{ X86_INSERTQI, X86_INS_INSERTQ, { 0 }, { 0 }, { X86_GRP_SSE4A, 0 } },
	{ X86_INT, X86_INS_INT, { 0 }, { 0 }, { 0 } },
	{ X86_INT3, X86_INS_INT3, { 0 }, { 0 }, { 0 } },
	{ X86_INTO, X86_INS_INTO, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_INVD, X86_INS_INVD, { 0 }, { 0 }, { 0 } },
	{ X86_INVEPT32, X86_INS_INVEPT, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_INVEPT64, X86_INS_INVEPT, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_INVLPG, X86_INS_INVLPG, { 0 }, { 0 }, { 0 } },
	{ X86_INVLPGA32, X86_INS_INVLPGA, { X86_REG_EAX, X86_REG_ECX, 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_INVLPGA64, X86_INS_INVLPGA, { X86_REG_RAX, X86_REG_ECX, 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_INVPCID32, X86_INS_INVPCID, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_INVPCID64, X86_INS_INVPCID, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_INVVPID32, X86_INS_INVVPID, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_INVVPID64, X86_INS_INVVPID, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_IRET16, X86_INS_IRET, { 0 }, { 0 }, { 0 } },
	{ X86_IRET32, X86_INS_IRETD, { 0 }, { 0 }, { 0 } },
	{ X86_IRET64, X86_INS_IRETQ, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_ISTT_FP16m, X86_INS_FISTTP, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_ISTT_FP32m, X86_INS_FISTTP, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_ISTT_FP64m, X86_INS_FISTTP, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_IST_F16m, X86_INS_FIST, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_IST_F32m, X86_INS_FIST, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_IST_FP16m, X86_INS_FISTP, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_IST_FP32m, X86_INS_FISTP, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_IST_FP64m, X86_INS_FISTP, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_Int_CMPSDrm, X86_INS_CMP, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_Int_CMPSDrr, X86_INS_CMP, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_Int_CMPSSrm, X86_INS_CMP, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_Int_CMPSSrr, X86_INS_CMP, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_Int_COMISDrm, X86_INS_COMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_Int_COMISDrr, X86_INS_COMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_Int_COMISSrm, X86_INS_COMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_Int_COMISSrr, X86_INS_COMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_Int_CVTSD2SSrm, X86_INS_CVTSD2SS, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_Int_CVTSD2SSrr, X86_INS_CVTSD2SS, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_Int_CVTSI2SD64rm, X86_INS_CVTSI2SD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_Int_CVTSI2SD64rr, X86_INS_CVTSI2SD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_Int_CVTSI2SDrm, X86_INS_CVTSI2SD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_Int_CVTSI2SDrr, X86_INS_CVTSI2SD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_Int_CVTSI2SS64rm, X86_INS_CVTSI2SS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_Int_CVTSI2SS64rr, X86_INS_CVTSI2SS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_Int_CVTSI2SSrm, X86_INS_CVTSI2SS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_Int_CVTSI2SSrr, X86_INS_CVTSI2SS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_Int_CVTSS2SDrm, X86_INS_CVTSS2SD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_Int_CVTSS2SDrr, X86_INS_CVTSS2SD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_Int_CVTTSD2SI64rm, X86_INS_CVTTSD2SI, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_Int_CVTTSD2SI64rr, X86_INS_CVTTSD2SI, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_Int_CVTTSD2SIrm, X86_INS_CVTTSD2SI, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_Int_CVTTSD2SIrr, X86_INS_CVTTSD2SI, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_Int_CVTTSS2SI64rm, X86_INS_CVTTSS2SI, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_Int_CVTTSS2SI64rr, X86_INS_CVTTSS2SI, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_Int_CVTTSS2SIrm, X86_INS_CVTTSS2SI, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_Int_CVTTSS2SIrr, X86_INS_CVTTSS2SI, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_Int_UCOMISDrm, X86_INS_UCOMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_Int_UCOMISDrr, X86_INS_UCOMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_Int_UCOMISSrm, X86_INS_UCOMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_Int_UCOMISSrr, X86_INS_UCOMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_Int_VCMPSDrm, X86_INS_VCMPSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCMPSDrr, X86_INS_VCMPSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCMPSSrm, X86_INS_VCMPSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCMPSSrr, X86_INS_VCMPSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCOMISDZrm, X86_INS_VCOMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_Int_VCOMISDZrr, X86_INS_VCOMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_Int_VCOMISDrm, X86_INS_VCOMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCOMISDrr, X86_INS_VCOMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCOMISSZrm, X86_INS_VCOMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_Int_VCOMISSZrr, X86_INS_VCOMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_Int_VCOMISSrm, X86_INS_VCOMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCOMISSrr, X86_INS_VCOMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCVTSD2SSrm, X86_INS_VCVTSD2SS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCVTSD2SSrr, X86_INS_VCVTSD2SS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCVTSI2SD64rm, X86_INS_VCVTSI2SD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCVTSI2SD64rr, X86_INS_VCVTSI2SD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCVTSI2SDrm, X86_INS_VCVTSI2SD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCVTSI2SDrr, X86_INS_VCVTSI2SD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCVTSI2SS64rm, X86_INS_VCVTSI2SS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCVTSI2SS64rr, X86_INS_VCVTSI2SS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCVTSI2SSrm, X86_INS_VCVTSI2SS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCVTSI2SSrr, X86_INS_VCVTSI2SS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCVTSS2SDrm, X86_INS_VCVTSS2SD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCVTSS2SDrr, X86_INS_VCVTSS2SD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCVTTSD2SI64rm, X86_INS_VCVTTSD2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCVTTSD2SI64rr, X86_INS_VCVTTSD2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCVTTSD2SIrm, X86_INS_VCVTTSD2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCVTTSD2SIrr, X86_INS_VCVTTSD2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCVTTSS2SI64rm, X86_INS_VCVTTSS2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCVTTSS2SI64rr, X86_INS_VCVTTSS2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCVTTSS2SIrm, X86_INS_VCVTTSS2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VCVTTSS2SIrr, X86_INS_VCVTTSS2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VUCOMISDZrm, X86_INS_VUCOMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_Int_VUCOMISDZrr, X86_INS_VUCOMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_Int_VUCOMISDrm, X86_INS_VUCOMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VUCOMISDrr, X86_INS_VUCOMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VUCOMISSZrm, X86_INS_VUCOMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_Int_VUCOMISSZrr, X86_INS_VUCOMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_Int_VUCOMISSrm, X86_INS_VUCOMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_Int_VUCOMISSrr, X86_INS_VUCOMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_JAE_1, X86_INS_JAE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JAE_4, X86_INS_JAE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JA_1, X86_INS_JA, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JA_4, X86_INS_JA, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JBE_1, X86_INS_JBE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JBE_4, X86_INS_JBE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JB_1, X86_INS_JB, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JB_4, X86_INS_JB, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JCXZ, X86_INS_JCXZ, { X86_REG_CX, 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_JECXZ_32, X86_INS_JECXZ, { X86_REG_ECX, 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_JECXZ_64, X86_INS_JECXZ, { X86_REG_ECX, 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_JE_1, X86_INS_JE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JE_4, X86_INS_JE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JGE_1, X86_INS_JGE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JGE_4, X86_INS_JGE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JG_1, X86_INS_JG, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JG_4, X86_INS_JG, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JLE_1, X86_INS_JLE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JLE_4, X86_INS_JLE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JL_1, X86_INS_JL, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JL_4, X86_INS_JL, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JMP32m, X86_INS_JMP, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_JMP32r, X86_INS_JMP, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_JMP64m, X86_INS_JMP, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_JMP64pcrel32, X86_INS_JMPQ, { 0 }, { 0 }, { 0 } },
	{ X86_JMP64r, X86_INS_JMP, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_JMP_1, X86_INS_JMP, { 0 }, { 0 }, { 0 } },
	{ X86_JMP_4, X86_INS_JMP, { 0 }, { 0 }, { 0 } },
	{ X86_JNE_1, X86_INS_JNE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JNE_4, X86_INS_JNE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JNO_1, X86_INS_JNO, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JNO_4, X86_INS_JNO, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JNP_1, X86_INS_JNP, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JNP_4, X86_INS_JNP, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JNS_1, X86_INS_JNS, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JNS_4, X86_INS_JNS, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JO_1, X86_INS_JO, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JO_4, X86_INS_JO, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JP_1, X86_INS_JP, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JP_4, X86_INS_JP, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JRCXZ, X86_INS_JRCXZ, { X86_REG_RCX, 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_JS_1, X86_INS_JS, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_JS_4, X86_INS_JS, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_KADDWrr, X86_INS_KADDW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_KANDNWrr, X86_INS_KANDNW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_KANDWrr, X86_INS_KANDW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_KMOVWkk, X86_INS_KMOVW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_KMOVWkm, X86_INS_KMOVW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_KMOVWkr, X86_INS_KMOVW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_KMOVWmk, X86_INS_KMOVW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_KMOVWrk, X86_INS_KMOVW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_KNOTWrr, X86_INS_KNOTW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_KORTESTWrr, X86_INS_KORTESTW, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_KORWrr, X86_INS_KORW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_KSHIFTLWri, X86_INS_KSHIFTLW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_KSHIFTRWri, X86_INS_KSHIFTRW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_KTESTWrr, X86_INS_KTESTW, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_KUNPCKBWrr, X86_INS_KUNPCKBW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_KXNORWrr, X86_INS_KXNORW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_KXORWrr, X86_INS_KXORW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_LAHF, X86_INS_LAHF, { X86_REG_FLAGS, 0 }, { X86_REG_AH, 0 }, { 0 } },
	{ X86_LAR16rm, X86_INS_LAR, { 0 }, { 0 }, { 0 } },
	{ X86_LAR16rr, X86_INS_LAR, { 0 }, { 0 }, { 0 } },
	{ X86_LAR32rm, X86_INS_LAR, { 0 }, { 0 }, { 0 } },
	{ X86_LAR32rr, X86_INS_LAR, { 0 }, { 0 }, { 0 } },
	{ X86_LAR64rm, X86_INS_LAR, { 0 }, { 0 }, { 0 } },
	{ X86_LAR64rr, X86_INS_LAR, { 0 }, { 0 }, { 0 } },
	{ X86_LCMPXCHG16, X86_INS_CMPXCHG, { X86_REG_AX, 0 }, { X86_REG_AX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LCMPXCHG16B, X86_INS_CMPXCHG16B, { X86_REG_RAX, X86_REG_RBX, X86_REG_RCX, X86_REG_RDX, 0 }, { X86_REG_RAX, X86_REG_RDX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LCMPXCHG32, X86_INS_CMPXCHG, { X86_REG_EAX, 0 }, { X86_REG_EAX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LCMPXCHG64, X86_INS_CMPXCHG, { X86_REG_RAX, 0 }, { X86_REG_RAX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LCMPXCHG8, X86_INS_CMPXCHG, { X86_REG_AL, 0 }, { X86_REG_AL, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LCMPXCHG8B, X86_INS_CMPXCHG8B, { X86_REG_EAX, X86_REG_EBX, X86_REG_ECX, X86_REG_EDX, 0 }, { X86_REG_EAX, X86_REG_EDX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LDDQUrm, X86_INS_LDDQU, { 0 }, { 0 }, { X86_GRP_SSE3, 0 } },
	{ X86_LDMXCSR, X86_INS_LDMXCSR, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_LDS16rm, X86_INS_LDS, { 0 }, { 0 }, { 0 } },
	{ X86_LDS32rm, X86_INS_LDS, { 0 }, { 0 }, { 0 } },
	{ X86_LD_F0, X86_INS_FLDZ, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_LD_F1, X86_INS_FLD1, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_LD_F32m, X86_INS_FLD, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_LD_F64m, X86_INS_FLD, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_LD_F80m, X86_INS_FLD, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_LD_Frr, X86_INS_FLD, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_LEA16r, X86_INS_LEA, { 0 }, { 0 }, { 0 } },
	{ X86_LEA32r, X86_INS_LEA, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_LEA64_32r, X86_INS_LEA, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_LEA64r, X86_INS_LEA, { 0 }, { 0 }, { 0 } },
	{ X86_LEAVE, X86_INS_LEAVE, { X86_REG_EBP, X86_REG_ESP, 0 }, { X86_REG_EBP, X86_REG_ESP, 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_LEAVE64, X86_INS_LEAVE, { X86_REG_RBP, X86_REG_RSP, 0 }, { X86_REG_RBP, X86_REG_RSP, 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_LES16rm, X86_INS_LES, { 0 }, { 0 }, { 0 } },
	{ X86_LES32rm, X86_INS_LES, { 0 }, { 0 }, { 0 } },
	{ X86_LFENCE, X86_INS_LFENCE, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_LFS16rm, X86_INS_LFS, { 0 }, { 0 }, { 0 } },
	{ X86_LFS32rm, X86_INS_LFS, { 0 }, { 0 }, { 0 } },
	{ X86_LFS64rm, X86_INS_LFS, { 0 }, { 0 }, { 0 } },
	{ X86_LGDT16m, X86_INS_LGDT, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_LGDTm, X86_INS_LGDT, { 0 }, { 0 }, { 0 } },
	{ X86_LGS16rm, X86_INS_LGS, { 0 }, { 0 }, { 0 } },
	{ X86_LGS32rm, X86_INS_LGS, { 0 }, { 0 }, { 0 } },
	{ X86_LGS64rm, X86_INS_LGS, { 0 }, { 0 }, { 0 } },
	{ X86_LIDT16m, X86_INS_LIDT, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_LIDTm, X86_INS_LIDT, { 0 }, { 0 }, { 0 } },
	{ X86_LLDT16m, X86_INS_LLDT, { 0 }, { 0 }, { 0 } },
	{ X86_LLDT16r, X86_INS_LLDT, { 0 }, { 0 }, { 0 } },
	{ X86_LMSW16m, X86_INS_LMSW, { 0 }, { 0 }, { 0 } },
	{ X86_LMSW16r, X86_INS_LMSW, { 0 }, { 0 }, { 0 } },
	{ X86_LOCK_ADD16mi, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_ADD16mi8, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_ADD16mr, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_ADD32mi, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_ADD32mi8, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_ADD32mr, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_ADD64mi32, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_ADD64mi8, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_ADD64mr, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_ADD8mi, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_ADD8mr, X86_INS_ADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_AND16mi, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_AND16mi8, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_AND16mr, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_AND32mi, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_AND32mi8, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_AND32mr, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_AND64mi32, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_AND64mi8, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_AND64mr, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_AND8mi, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_AND8mr, X86_INS_AND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_DEC16m, X86_INS_DEC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_DEC32m, X86_INS_DEC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_DEC64m, X86_INS_DEC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_DEC8m, X86_INS_DEC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_INC16m, X86_INS_INC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_INC32m, X86_INS_INC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_INC64m, X86_INS_INC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_INC8m, X86_INS_INC, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_OR16mi, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_OR16mi8, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_OR16mr, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_OR32mi, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_OR32mi8, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_OR32mr, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_OR64mi32, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_OR64mi8, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_OR64mr, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_OR8mi, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_OR8mr, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_PREFIX, X86_INS_LOCK, { 0 }, { 0 }, { 0 } },
	{ X86_LOCK_SUB16mi, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_SUB16mi8, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_SUB16mr, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_SUB32mi, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_SUB32mi8, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_SUB32mr, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_SUB64mi32, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_SUB64mi8, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_SUB64mr, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_SUB8mi, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_SUB8mr, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_XOR16mi, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_XOR16mi8, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_XOR16mr, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_XOR32mi, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_XOR32mi8, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_XOR32mr, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_XOR64mi32, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_XOR64mi8, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_XOR64mr, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_XOR8mi, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LOCK_XOR8mr, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LODSB, X86_INS_LODSB, { 0 }, { 0 }, { 0 } },
	{ X86_LODSD, X86_INS_LODSD, { 0 }, { 0 }, { 0 } },
	{ X86_LODSQ, X86_INS_LODSQ, { 0 }, { 0 }, { 0 } },
	{ X86_LODSW, X86_INS_LODSW, { 0 }, { 0 }, { 0 } },
	{ X86_LOOP, X86_INS_LOOP, { 0 }, { 0 }, { 0 } },
	{ X86_LOOPE, X86_INS_LOOPE, { 0 }, { 0 }, { 0 } },
	{ X86_LOOPNE, X86_INS_LOOPNE, { 0 }, { 0 }, { 0 } },
	{ X86_LRETI, X86_INS_RETF, { 0 }, { 0 }, { 0 } },
	{ X86_LRETIW, X86_INS_RETF, { 0 }, { 0 }, { 0 } },
	{ X86_LRETL, X86_INS_RETF, { 0 }, { 0 }, { 0 } },
	{ X86_LRETQ, X86_INS_RETF, { 0 }, { 0 }, { 0 } },
	{ X86_LRETW, X86_INS_RETF, { 0 }, { 0 }, { 0 } },
	{ X86_LSL16rm, X86_INS_LSL, { 0 }, { 0 }, { 0 } },
	{ X86_LSL16rr, X86_INS_LSL, { 0 }, { 0 }, { 0 } },
	{ X86_LSL32rm, X86_INS_LSL, { 0 }, { 0 }, { 0 } },
	{ X86_LSL32rr, X86_INS_LSL, { 0 }, { 0 }, { 0 } },
	{ X86_LSL64rm, X86_INS_LSL, { 0 }, { 0 }, { 0 } },
	{ X86_LSL64rr, X86_INS_LSL, { 0 }, { 0 }, { 0 } },
	{ X86_LSS16rm, X86_INS_LSS, { 0 }, { 0 }, { 0 } },
	{ X86_LSS32rm, X86_INS_LSS, { 0 }, { 0 }, { 0 } },
	{ X86_LSS64rm, X86_INS_LSS, { 0 }, { 0 }, { 0 } },
	{ X86_LTRm, X86_INS_LTR, { 0 }, { 0 }, { 0 } },
	{ X86_LTRr, X86_INS_LTR, { 0 }, { 0 }, { 0 } },
	{ X86_LXADD16, X86_INS_XADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LXADD32, X86_INS_XADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LXADD64, X86_INS_XADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LXADD8, X86_INS_XADD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LZCNT16rm, X86_INS_LZCNT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LZCNT16rr, X86_INS_LZCNT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LZCNT32rm, X86_INS_LZCNT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LZCNT32rr, X86_INS_LZCNT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LZCNT64rm, X86_INS_LZCNT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_LZCNT64rr, X86_INS_LZCNT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_MASKMOVDQU, X86_INS_MASKMOVDQU, { X86_REG_EDI, 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MASKMOVDQU64, X86_INS_MASKMOVDQU, { X86_REG_RDI, 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MAXCPDrm, X86_INS_MAXPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MAXCPDrr, X86_INS_MAXPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MAXCPSrm, X86_INS_MAXPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MAXCPSrr, X86_INS_MAXPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MAXCSDrm, X86_INS_MAXSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MAXCSDrr, X86_INS_MAXSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MAXCSSrm, X86_INS_MAXSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MAXCSSrr, X86_INS_MAXSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MAXPDrm, X86_INS_MAXPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MAXPDrr, X86_INS_MAXPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MAXPSrm, X86_INS_MAXPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MAXPSrr, X86_INS_MAXPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MAXSDrm, X86_INS_MAXSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MAXSDrm_Int, X86_INS_MAXSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MAXSDrr, X86_INS_MAXSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MAXSDrr_Int, X86_INS_MAXSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MAXSSrm, X86_INS_MAXSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MAXSSrm_Int, X86_INS_MAXSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MAXSSrr, X86_INS_MAXSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MAXSSrr_Int, X86_INS_MAXSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MFENCE, X86_INS_MFENCE, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MINCPDrm, X86_INS_MINPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MINCPDrr, X86_INS_MINPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MINCPSrm, X86_INS_MINPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MINCPSrr, X86_INS_MINPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MINCSDrm, X86_INS_MINSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MINCSDrr, X86_INS_MINSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MINCSSrm, X86_INS_MINSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MINCSSrr, X86_INS_MINSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MINPDrm, X86_INS_MINPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MINPDrr, X86_INS_MINPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MINPSrm, X86_INS_MINPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MINPSrr, X86_INS_MINPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MINSDrm, X86_INS_MINSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MINSDrm_Int, X86_INS_MINSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MINSDrr, X86_INS_MINSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MINSDrr_Int, X86_INS_MINSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MINSSrm, X86_INS_MINSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MINSSrm_Int, X86_INS_MINSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MINSSrr, X86_INS_MINSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MINSSrr_Int, X86_INS_MINSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MMX_CVTPD2PIirm, X86_INS_CVTPD2PI, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MMX_CVTPD2PIirr, X86_INS_CVTPD2PI, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MMX_CVTPI2PDirm, X86_INS_CVTPI2PD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MMX_CVTPI2PDirr, X86_INS_CVTPI2PD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MMX_CVTPI2PSirm, X86_INS_CVTPI2PS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MMX_CVTPI2PSirr, X86_INS_CVTPI2PS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MMX_CVTPS2PIirm, X86_INS_CVTPS2PI, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MMX_CVTPS2PIirr, X86_INS_CVTPS2PI, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MMX_CVTTPD2PIirm, X86_INS_CVTTPD2PI, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MMX_CVTTPD2PIirr, X86_INS_CVTTPD2PI, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MMX_CVTTPS2PIirm, X86_INS_CVTTPS2PI, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MMX_CVTTPS2PIirr, X86_INS_CVTTPS2PI, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MMX_EMMS, X86_INS_EMMS, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_MASKMOVQ, X86_INS_MASKMOVQ, { X86_REG_EDI, 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_MASKMOVQ64, X86_INS_MASKMOVQ, { X86_REG_RDI, 0 }, { 0 }, { X86_GRP_MMX, X86_GRP_MODE64, 0 } },
	{ X86_MMX_MOVD64from64rr, X86_INS_MOVD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_MOVD64grr, X86_INS_MOVD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_MOVD64mr, X86_INS_MOVD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_MOVD64rm, X86_INS_MOVD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_MOVD64rr, X86_INS_MOVD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_MOVD64rrv164, X86_INS_MOVD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_MOVD64to64rr, X86_INS_MOVD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_MOVDQ2Qrr, X86_INS_MOVDQ2Q, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MMX_MOVFR642Qrr, X86_INS_MOVDQ2Q, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MMX_MOVNTQmr, X86_INS_MOVNTQ, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_MOVQ2DQrr, X86_INS_MOVQ2DQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MMX_MOVQ2FR64rr, X86_INS_MOVQ2DQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MMX_MOVQ64mr, X86_INS_MOVQ, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_MOVQ64rm, X86_INS_MOVQ, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_MOVQ64rr, X86_INS_MOVQ, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_MOVZDI2PDIrm, X86_INS_MOVD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_MOVZDI2PDIrr, X86_INS_MOVD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PABSBrm64, X86_INS_PABSB, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PABSBrr64, X86_INS_PABSB, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PABSDrm64, X86_INS_PABSD, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PABSDrr64, X86_INS_PABSD, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PABSWrm64, X86_INS_PABSW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PABSWrr64, X86_INS_PABSW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PACKSSDWirm, X86_INS_PACKSSDW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PACKSSDWirr, X86_INS_PACKSSDW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PACKSSWBirm, X86_INS_PACKSSWB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PACKSSWBirr, X86_INS_PACKSSWB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PACKUSWBirm, X86_INS_PACKUSWB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PACKUSWBirr, X86_INS_PACKUSWB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PADDBirm, X86_INS_PADDB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PADDBirr, X86_INS_PADDB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PADDDirm, X86_INS_PADDD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PADDDirr, X86_INS_PADDD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PADDQirm, X86_INS_PADDQ, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PADDQirr, X86_INS_PADDQ, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PADDSBirm, X86_INS_PADDSB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PADDSBirr, X86_INS_PADDSB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PADDSWirm, X86_INS_PADDSW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PADDSWirr, X86_INS_PADDSW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PADDUSBirm, X86_INS_PADDUSB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PADDUSBirr, X86_INS_PADDUSB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PADDUSWirm, X86_INS_PADDUSW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PADDUSWirr, X86_INS_PADDUSW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PADDWirm, X86_INS_PADDW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PADDWirr, X86_INS_PADDW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PALIGNR64irm, X86_INS_PALIGNR, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PALIGNR64irr, X86_INS_PALIGNR, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PANDNirm, X86_INS_PANDN, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PANDNirr, X86_INS_PANDN, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PANDirm, X86_INS_PAND, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PANDirr, X86_INS_PAND, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PAVGBirm, X86_INS_PAVGB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PAVGBirr, X86_INS_PAVGB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PAVGWirm, X86_INS_PAVGW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PAVGWirr, X86_INS_PAVGW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PCMPEQBirm, X86_INS_PCMPEQB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PCMPEQBirr, X86_INS_PCMPEQB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PCMPEQDirm, X86_INS_PCMPEQD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PCMPEQDirr, X86_INS_PCMPEQD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PCMPEQWirm, X86_INS_PCMPEQW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PCMPEQWirr, X86_INS_PCMPEQW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PCMPGTBirm, X86_INS_PCMPGTB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PCMPGTBirr, X86_INS_PCMPGTB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PCMPGTDirm, X86_INS_PCMPGTD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PCMPGTDirr, X86_INS_PCMPGTD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PCMPGTWirm, X86_INS_PCMPGTW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PCMPGTWirr, X86_INS_PCMPGTW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PEXTRWirri, X86_INS_PEXTRW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PHADDSWrm64, X86_INS_PHADDSW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PHADDSWrr64, X86_INS_PHADDSW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PHADDWrm64, X86_INS_PHADDW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PHADDWrr64, X86_INS_PHADDW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PHADDrm64, X86_INS_PHADDD, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PHADDrr64, X86_INS_PHADDD, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PHSUBDrm64, X86_INS_PHSUBD, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PHSUBDrr64, X86_INS_PHSUBD, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PHSUBSWrm64, X86_INS_PHSUBSW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PHSUBSWrr64, X86_INS_PHSUBSW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PHSUBWrm64, X86_INS_PHSUBW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PHSUBWrr64, X86_INS_PHSUBW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PINSRWirmi, X86_INS_PINSRW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PINSRWirri, X86_INS_PINSRW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PMADDUBSWrm64, X86_INS_PMADDUBSW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PMADDUBSWrr64, X86_INS_PMADDUBSW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PMADDWDirm, X86_INS_PMADDWD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PMADDWDirr, X86_INS_PMADDWD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PMAXSWirm, X86_INS_PMAXSW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PMAXSWirr, X86_INS_PMAXSW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PMAXUBirm, X86_INS_PMAXUB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PMAXUBirr, X86_INS_PMAXUB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PMINSWirm, X86_INS_PMINSW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PMINSWirr, X86_INS_PMINSW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PMINUBirm, X86_INS_PMINUB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PMINUBirr, X86_INS_PMINUB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PMOVMSKBrr, X86_INS_PMOVMSKB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PMULHRSWrm64, X86_INS_PMULHRSW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PMULHRSWrr64, X86_INS_PMULHRSW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PMULHUWirm, X86_INS_PMULHUW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PMULHUWirr, X86_INS_PMULHUW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PMULHWirm, X86_INS_PMULHW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PMULHWirr, X86_INS_PMULHW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PMULLWirm, X86_INS_PMULLW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PMULLWirr, X86_INS_PMULLW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PMULUDQirm, X86_INS_PMULUDQ, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PMULUDQirr, X86_INS_PMULUDQ, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PORirm, X86_INS_POR, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PORirr, X86_INS_POR, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSADBWirm, X86_INS_PSADBW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSADBWirr, X86_INS_PSADBW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSHUFBrm64, X86_INS_PSHUFB, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PSHUFBrr64, X86_INS_PSHUFB, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PSHUFWmi, X86_INS_PSHUFW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSHUFWri, X86_INS_PSHUFW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSIGNBrm64, X86_INS_PSIGNB, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PSIGNBrr64, X86_INS_PSIGNB, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PSIGNDrm64, X86_INS_PSIGND, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PSIGNDrr64, X86_INS_PSIGND, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PSIGNWrm64, X86_INS_PSIGNW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PSIGNWrr64, X86_INS_PSIGNW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_MMX_PSLLDri, X86_INS_PSLLD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSLLDrm, X86_INS_PSLLD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSLLDrr, X86_INS_PSLLD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSLLQri, X86_INS_PSLLQ, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSLLQrm, X86_INS_PSLLQ, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSLLQrr, X86_INS_PSLLQ, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSLLWri, X86_INS_PSLLW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSLLWrm, X86_INS_PSLLW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSLLWrr, X86_INS_PSLLW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSRADri, X86_INS_PSRAD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSRADrm, X86_INS_PSRAD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSRADrr, X86_INS_PSRAD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSRAWri, X86_INS_PSRAW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSRAWrm, X86_INS_PSRAW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSRAWrr, X86_INS_PSRAW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSRLDri, X86_INS_PSRLD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSRLDrm, X86_INS_PSRLD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSRLDrr, X86_INS_PSRLD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSRLQri, X86_INS_PSRLQ, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSRLQrm, X86_INS_PSRLQ, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSRLQrr, X86_INS_PSRLQ, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSRLWri, X86_INS_PSRLW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSRLWrm, X86_INS_PSRLW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSRLWrr, X86_INS_PSRLW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSUBBirm, X86_INS_PSUBB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSUBBirr, X86_INS_PSUBB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSUBDirm, X86_INS_PSUBD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSUBDirr, X86_INS_PSUBD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSUBQirm, X86_INS_PSUBQ, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSUBQirr, X86_INS_PSUBQ, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSUBSBirm, X86_INS_PSUBSB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSUBSBirr, X86_INS_PSUBSB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSUBSWirm, X86_INS_PSUBSW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSUBSWirr, X86_INS_PSUBSW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSUBUSBirm, X86_INS_PSUBUSB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSUBUSBirr, X86_INS_PSUBUSB, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSUBUSWirm, X86_INS_PSUBUSW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSUBUSWirr, X86_INS_PSUBUSW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSUBWirm, X86_INS_PSUBW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PSUBWirr, X86_INS_PSUBW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PUNPCKHBWirm, X86_INS_PUNPCKHBW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PUNPCKHBWirr, X86_INS_PUNPCKHBW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PUNPCKHDQirm, X86_INS_PUNPCKHDQ, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PUNPCKHDQirr, X86_INS_PUNPCKHDQ, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PUNPCKHWDirm, X86_INS_PUNPCKHWD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PUNPCKHWDirr, X86_INS_PUNPCKHWD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PUNPCKLBWirm, X86_INS_PUNPCKLBW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PUNPCKLBWirr, X86_INS_PUNPCKLBW, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PUNPCKLDQirm, X86_INS_PUNPCKLDQ, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PUNPCKLDQirr, X86_INS_PUNPCKLDQ, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PUNPCKLWDirm, X86_INS_PUNPCKLWD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PUNPCKLWDirr, X86_INS_PUNPCKLWD, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PXORirm, X86_INS_PXOR, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MMX_PXORirr, X86_INS_PXOR, { 0 }, { 0 }, { X86_GRP_MMX, 0 } },
	{ X86_MONITORrrr, X86_INS_MONITOR, { X86_REG_EAX, X86_REG_ECX, X86_REG_EDX, 0 }, { 0 }, { X86_GRP_SSE3, 0 } },
	{ X86_MONTMUL, X86_INS_MONTMUL, { X86_REG_RAX, X86_REG_RSI, 0 }, { X86_REG_RAX, X86_REG_RDX, X86_REG_RSI, 0 }, { 0 } },
	{ X86_MOV16ao16, X86_INS_MOV, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_MOV16mi, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV16mr, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV16ms, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV16o16a, X86_INS_MOV, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_MOV16ri, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV16rm, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV16rr, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV16rr_REV, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV16rs, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV16sm, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV16sr, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV32ao32, X86_INS_MOV, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_MOV32cr, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV32dr, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV32mi, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV32mr, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV32ms, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV32o32a, X86_INS_MOV, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_MOV32rc, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV32rd, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV32ri, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV32rm, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV32rr, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV32rr_REV, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV32rs, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV32sm, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV32sr, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV64ao16, X86_INS_MOVABS, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_MOV64ao32, X86_INS_MOVABS, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_MOV64ao64, X86_INS_MOVABS, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_MOV64ao8, X86_INS_MOVABS, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_MOV64cr, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV64dr, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV64mi32, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV64mr, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV64ms, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV64o16a, X86_INS_MOVABS, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_MOV64o32a, X86_INS_MOVABS, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_MOV64o64a, X86_INS_MOVABS, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_MOV64o8a, X86_INS_MOVABS, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_MOV64rc, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV64rd, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV64ri, X86_INS_MOVABS, { 0 }, { 0 }, { 0 } },
	{ X86_MOV64ri32, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV64rm, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV64rr, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV64rr_REV, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV64rs, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV64sm, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV64sr, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV64toPQIrr, X86_INS_MOVQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOV64toSDrm, X86_INS_MOVQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOV64toSDrr, X86_INS_MOVQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOV8ao8, X86_INS_MOV, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_MOV8mi, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV8mr, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV8o8a, X86_INS_MOV, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_MOV8ri, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV8rm, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV8rr, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOV8rr_REV, X86_INS_MOV, { 0 }, { 0 }, { 0 } },
	{ X86_MOVAPDmr, X86_INS_MOVAPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVAPDrm, X86_INS_MOVAPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVAPDrr, X86_INS_MOVAPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVAPDrr_REV, X86_INS_MOVAPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVAPSmr, X86_INS_MOVAPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MOVAPSrm, X86_INS_MOVAPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MOVAPSrr, X86_INS_MOVAPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MOVAPSrr_REV, X86_INS_MOVAPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MOVBE16mr, X86_INS_MOVBE, { 0 }, { 0 }, { 0 } },
	{ X86_MOVBE16rm, X86_INS_MOVBE, { 0 }, { 0 }, { 0 } },
	{ X86_MOVBE32mr, X86_INS_MOVBE, { 0 }, { 0 }, { 0 } },
	{ X86_MOVBE32rm, X86_INS_MOVBE, { 0 }, { 0 }, { 0 } },
	{ X86_MOVBE64mr, X86_INS_MOVBE, { 0 }, { 0 }, { 0 } },
	{ X86_MOVBE64rm, X86_INS_MOVBE, { 0 }, { 0 }, { 0 } },
	{ X86_MOVDDUPrm, X86_INS_MOVDDUP, { 0 }, { 0 }, { X86_GRP_SSE3, 0 } },
	{ X86_MOVDDUPrr, X86_INS_MOVDDUP, { 0 }, { 0 }, { X86_GRP_SSE3, 0 } },
	{ X86_MOVDI2PDIrm, X86_INS_MOVD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVDI2PDIrr, X86_INS_MOVD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVDI2SSrm, X86_INS_MOVD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVDI2SSrr, X86_INS_MOVD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVDQAmr, X86_INS_MOVDQA, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVDQArm, X86_INS_MOVDQA, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVDQArr, X86_INS_MOVDQA, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVDQArr_REV, X86_INS_MOVDQA, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVDQUmr, X86_INS_MOVDQU, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVDQUrm, X86_INS_MOVDQU, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVDQUrr, X86_INS_MOVDQU, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVDQUrr_REV, X86_INS_MOVDQU, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVHLPSrr, X86_INS_MOVHLPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MOVHPDmr, X86_INS_MOVHPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVHPDrm, X86_INS_MOVHPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVHPSmr, X86_INS_MOVHPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MOVHPSrm, X86_INS_MOVHPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MOVLHPSrr, X86_INS_MOVLHPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MOVLPDmr, X86_INS_MOVLPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVLPDrm, X86_INS_MOVLPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVLPSmr, X86_INS_MOVLPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MOVLPSrm, X86_INS_MOVLPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MOVLQ128mr, X86_INS_MOVQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVMSKPDrr32, X86_INS_MOVMSKPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVMSKPDrr64, X86_INS_MOVMSKPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVMSKPSrr32, X86_INS_MOVMSKPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MOVMSKPSrr64, X86_INS_MOVMSKPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MOVNTDQArm, X86_INS_MOVNTDQA, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_MOVNTDQmr, X86_INS_MOVNTDQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVNTI_64mr, X86_INS_MOVNTI, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVNTImr, X86_INS_MOVNTI, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVNTPDmr, X86_INS_MOVNTPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVNTPSmr, X86_INS_MOVNTPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MOVNTSD, X86_INS_MOVNTSD, { 0 }, { 0 }, { X86_GRP_SSE4A, 0 } },
	{ X86_MOVNTSS, X86_INS_MOVNTSS, { 0 }, { 0 }, { X86_GRP_SSE4A, 0 } },
	{ X86_MOVPDI2DImr, X86_INS_MOVD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVPDI2DIrr, X86_INS_MOVD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVPQI2QImr, X86_INS_MOVQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVPQIto64rr, X86_INS_MOVQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVQI2PQIrm, X86_INS_MOVQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVQxrxr, X86_INS_MOVQ, { 0 }, { 0 }, { 0 } },
	{ X86_MOVSB, X86_INS_MOVSB, { X86_REG_EDI, X86_REG_ESI, X86_REG_FLAGS, 0 }, { X86_REG_EDI, X86_REG_ESI, 0 }, { 0 } },
	{ X86_MOVSD, X86_INS_MOVSD, { X86_REG_EDI, X86_REG_ESI, X86_REG_FLAGS, 0 }, { X86_REG_EDI, X86_REG_ESI, 0 }, { 0 } },
	{ X86_MOVSDmr, X86_INS_MOVSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVSDrm, X86_INS_MOVSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVSDrr, X86_INS_MOVSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVSDrr_REV, X86_INS_MOVSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVSDto64mr, X86_INS_MOVQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVSDto64rr, X86_INS_MOVQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVSHDUPrm, X86_INS_MOVSHDUP, { 0 }, { 0 }, { X86_GRP_SSE3, 0 } },
	{ X86_MOVSHDUPrr, X86_INS_MOVSHDUP, { 0 }, { 0 }, { X86_GRP_SSE3, 0 } },
	{ X86_MOVSLDUPrm, X86_INS_MOVSLDUP, { 0 }, { 0 }, { X86_GRP_SSE3, 0 } },
	{ X86_MOVSLDUPrr, X86_INS_MOVSLDUP, { 0 }, { 0 }, { X86_GRP_SSE3, 0 } },
	{ X86_MOVSQ, X86_INS_MOVSQ, { X86_REG_EDI, X86_REG_ESI, X86_REG_FLAGS, 0 }, { X86_REG_EDI, X86_REG_ESI, 0 }, { 0 } },
	{ X86_MOVSS2DImr, X86_INS_MOVD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVSS2DIrr, X86_INS_MOVD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVSSmr, X86_INS_MOVSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MOVSSrm, X86_INS_MOVSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MOVSSrr, X86_INS_MOVSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MOVSSrr_REV, X86_INS_MOVSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MOVSW, X86_INS_MOVSW, { X86_REG_EDI, X86_REG_ESI, X86_REG_FLAGS, 0 }, { X86_REG_EDI, X86_REG_ESI, 0 }, { 0 } },
	{ X86_MOVSX16rm8, X86_INS_MOVSX, { 0 }, { 0 }, { 0 } },
	{ X86_MOVSX16rr8, X86_INS_MOVSX, { 0 }, { 0 }, { 0 } },
	{ X86_MOVSX32rm16, X86_INS_MOVSX, { 0 }, { 0 }, { 0 } },
	{ X86_MOVSX32rm8, X86_INS_MOVSX, { 0 }, { 0 }, { 0 } },
	{ X86_MOVSX32rr16, X86_INS_MOVSX, { 0 }, { 0 }, { 0 } },
	{ X86_MOVSX32rr8, X86_INS_MOVSX, { 0 }, { 0 }, { 0 } },
	{ X86_MOVSX64rm16, X86_INS_MOVSX, { 0 }, { 0 }, { 0 } },
	{ X86_MOVSX64rm32, X86_INS_MOVSXD, { 0 }, { 0 }, { 0 } },
	{ X86_MOVSX64rm8, X86_INS_MOVSX, { 0 }, { 0 }, { 0 } },
	{ X86_MOVSX64rr16, X86_INS_MOVSX, { 0 }, { 0 }, { 0 } },
	{ X86_MOVSX64rr32, X86_INS_MOVSXD, { 0 }, { 0 }, { 0 } },
	{ X86_MOVSX64rr8, X86_INS_MOVSX, { 0 }, { 0 }, { 0 } },
	{ X86_MOVUPDmr, X86_INS_MOVUPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVUPDrm, X86_INS_MOVUPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVUPDrr, X86_INS_MOVUPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVUPDrr_REV, X86_INS_MOVUPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVUPSmr, X86_INS_MOVUPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MOVUPSrm, X86_INS_MOVUPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MOVUPSrr, X86_INS_MOVUPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MOVUPSrr_REV, X86_INS_MOVUPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MOVZDI2PDIrm, X86_INS_MOVD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVZDI2PDIrr, X86_INS_MOVD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVZPQILo2PQIrm, X86_INS_MOVQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVZPQILo2PQIrr, X86_INS_MOVQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVZQI2PQIrm, X86_INS_MOVQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVZQI2PQIrr, X86_INS_MOVQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MOVZX16rm8, X86_INS_MOVZX, { 0 }, { 0 }, { 0 } },
	{ X86_MOVZX16rr8, X86_INS_MOVZX, { 0 }, { 0 }, { 0 } },
	{ X86_MOVZX32_NOREXrm8, X86_INS_MOVZX, { 0 }, { 0 }, { 0 } },
	{ X86_MOVZX32_NOREXrr8, X86_INS_MOVZX, { 0 }, { 0 }, { 0 } },
	{ X86_MOVZX32rm16, X86_INS_MOVZX, { 0 }, { 0 }, { 0 } },
	{ X86_MOVZX32rm8, X86_INS_MOVZX, { 0 }, { 0 }, { 0 } },
	{ X86_MOVZX32rr16, X86_INS_MOVZX, { 0 }, { 0 }, { 0 } },
	{ X86_MOVZX32rr8, X86_INS_MOVZX, { 0 }, { 0 }, { 0 } },
	{ X86_MOVZX64rm16_Q, X86_INS_MOVZX, { 0 }, { 0 }, { 0 } },
	{ X86_MOVZX64rm8_Q, X86_INS_MOVZX, { 0 }, { 0 }, { 0 } },
	{ X86_MOVZX64rr16_Q, X86_INS_MOVZX, { 0 }, { 0 }, { 0 } },
	{ X86_MOVZX64rr8_Q, X86_INS_MOVZX, { 0 }, { 0 }, { 0 } },
	{ X86_MPSADBWrmi, X86_INS_MPSADBW, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_MPSADBWrri, X86_INS_MPSADBW, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_MUL16m, X86_INS_MUL, { X86_REG_AX, 0 }, { X86_REG_AX, X86_REG_DX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_MUL16r, X86_INS_MUL, { X86_REG_AX, 0 }, { X86_REG_AX, X86_REG_DX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_MUL32m, X86_INS_MUL, { X86_REG_EAX, 0 }, { X86_REG_EAX, X86_REG_EDX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_MUL32r, X86_INS_MUL, { X86_REG_EAX, 0 }, { X86_REG_EAX, X86_REG_EDX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_MUL64m, X86_INS_MUL, { X86_REG_RAX, 0 }, { X86_REG_RAX, X86_REG_RDX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_MUL64r, X86_INS_MUL, { X86_REG_RAX, 0 }, { X86_REG_RAX, X86_REG_RDX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_MUL8m, X86_INS_MUL, { X86_REG_AL, 0 }, { X86_REG_AL, X86_REG_FLAGS, X86_REG_AX, 0 }, { 0 } },
	{ X86_MUL8r, X86_INS_MUL, { X86_REG_AL, 0 }, { X86_REG_AL, X86_REG_FLAGS, X86_REG_AX, 0 }, { 0 } },
	{ X86_MULPDrm, X86_INS_MULPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MULPDrr, X86_INS_MULPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MULPSrm, X86_INS_MULPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MULPSrr, X86_INS_MULPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MULSDrm, X86_INS_MULSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MULSDrm_Int, X86_INS_MULSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MULSDrr, X86_INS_MULSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MULSDrr_Int, X86_INS_MULSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_MULSSrm, X86_INS_MULSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MULSSrm_Int, X86_INS_MULSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MULSSrr, X86_INS_MULSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MULSSrr_Int, X86_INS_MULSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_MULX32rm, X86_INS_MULX, { X86_REG_EDX, 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_MULX32rr, X86_INS_MULX, { X86_REG_EDX, 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_MULX64rm, X86_INS_MULX, { X86_REG_RDX, 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_MULX64rr, X86_INS_MULX, { X86_REG_RDX, 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_MUL_F32m, X86_INS_FMUL, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_MUL_F64m, X86_INS_FMUL, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_MUL_FI16m, X86_INS_FIMUL, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_MUL_FI32m, X86_INS_FIMUL, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_MUL_FPrST0, X86_INS_FMULP, { 0 }, { 0 }, { 0 } },
	{ X86_MUL_FST0r, X86_INS_FMUL, { 0 }, { 0 }, { 0 } },
	{ X86_MUL_FrST0, X86_INS_FMUL, { 0 }, { 0 }, { 0 } },
	{ X86_MWAITrr, X86_INS_MWAIT, { X86_REG_ECX, X86_REG_EAX, 0 }, { 0 }, { X86_GRP_SSE3, 0 } },
	{ X86_NEG16m, X86_INS_NEG, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_NEG16r, X86_INS_NEG, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_NEG32m, X86_INS_NEG, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_NEG32r, X86_INS_NEG, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_NEG64m, X86_INS_NEG, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_NEG64r, X86_INS_NEG, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_NEG8m, X86_INS_NEG, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_NEG8r, X86_INS_NEG, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_NOOP, X86_INS_NOP, { 0 }, { 0 }, { 0 } },
	{ X86_NOOPL, X86_INS_NOP, { 0 }, { 0 }, { 0 } },
	{ X86_NOOPW, X86_INS_NOP, { 0 }, { 0 }, { 0 } },
	{ X86_NOT16m, X86_INS_NOT, { 0 }, { 0 }, { 0 } },
	{ X86_NOT16r, X86_INS_NOT, { 0 }, { 0 }, { 0 } },
	{ X86_NOT32m, X86_INS_NOT, { 0 }, { 0 }, { 0 } },
	{ X86_NOT32r, X86_INS_NOT, { 0 }, { 0 }, { 0 } },
	{ X86_NOT64m, X86_INS_NOT, { 0 }, { 0 }, { 0 } },
	{ X86_NOT64r, X86_INS_NOT, { 0 }, { 0 }, { 0 } },
	{ X86_NOT8m, X86_INS_NOT, { 0 }, { 0 }, { 0 } },
	{ X86_NOT8r, X86_INS_NOT, { 0 }, { 0 }, { 0 } },
	{ X86_OR16i16, X86_INS_OR, { X86_REG_AX, 0 }, { X86_REG_AX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR16mi, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR16mi8, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR16mr, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR16ri, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR16ri8, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR16rm, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR16rr, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR16rr_REV, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR32i32, X86_INS_OR, { X86_REG_EAX, 0 }, { X86_REG_EAX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR32mi, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR32mi8, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR32mr, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR32mrLocked, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_OR32ri, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR32ri8, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR32rm, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR32rr, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR32rr_REV, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR64i32, X86_INS_OR, { X86_REG_RAX, 0 }, { X86_REG_RAX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR64mi32, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR64mi8, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR64mr, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR64ri32, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR64ri8, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR64rm, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR64rr, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR64rr_REV, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR8i8, X86_INS_OR, { X86_REG_AL, 0 }, { X86_REG_AL, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR8mi, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR8mr, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR8ri, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR8rm, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR8rr, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_OR8rr_REV, X86_INS_OR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ORPDrm, X86_INS_ORPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_ORPDrr, X86_INS_ORPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_ORPSrm, X86_INS_ORPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_ORPSrr, X86_INS_ORPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_OUT16ir, X86_INS_OUT, { X86_REG_AX, 0 }, { 0 }, { 0 } },
	{ X86_OUT16rr, X86_INS_OUT, { X86_REG_DX, X86_REG_AX, 0 }, { 0 }, { 0 } },
	{ X86_OUT32ir, X86_INS_OUT, { X86_REG_EAX, 0 }, { 0 }, { 0 } },
	{ X86_OUT32rr, X86_INS_OUT, { X86_REG_DX, X86_REG_EAX, 0 }, { 0 }, { 0 } },
	{ X86_OUT8ir, X86_INS_OUT, { X86_REG_AL, 0 }, { 0 }, { 0 } },
	{ X86_OUT8rr, X86_INS_OUT, { X86_REG_DX, X86_REG_AL, 0 }, { 0 }, { 0 } },
	{ X86_OUTSB, X86_INS_OUTSB, { 0 }, { 0 }, { 0 } },
	{ X86_OUTSD, X86_INS_OUTSD, { 0 }, { 0 }, { 0 } },
	{ X86_OUTSW, X86_INS_OUTSW, { 0 }, { 0 }, { 0 } },
	{ X86_PABSBrm128, X86_INS_PABSB, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PABSBrr128, X86_INS_PABSB, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PABSDrm128, X86_INS_PABSD, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PABSDrr128, X86_INS_PABSD, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PABSWrm128, X86_INS_PABSW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PABSWrr128, X86_INS_PABSW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PACKSSDWrm, X86_INS_PACKSSDW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PACKSSDWrr, X86_INS_PACKSSDW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PACKSSWBrm, X86_INS_PACKSSWB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PACKSSWBrr, X86_INS_PACKSSWB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PACKUSDWrm, X86_INS_PACKUSDW, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PACKUSDWrr, X86_INS_PACKUSDW, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PACKUSWBrm, X86_INS_PACKUSWB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PACKUSWBrr, X86_INS_PACKUSWB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PADDBrm, X86_INS_PADDB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PADDBrr, X86_INS_PADDB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PADDDrm, X86_INS_PADDD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PADDDrr, X86_INS_PADDD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PADDQrm, X86_INS_PADDQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PADDQrr, X86_INS_PADDQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PADDSBrm, X86_INS_PADDSB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PADDSBrr, X86_INS_PADDSB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PADDSWrm, X86_INS_PADDSW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PADDSWrr, X86_INS_PADDSW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PADDUSBrm, X86_INS_PADDUSB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PADDUSBrr, X86_INS_PADDUSB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PADDUSWrm, X86_INS_PADDUSW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PADDUSWrr, X86_INS_PADDUSW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PADDWrm, X86_INS_PADDW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PADDWrr, X86_INS_PADDW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PALIGNR128rm, X86_INS_PALIGNR, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PALIGNR128rr, X86_INS_PALIGNR, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PANDNrm, X86_INS_PANDN, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PANDNrr, X86_INS_PANDN, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PANDrm, X86_INS_PAND, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PANDrr, X86_INS_PAND, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PAUSE, X86_INS_PAUSE, { 0 }, { 0 }, { 0 } },
	{ X86_PAVGBrm, X86_INS_PAVGB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PAVGBrr, X86_INS_PAVGB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PAVGUSBrm, X86_INS_PAVGUSB, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PAVGUSBrr, X86_INS_PAVGUSB, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PAVGWrm, X86_INS_PAVGW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PAVGWrr, X86_INS_PAVGW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PBLENDVBrm0, X86_INS_PBLENDVB, { X86_REG_XMM0, 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PBLENDVBrr0, X86_INS_PBLENDVB, { X86_REG_XMM0, 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PBLENDWrmi, X86_INS_PBLENDW, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PBLENDWrri, X86_INS_PBLENDW, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PCLMULQDQrm, X86_INS_PCLMULQDQ, { 0 }, { 0 }, { X86_GRP_PCLMUL, 0 } },
	{ X86_PCLMULQDQrr, X86_INS_PCLMULQDQ, { 0 }, { 0 }, { X86_GRP_PCLMUL, 0 } },
	{ X86_PCMPEQBrm, X86_INS_PCMPEQB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PCMPEQBrr, X86_INS_PCMPEQB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PCMPEQDrm, X86_INS_PCMPEQD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PCMPEQDrr, X86_INS_PCMPEQD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PCMPEQQrm, X86_INS_PCMPEQQ, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PCMPEQQrr, X86_INS_PCMPEQQ, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PCMPEQWrm, X86_INS_PCMPEQW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PCMPEQWrr, X86_INS_PCMPEQW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PCMPESTRIrm, X86_INS_PCMPESTRI, { X86_REG_EAX, X86_REG_EDX, 0 }, { X86_REG_ECX, X86_REG_FLAGS, 0 }, { X86_GRP_SSE42, 0 } },
	{ X86_PCMPESTRIrr, X86_INS_PCMPESTRI, { X86_REG_EAX, X86_REG_EDX, 0 }, { X86_REG_ECX, X86_REG_FLAGS, 0 }, { X86_GRP_SSE42, 0 } },
	{ X86_PCMPESTRM128rm, X86_INS_PCMPESTRM, { X86_REG_EAX, X86_REG_EDX, 0 }, { X86_REG_XMM0, X86_REG_FLAGS, 0 }, { X86_GRP_SSE42, 0 } },
	{ X86_PCMPESTRM128rr, X86_INS_PCMPESTRM, { X86_REG_EAX, X86_REG_EDX, 0 }, { X86_REG_XMM0, X86_REG_FLAGS, 0 }, { X86_GRP_SSE42, 0 } },
	{ X86_PCMPGTBrm, X86_INS_PCMPGTB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PCMPGTBrr, X86_INS_PCMPGTB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PCMPGTDrm, X86_INS_PCMPGTD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PCMPGTDrr, X86_INS_PCMPGTD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PCMPGTQrm, X86_INS_PCMPGTQ, { 0 }, { 0 }, { X86_GRP_SSE42, 0 } },
	{ X86_PCMPGTQrr, X86_INS_PCMPGTQ, { 0 }, { 0 }, { X86_GRP_SSE42, 0 } },
	{ X86_PCMPGTWrm, X86_INS_PCMPGTW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PCMPGTWrr, X86_INS_PCMPGTW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PCMPISTRIrm, X86_INS_PCMPISTRI, { 0 }, { X86_REG_ECX, X86_REG_FLAGS, 0 }, { X86_GRP_SSE42, 0 } },
	{ X86_PCMPISTRIrr, X86_INS_PCMPISTRI, { 0 }, { X86_REG_ECX, X86_REG_FLAGS, 0 }, { X86_GRP_SSE42, 0 } },
	{ X86_PCMPISTRM128rm, X86_INS_PCMPISTRM, { 0 }, { X86_REG_XMM0, X86_REG_FLAGS, 0 }, { X86_GRP_SSE42, 0 } },
	{ X86_PCMPISTRM128rr, X86_INS_PCMPISTRM, { 0 }, { X86_REG_XMM0, X86_REG_FLAGS, 0 }, { X86_GRP_SSE42, 0 } },
	{ X86_PDEP32rm, X86_INS_PDEP, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_PDEP32rr, X86_INS_PDEP, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_PDEP64rm, X86_INS_PDEP, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_PDEP64rr, X86_INS_PDEP, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_PEXT32rm, X86_INS_PEXT, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_PEXT32rr, X86_INS_PEXT, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_PEXT64rm, X86_INS_PEXT, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_PEXT64rr, X86_INS_PEXT, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_PEXTRBmr, X86_INS_PEXTRB, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PEXTRBrr, X86_INS_PEXTRB, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PEXTRDmr, X86_INS_PEXTRD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PEXTRDrr, X86_INS_PEXTRD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PEXTRQmr, X86_INS_PEXTRQ, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PEXTRQrr, X86_INS_PEXTRQ, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PEXTRWmr, X86_INS_PEXTRW, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PEXTRWri, X86_INS_PEXTRW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PF2IDrm, X86_INS_PF2ID, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PF2IDrr, X86_INS_PF2ID, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PF2IWrm, X86_INS_PF2IW, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PF2IWrr, X86_INS_PF2IW, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFACCrm, X86_INS_PFACC, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFACCrr, X86_INS_PFACC, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFADDrm, X86_INS_PFADD, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFADDrr, X86_INS_PFADD, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFCMPEQrm, X86_INS_PFCMPEQ, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFCMPEQrr, X86_INS_PFCMPEQ, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFCMPGErm, X86_INS_PFCMPGE, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFCMPGErr, X86_INS_PFCMPGE, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFCMPGTrm, X86_INS_PFCMPGT, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFCMPGTrr, X86_INS_PFCMPGT, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFMAXrm, X86_INS_PFMAX, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFMAXrr, X86_INS_PFMAX, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFMINrm, X86_INS_PFMIN, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFMINrr, X86_INS_PFMIN, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFMULrm, X86_INS_PFMUL, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFMULrr, X86_INS_PFMUL, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFNACCrm, X86_INS_PFNACC, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFNACCrr, X86_INS_PFNACC, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFPNACCrm, X86_INS_PFPNACC, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFPNACCrr, X86_INS_PFPNACC, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFRCPIT1rm, X86_INS_PFRCPIT1, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFRCPIT1rr, X86_INS_PFRCPIT1, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFRCPIT2rm, X86_INS_PFRCPIT2, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFRCPIT2rr, X86_INS_PFRCPIT2, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFRCPrm, X86_INS_PFRCP, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFRCPrr, X86_INS_PFRCP, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFRSQIT1rm, X86_INS_PFRSQIT1, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFRSQIT1rr, X86_INS_PFRSQIT1, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFRSQRTrm, X86_INS_PFRSQRT, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFRSQRTrr, X86_INS_PFRSQRT, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFSUBRrm, X86_INS_PFSUBR, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFSUBRrr, X86_INS_PFSUBR, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFSUBrm, X86_INS_PFSUB, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PFSUBrr, X86_INS_PFSUB, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PHADDDrm, X86_INS_PHADDD, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PHADDDrr, X86_INS_PHADDD, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PHADDSWrm128, X86_INS_PHADDSW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PHADDSWrr128, X86_INS_PHADDSW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PHADDWrm, X86_INS_PHADDW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PHADDWrr, X86_INS_PHADDW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PHMINPOSUWrm128, X86_INS_PHMINPOSUW, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PHMINPOSUWrr128, X86_INS_PHMINPOSUW, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PHSUBDrm, X86_INS_PHSUBD, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PHSUBDrr, X86_INS_PHSUBD, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PHSUBSWrm128, X86_INS_PHSUBSW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PHSUBSWrr128, X86_INS_PHSUBSW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PHSUBWrm, X86_INS_PHSUBW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PHSUBWrr, X86_INS_PHSUBW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PI2FDrm, X86_INS_PI2FD, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PI2FDrr, X86_INS_PI2FD, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PI2FWrm, X86_INS_PI2FW, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PI2FWrr, X86_INS_PI2FW, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PINSRBrm, X86_INS_PINSRB, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PINSRBrr, X86_INS_PINSRB, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PINSRDrm, X86_INS_PINSRD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PINSRDrr, X86_INS_PINSRD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PINSRQrm, X86_INS_PINSRQ, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PINSRQrr, X86_INS_PINSRQ, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PINSRWrmi, X86_INS_PINSRW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PINSRWrri, X86_INS_PINSRW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PMADDUBSWrm128, X86_INS_PMADDUBSW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PMADDUBSWrr128, X86_INS_PMADDUBSW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PMADDWDrm, X86_INS_PMADDWD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PMADDWDrr, X86_INS_PMADDWD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PMAXSBrm, X86_INS_PMAXSB, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMAXSBrr, X86_INS_PMAXSB, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMAXSDrm, X86_INS_PMAXSD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMAXSDrr, X86_INS_PMAXSD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMAXSWrm, X86_INS_PMAXSW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PMAXSWrr, X86_INS_PMAXSW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PMAXUBrm, X86_INS_PMAXUB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PMAXUBrr, X86_INS_PMAXUB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PMAXUDrm, X86_INS_PMAXUD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMAXUDrr, X86_INS_PMAXUD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMAXUWrm, X86_INS_PMAXUW, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMAXUWrr, X86_INS_PMAXUW, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMINSBrm, X86_INS_PMINSB, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMINSBrr, X86_INS_PMINSB, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMINSDrm, X86_INS_PMINSD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMINSDrr, X86_INS_PMINSD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMINSWrm, X86_INS_PMINSW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PMINSWrr, X86_INS_PMINSW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PMINUBrm, X86_INS_PMINUB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PMINUBrr, X86_INS_PMINUB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PMINUDrm, X86_INS_PMINUD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMINUDrr, X86_INS_PMINUD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMINUWrm, X86_INS_PMINUW, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMINUWrr, X86_INS_PMINUW, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVMSKBrr, X86_INS_PMOVMSKB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PMOVSXBDrm, X86_INS_PMOVSXBD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVSXBDrr, X86_INS_PMOVSXBD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVSXBQrm, X86_INS_PMOVSXBQ, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVSXBQrr, X86_INS_PMOVSXBQ, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVSXBWrm, X86_INS_PMOVSXBW, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVSXBWrr, X86_INS_PMOVSXBW, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVSXDQrm, X86_INS_PMOVSXDQ, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVSXDQrr, X86_INS_PMOVSXDQ, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVSXWDrm, X86_INS_PMOVSXWD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVSXWDrr, X86_INS_PMOVSXWD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVSXWQrm, X86_INS_PMOVSXWQ, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVSXWQrr, X86_INS_PMOVSXWQ, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVZXBDrm, X86_INS_PMOVZXBD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVZXBDrr, X86_INS_PMOVZXBD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVZXBQrm, X86_INS_PMOVZXBQ, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVZXBQrr, X86_INS_PMOVZXBQ, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVZXBWrm, X86_INS_PMOVZXBW, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVZXBWrr, X86_INS_PMOVZXBW, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVZXDQrm, X86_INS_PMOVZXDQ, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVZXDQrr, X86_INS_PMOVZXDQ, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVZXWDrm, X86_INS_PMOVZXWD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVZXWDrr, X86_INS_PMOVZXWD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVZXWQrm, X86_INS_PMOVZXWQ, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMOVZXWQrr, X86_INS_PMOVZXWQ, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMULDQrm, X86_INS_PMULDQ, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMULDQrr, X86_INS_PMULDQ, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMULHRSWrm128, X86_INS_PMULHRSW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PMULHRSWrr128, X86_INS_PMULHRSW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PMULHRWrm, X86_INS_PMULHRW, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PMULHRWrr, X86_INS_PMULHRW, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PMULHUWrm, X86_INS_PMULHUW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PMULHUWrr, X86_INS_PMULHUW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PMULHWrm, X86_INS_PMULHW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PMULHWrr, X86_INS_PMULHW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PMULLDrm, X86_INS_PMULLD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMULLDrr, X86_INS_PMULLD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PMULLWrm, X86_INS_PMULLW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PMULLWrr, X86_INS_PMULLW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PMULUDQrm, X86_INS_PMULUDQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PMULUDQrr, X86_INS_PMULUDQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_POP16r, X86_INS_POP, { X86_REG_ESP, 0 }, { X86_REG_ESP, 0 }, { 0 } },
	{ X86_POP16rmm, X86_INS_POP, { X86_REG_ESP, 0 }, { X86_REG_ESP, 0 }, { 0 } },
	{ X86_POP16rmr, X86_INS_POP, { X86_REG_ESP, 0 }, { X86_REG_ESP, 0 }, { 0 } },
	{ X86_POP32r, X86_INS_POP, { X86_REG_ESP, 0 }, { X86_REG_ESP, 0 }, { 0 } },
	{ X86_POP32rmm, X86_INS_POP, { X86_REG_ESP, 0 }, { X86_REG_ESP, 0 }, { 0 } },
	{ X86_POP32rmr, X86_INS_POP, { X86_REG_ESP, 0 }, { X86_REG_ESP, 0 }, { 0 } },
	{ X86_POP64r, X86_INS_POP, { X86_REG_RSP, 0 }, { X86_REG_RSP, 0 }, { 0 } },
	{ X86_POP64rmm, X86_INS_POP, { X86_REG_RSP, 0 }, { X86_REG_RSP, 0 }, { 0 } },
	{ X86_POP64rmr, X86_INS_POP, { X86_REG_RSP, 0 }, { X86_REG_RSP, 0 }, { 0 } },
	{ X86_POPA32, X86_INS_POPA, { X86_REG_ESP, 0 }, { X86_REG_EDI, X86_REG_ESI, X86_REG_EBP, X86_REG_EBX, X86_REG_EDX, X86_REG_ECX, X86_REG_EAX, X86_REG_ESP, 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_POPCNT16rm, X86_INS_POPCNT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_POPCNT16rr, X86_INS_POPCNT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_POPCNT32rm, X86_INS_POPCNT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_POPCNT32rr, X86_INS_POPCNT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_POPCNT64rm, X86_INS_POPCNT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_POPCNT64rr, X86_INS_POPCNT, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_POPDS16, X86_INS_POP, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_POPDS32, X86_INS_POP, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_POPES16, X86_INS_POP, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_POPES32, X86_INS_POP, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_POPF16, X86_INS_POPF, { X86_REG_ESP, 0 }, { X86_REG_ESP, 0 }, { 0 } },
	{ X86_POPF32, X86_INS_POPFD, { X86_REG_ESP, 0 }, { X86_REG_ESP, 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_POPF64, X86_INS_POPFQ, { X86_REG_RSP, 0 }, { X86_REG_RSP, X86_REG_FLAGS, 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_POPFS16, X86_INS_POP, { 0 }, { 0 }, { 0 } },
	{ X86_POPFS32, X86_INS_POP, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_POPFS64, X86_INS_POP, { 0 }, { 0 }, { 0 } },
	{ X86_POPGS16, X86_INS_POP, { 0 }, { 0 }, { 0 } },
	{ X86_POPGS32, X86_INS_POP, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_POPGS64, X86_INS_POP, { 0 }, { 0 }, { 0 } },
	{ X86_POPSS16, X86_INS_POP, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_POPSS32, X86_INS_POP, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_PORrm, X86_INS_POR, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PORrr, X86_INS_POR, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PREFETCH, X86_INS_PREFETCH, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PREFETCHNTA, X86_INS_PREFETCHNTA, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_PREFETCHT0, X86_INS_PREFETCHT0, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_PREFETCHT1, X86_INS_PREFETCHT1, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_PREFETCHT2, X86_INS_PREFETCHT2, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_PREFETCHW, X86_INS_PREFETCHW, { 0 }, { 0 }, { 0 } },
	{ X86_PSADBWrm, X86_INS_PSADBW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSADBWrr, X86_INS_PSADBW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSHUFBrm, X86_INS_PSHUFB, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PSHUFBrr, X86_INS_PSHUFB, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PSHUFDmi, X86_INS_PSHUFD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSHUFDri, X86_INS_PSHUFD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSHUFHWmi, X86_INS_PSHUFHW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSHUFHWri, X86_INS_PSHUFHW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSHUFLWmi, X86_INS_PSHUFLW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSHUFLWri, X86_INS_PSHUFLW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSIGNBrm, X86_INS_PSIGNB, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PSIGNBrr, X86_INS_PSIGNB, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PSIGNDrm, X86_INS_PSIGND, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PSIGNDrr, X86_INS_PSIGND, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PSIGNWrm, X86_INS_PSIGNW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PSIGNWrr, X86_INS_PSIGNW, { 0 }, { 0 }, { X86_GRP_SSSE3, 0 } },
	{ X86_PSLLDQri, X86_INS_PSLLDQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSLLDri, X86_INS_PSLLD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSLLDrm, X86_INS_PSLLD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSLLDrr, X86_INS_PSLLD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSLLQri, X86_INS_PSLLQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSLLQrm, X86_INS_PSLLQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSLLQrr, X86_INS_PSLLQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSLLWri, X86_INS_PSLLW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSLLWrm, X86_INS_PSLLW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSLLWrr, X86_INS_PSLLW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSRADri, X86_INS_PSRAD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSRADrm, X86_INS_PSRAD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSRADrr, X86_INS_PSRAD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSRAWri, X86_INS_PSRAW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSRAWrm, X86_INS_PSRAW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSRAWrr, X86_INS_PSRAW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSRLDQri, X86_INS_PSRLDQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSRLDri, X86_INS_PSRLD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSRLDrm, X86_INS_PSRLD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSRLDrr, X86_INS_PSRLD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSRLQri, X86_INS_PSRLQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSRLQrm, X86_INS_PSRLQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSRLQrr, X86_INS_PSRLQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSRLWri, X86_INS_PSRLW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSRLWrm, X86_INS_PSRLW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSRLWrr, X86_INS_PSRLW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSUBBrm, X86_INS_PSUBB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSUBBrr, X86_INS_PSUBB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSUBDrm, X86_INS_PSUBD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSUBDrr, X86_INS_PSUBD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSUBQrm, X86_INS_PSUBQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSUBQrr, X86_INS_PSUBQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSUBSBrm, X86_INS_PSUBSB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSUBSBrr, X86_INS_PSUBSB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSUBSWrm, X86_INS_PSUBSW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSUBSWrr, X86_INS_PSUBSW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSUBUSBrm, X86_INS_PSUBUSB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSUBUSBrr, X86_INS_PSUBUSB, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSUBUSWrm, X86_INS_PSUBUSW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSUBUSWrr, X86_INS_PSUBUSW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSUBWrm, X86_INS_PSUBW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSUBWrr, X86_INS_PSUBW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PSWAPDrm, X86_INS_PSWAPD, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PSWAPDrr, X86_INS_PSWAPD, { 0 }, { 0 }, { X86_GRP_3DNOW, 0 } },
	{ X86_PTESTrm, X86_INS_PTEST, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PTESTrr, X86_INS_PTEST, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_PUNPCKHBWrm, X86_INS_PUNPCKHBW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PUNPCKHBWrr, X86_INS_PUNPCKHBW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PUNPCKHDQrm, X86_INS_PUNPCKHDQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PUNPCKHDQrr, X86_INS_PUNPCKHDQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PUNPCKHQDQrm, X86_INS_PUNPCKHQDQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PUNPCKHQDQrr, X86_INS_PUNPCKHQDQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PUNPCKHWDrm, X86_INS_PUNPCKHWD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PUNPCKHWDrr, X86_INS_PUNPCKHWD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PUNPCKLBWrm, X86_INS_PUNPCKLBW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PUNPCKLBWrr, X86_INS_PUNPCKLBW, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PUNPCKLDQrm, X86_INS_PUNPCKLDQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PUNPCKLDQrr, X86_INS_PUNPCKLDQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PUNPCKLQDQrm, X86_INS_PUNPCKLQDQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PUNPCKLQDQrr, X86_INS_PUNPCKLQDQ, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PUNPCKLWDrm, X86_INS_PUNPCKLWD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PUNPCKLWDrr, X86_INS_PUNPCKLWD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PUSH16r, X86_INS_PUSH, { X86_REG_ESP, 0 }, { X86_REG_ESP, 0 }, { 0 } },
	{ X86_PUSH16rmm, X86_INS_PUSH, { X86_REG_ESP, 0 }, { X86_REG_ESP, 0 }, { 0 } },
	{ X86_PUSH16rmr, X86_INS_PUSH, { X86_REG_ESP, 0 }, { X86_REG_ESP, 0 }, { 0 } },
	{ X86_PUSH32r, X86_INS_PUSH, { X86_REG_ESP, 0 }, { X86_REG_ESP, 0 }, { 0 } },
	{ X86_PUSH32rmm, X86_INS_PUSH, { X86_REG_ESP, 0 }, { X86_REG_ESP, 0 }, { 0 } },
	{ X86_PUSH32rmr, X86_INS_PUSH, { X86_REG_ESP, 0 }, { X86_REG_ESP, 0 }, { 0 } },
	{ X86_PUSH64i16, X86_INS_PUSH, { X86_REG_RSP, 0 }, { X86_REG_RSP, 0 }, { 0 } },
	{ X86_PUSH64i32, X86_INS_PUSH, { X86_REG_RSP, 0 }, { X86_REG_RSP, 0 }, { 0 } },
	{ X86_PUSH64i8, X86_INS_PUSH, { X86_REG_RSP, 0 }, { X86_REG_RSP, 0 }, { 0 } },
	{ X86_PUSH64r, X86_INS_PUSH, { X86_REG_RSP, 0 }, { X86_REG_RSP, 0 }, { 0 } },
	{ X86_PUSH64rmm, X86_INS_PUSH, { X86_REG_RSP, 0 }, { X86_REG_RSP, 0 }, { 0 } },
	{ X86_PUSH64rmr, X86_INS_PUSH, { X86_REG_RSP, 0 }, { X86_REG_RSP, 0 }, { 0 } },
	{ X86_PUSHA32, X86_INS_PUSHA, { X86_REG_EDI, X86_REG_ESI, X86_REG_EBP, X86_REG_EBX, X86_REG_EDX, X86_REG_ECX, X86_REG_EAX, X86_REG_ESP, 0 }, { X86_REG_ESP, 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_PUSHCS16, X86_INS_PUSH, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_PUSHCS32, X86_INS_PUSH, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_PUSHDS16, X86_INS_PUSH, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_PUSHDS32, X86_INS_PUSH, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_PUSHES16, X86_INS_PUSH, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_PUSHES32, X86_INS_PUSH, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_PUSHF16, X86_INS_PUSHF, { X86_REG_ESP, 0 }, { X86_REG_ESP, 0 }, { 0 } },
	{ X86_PUSHF32, X86_INS_PUSHFD, { X86_REG_ESP, 0 }, { X86_REG_ESP, 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_PUSHF64, X86_INS_PUSHFQ, { X86_REG_RSP, X86_REG_FLAGS, 0 }, { X86_REG_RSP, 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_PUSHFS16, X86_INS_PUSH, { 0 }, { 0 }, { 0 } },
	{ X86_PUSHFS32, X86_INS_PUSH, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_PUSHFS64, X86_INS_PUSH, { 0 }, { 0 }, { 0 } },
	{ X86_PUSHGS16, X86_INS_PUSH, { 0 }, { 0 }, { 0 } },
	{ X86_PUSHGS32, X86_INS_PUSH, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_PUSHGS64, X86_INS_PUSH, { 0 }, { 0 }, { 0 } },
	{ X86_PUSHSS16, X86_INS_PUSH, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_PUSHSS32, X86_INS_PUSH, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_PUSHi16, X86_INS_PUSH, { X86_REG_ESP, 0 }, { X86_REG_ESP, 0 }, { 0 } },
	{ X86_PUSHi32, X86_INS_PUSH, { X86_REG_ESP, 0 }, { X86_REG_ESP, 0 }, { 0 } },
	{ X86_PUSHi8, X86_INS_PUSH, { X86_REG_ESP, 0 }, { X86_REG_ESP, 0 }, { 0 } },
	{ X86_PXORrm, X86_INS_PXOR, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_PXORrr, X86_INS_PXOR, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_RCL16m1, X86_INS_RCL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL16mCL, X86_INS_RCL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL16mi, X86_INS_RCL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL16r1, X86_INS_RCL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL16rCL, X86_INS_RCL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL16ri, X86_INS_RCL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL32m1, X86_INS_RCL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL32mCL, X86_INS_RCL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL32mi, X86_INS_RCL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL32r1, X86_INS_RCL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL32rCL, X86_INS_RCL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL32ri, X86_INS_RCL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL64m1, X86_INS_RCL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL64mCL, X86_INS_RCL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL64mi, X86_INS_RCL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL64r1, X86_INS_RCL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL64rCL, X86_INS_RCL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL64ri, X86_INS_RCL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL8m1, X86_INS_RCL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL8mCL, X86_INS_RCL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL8mi, X86_INS_RCL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL8r1, X86_INS_RCL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL8rCL, X86_INS_RCL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCL8ri, X86_INS_RCL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCPPSm, X86_INS_RCPPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_RCPPSm_Int, X86_INS_RCPPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_RCPPSr, X86_INS_RCPPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_RCPPSr_Int, X86_INS_RCPPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_RCPSSm, X86_INS_RCPSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_RCPSSm_Int, X86_INS_RCPSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_RCPSSr, X86_INS_RCPSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_RCPSSr_Int, X86_INS_RCPSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_RCR16m1, X86_INS_RCR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR16mCL, X86_INS_RCR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR16mi, X86_INS_RCR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR16r1, X86_INS_RCR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR16rCL, X86_INS_RCR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR16ri, X86_INS_RCR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR32m1, X86_INS_RCR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR32mCL, X86_INS_RCR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR32mi, X86_INS_RCR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR32r1, X86_INS_RCR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR32rCL, X86_INS_RCR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR32ri, X86_INS_RCR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR64m1, X86_INS_RCR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR64mCL, X86_INS_RCR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR64mi, X86_INS_RCR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR64r1, X86_INS_RCR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR64rCL, X86_INS_RCR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR64ri, X86_INS_RCR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR8m1, X86_INS_RCR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR8mCL, X86_INS_RCR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR8mi, X86_INS_RCR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR8r1, X86_INS_RCR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR8rCL, X86_INS_RCR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RCR8ri, X86_INS_RCR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RDFSBASE, X86_INS_RDFSBASE, { 0 }, { 0 }, { X86_GRP_FSGSBASE, X86_GRP_MODE64, 0 } },
	{ X86_RDFSBASE64, X86_INS_RDFSBASE, { 0 }, { 0 }, { X86_GRP_FSGSBASE, X86_GRP_MODE64, 0 } },
	{ X86_RDGSBASE, X86_INS_RDGSBASE, { 0 }, { 0 }, { X86_GRP_FSGSBASE, X86_GRP_MODE64, 0 } },
	{ X86_RDGSBASE64, X86_INS_RDGSBASE, { 0 }, { 0 }, { X86_GRP_FSGSBASE, X86_GRP_MODE64, 0 } },
	{ X86_RDMSR, X86_INS_RDMSR, { 0 }, { 0 }, { 0 } },
	{ X86_RDPMC, X86_INS_RDPMC, { 0 }, { 0 }, { 0 } },
	{ X86_RDRAND16r, X86_INS_RDRAND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RDRAND32r, X86_INS_RDRAND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RDRAND64r, X86_INS_RDRAND, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RDSEED16r, X86_INS_RDSEED, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RDSEED32r, X86_INS_RDSEED, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RDSEED64r, X86_INS_RDSEED, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RDTSC, X86_INS_RDTSC, { 0 }, { X86_REG_RAX, X86_REG_RDX, 0 }, { 0 } },
	{ X86_RDTSCP, X86_INS_RDTSCP, { 0 }, { X86_REG_RAX, X86_REG_RCX, X86_REG_RDX, 0 }, { 0 } },
	{ X86_REPNE_PREFIX, X86_INS_REPNE, { X86_REG_ECX, X86_REG_FLAGS, 0 }, { X86_REG_ECX, 0 }, { 0 } },
	{ X86_REP_MOVSB_32, X86_INS_REP, { X86_REG_ECX, X86_REG_EDI, X86_REG_ESI, 0 }, { X86_REG_ECX, X86_REG_EDI, X86_REG_ESI, 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_REP_MOVSB_64, X86_INS_REP, { X86_REG_RCX, X86_REG_RDI, X86_REG_RSI, 0 }, { X86_REG_RCX, X86_REG_RDI, X86_REG_RSI, 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_REP_MOVSD_32, X86_INS_REP, { X86_REG_ECX, X86_REG_EDI, X86_REG_ESI, 0 }, { X86_REG_ECX, X86_REG_EDI, X86_REG_ESI, 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_REP_MOVSD_64, X86_INS_REP, { X86_REG_RCX, X86_REG_RDI, X86_REG_RSI, 0 }, { X86_REG_RCX, X86_REG_RDI, X86_REG_RSI, 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_REP_MOVSQ_64, X86_INS_REP, { X86_REG_RCX, X86_REG_RDI, X86_REG_RSI, 0 }, { X86_REG_RCX, X86_REG_RDI, X86_REG_RSI, 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_REP_MOVSW_32, X86_INS_REP, { X86_REG_ECX, X86_REG_EDI, X86_REG_ESI, 0 }, { X86_REG_ECX, X86_REG_EDI, X86_REG_ESI, 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_REP_MOVSW_64, X86_INS_REP, { X86_REG_RCX, X86_REG_RDI, X86_REG_RSI, 0 }, { X86_REG_RCX, X86_REG_RDI, X86_REG_RSI, 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_REP_PREFIX, X86_INS_REP, { X86_REG_ECX, X86_REG_FLAGS, 0 }, { X86_REG_ECX, 0 }, { 0 } },
	{ X86_REP_STOSB_32, X86_INS_REP, { X86_REG_AL, X86_REG_ECX, X86_REG_EDI, 0 }, { X86_REG_ECX, X86_REG_EDI, 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_REP_STOSB_64, X86_INS_REP, { X86_REG_AL, X86_REG_RCX, X86_REG_RDI, 0 }, { X86_REG_RCX, X86_REG_RDI, 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_REP_STOSD_32, X86_INS_REP, { X86_REG_EAX, X86_REG_ECX, X86_REG_EDI, 0 }, { X86_REG_ECX, X86_REG_EDI, 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_REP_STOSD_64, X86_INS_REP, { X86_REG_RAX, X86_REG_RCX, X86_REG_RDI, 0 }, { X86_REG_RCX, X86_REG_RDI, 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_REP_STOSQ_64, X86_INS_REP, { X86_REG_RAX, X86_REG_RCX, X86_REG_RDI, 0 }, { X86_REG_RCX, X86_REG_RDI, 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_REP_STOSW_32, X86_INS_REP, { X86_REG_AX, X86_REG_ECX, X86_REG_EDI, 0 }, { X86_REG_ECX, X86_REG_EDI, 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_REP_STOSW_64, X86_INS_REP, { X86_REG_AX, X86_REG_RCX, X86_REG_RDI, 0 }, { X86_REG_RCX, X86_REG_RDI, 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_RET, X86_INS_RET, { 0 }, { 0 }, { 0 } },
	{ X86_RETI, X86_INS_RET, { 0 }, { 0 }, { 0 } },
	{ X86_RETIW, X86_INS_RET, { 0 }, { 0 }, { 0 } },
	{ X86_RETW, X86_INS_RET, { 0 }, { 0 }, { 0 } },
	{ X86_REX64_PREFIX, X86_INS_REX64, { 0 }, { 0 }, { 0 } },
	{ X86_ROL16m1, X86_INS_ROL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL16mCL, X86_INS_ROL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL16mi, X86_INS_ROL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL16r1, X86_INS_ROL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL16rCL, X86_INS_ROL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL16ri, X86_INS_ROL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL32m1, X86_INS_ROL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL32mCL, X86_INS_ROL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL32mi, X86_INS_ROL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL32r1, X86_INS_ROL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL32rCL, X86_INS_ROL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL32ri, X86_INS_ROL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL64m1, X86_INS_ROL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL64mCL, X86_INS_ROL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL64mi, X86_INS_ROL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL64r1, X86_INS_ROL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL64rCL, X86_INS_ROL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL64ri, X86_INS_ROL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL8m1, X86_INS_ROL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL8mCL, X86_INS_ROL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL8mi, X86_INS_ROL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL8r1, X86_INS_ROL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL8rCL, X86_INS_ROL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROL8ri, X86_INS_ROL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR16m1, X86_INS_ROR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR16mCL, X86_INS_ROR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR16mi, X86_INS_ROR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR16r1, X86_INS_ROR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR16rCL, X86_INS_ROR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR16ri, X86_INS_ROR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR32m1, X86_INS_ROR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR32mCL, X86_INS_ROR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR32mi, X86_INS_ROR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR32r1, X86_INS_ROR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR32rCL, X86_INS_ROR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR32ri, X86_INS_ROR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR64m1, X86_INS_ROR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR64mCL, X86_INS_ROR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR64mi, X86_INS_ROR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR64r1, X86_INS_ROR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR64rCL, X86_INS_ROR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR64ri, X86_INS_ROR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR8m1, X86_INS_ROR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR8mCL, X86_INS_ROR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR8mi, X86_INS_ROR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR8r1, X86_INS_ROR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR8rCL, X86_INS_ROR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_ROR8ri, X86_INS_ROR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_RORX32mi, X86_INS_RORX, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_RORX32ri, X86_INS_RORX, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_RORX64mi, X86_INS_RORX, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_RORX64ri, X86_INS_RORX, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_ROUNDPDm, X86_INS_ROUNDPD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_ROUNDPDr, X86_INS_ROUNDPD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_ROUNDPSm, X86_INS_ROUNDPS, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_ROUNDPSr, X86_INS_ROUNDPS, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_ROUNDSDm, X86_INS_ROUNDSD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_ROUNDSDr, X86_INS_ROUNDSD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_ROUNDSDr_Int, X86_INS_ROUNDSD, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_ROUNDSSm, X86_INS_ROUNDSS, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_ROUNDSSr, X86_INS_ROUNDSS, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_ROUNDSSr_Int, X86_INS_ROUNDSS, { 0 }, { 0 }, { X86_GRP_SSE41, 0 } },
	{ X86_RSM, X86_INS_RSM, { 0 }, { 0 }, { 0 } },
	{ X86_RSQRTPSm, X86_INS_RSQRTPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_RSQRTPSm_Int, X86_INS_RSQRTPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_RSQRTPSr, X86_INS_RSQRTPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_RSQRTPSr_Int, X86_INS_RSQRTPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_RSQRTSSm, X86_INS_RSQRTSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_RSQRTSSm_Int, X86_INS_RSQRTSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_RSQRTSSr, X86_INS_RSQRTSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_RSQRTSSr_Int, X86_INS_RSQRTSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_SAHF, X86_INS_SAHF, { X86_REG_AH, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR16m1, X86_INS_SAR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR16mCL, X86_INS_SAR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR16mi, X86_INS_SAR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR16r1, X86_INS_SAR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR16rCL, X86_INS_SAR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR16ri, X86_INS_SAR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR32m1, X86_INS_SAR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR32mCL, X86_INS_SAR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR32mi, X86_INS_SAR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR32r1, X86_INS_SAR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR32rCL, X86_INS_SAR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR32ri, X86_INS_SAR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR64m1, X86_INS_SAR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR64mCL, X86_INS_SAR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR64mi, X86_INS_SAR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR64r1, X86_INS_SAR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR64rCL, X86_INS_SAR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR64ri, X86_INS_SAR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR8m1, X86_INS_SAR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR8mCL, X86_INS_SAR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR8mi, X86_INS_SAR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR8r1, X86_INS_SAR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR8rCL, X86_INS_SAR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SAR8ri, X86_INS_SAR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SARX32rm, X86_INS_SARX, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_SARX32rr, X86_INS_SARX, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_SARX64rm, X86_INS_SARX, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_SARX64rr, X86_INS_SARX, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_SBB16i16, X86_INS_SBB, { X86_REG_AX, X86_REG_FLAGS, 0 }, { X86_REG_AX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB16mi, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB16mi8, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB16mr, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB16ri, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB16ri8, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB16rm, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB16rr, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB16rr_REV, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB32i32, X86_INS_SBB, { X86_REG_EAX, X86_REG_FLAGS, 0 }, { X86_REG_EAX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB32mi, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB32mi8, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB32mr, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB32ri, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB32ri8, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB32rm, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB32rr, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB32rr_REV, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB64i32, X86_INS_SBB, { X86_REG_RAX, X86_REG_FLAGS, 0 }, { X86_REG_RAX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB64mi32, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB64mi8, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB64mr, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB64ri32, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB64ri8, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB64rm, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB64rr, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB64rr_REV, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB8i8, X86_INS_SBB, { X86_REG_AL, X86_REG_FLAGS, 0 }, { X86_REG_AL, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB8mi, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB8mr, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB8ri, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB8rm, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB8rr, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SBB8rr_REV, X86_INS_SBB, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SCAS16, X86_INS_SCASW, { 0 }, { 0 }, { 0 } },
	{ X86_SCAS32, X86_INS_SCASD, { 0 }, { 0 }, { 0 } },
	{ X86_SCAS64, X86_INS_SCASQ, { 0 }, { 0 }, { 0 } },
	{ X86_SCAS8, X86_INS_SCASB, { 0 }, { 0 }, { 0 } },
	{ X86_SETAEm, X86_INS_SETAE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETAEr, X86_INS_SETAE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETAm, X86_INS_SETA, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETAr, X86_INS_SETA, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETBEm, X86_INS_SETBE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETBEr, X86_INS_SETBE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETBm, X86_INS_SETB, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETBr, X86_INS_SETB, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETEm, X86_INS_SETE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETEr, X86_INS_SETE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETGEm, X86_INS_SETGE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETGEr, X86_INS_SETGE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETGm, X86_INS_SETG, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETGr, X86_INS_SETG, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETLEm, X86_INS_SETLE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETLEr, X86_INS_SETLE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETLm, X86_INS_SETL, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETLr, X86_INS_SETL, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETNEm, X86_INS_SETNE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETNEr, X86_INS_SETNE, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETNOm, X86_INS_SETNO, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETNOr, X86_INS_SETNO, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETNPm, X86_INS_SETNP, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETNPr, X86_INS_SETNP, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETNSm, X86_INS_SETNS, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETNSr, X86_INS_SETNS, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETOm, X86_INS_SETO, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETOr, X86_INS_SETO, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETPm, X86_INS_SETP, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETPr, X86_INS_SETP, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETSm, X86_INS_SETS, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SETSr, X86_INS_SETS, { X86_REG_FLAGS, 0 }, { 0 }, { 0 } },
	{ X86_SFENCE, X86_INS_SFENCE, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_SGDT16m, X86_INS_SGDT, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_SGDTm, X86_INS_SGDT, { 0 }, { 0 }, { 0 } },
	{ X86_SHA1MSG1rm, X86_INS_SHA1MSG1, { 0 }, { 0 }, { X86_GRP_SHA, 0 } },
	{ X86_SHA1MSG1rr, X86_INS_SHA1MSG1, { 0 }, { 0 }, { X86_GRP_SHA, 0 } },
	{ X86_SHA1MSG2rm, X86_INS_SHA1MSG2, { 0 }, { 0 }, { X86_GRP_SHA, 0 } },
	{ X86_SHA1MSG2rr, X86_INS_SHA1MSG2, { 0 }, { 0 }, { X86_GRP_SHA, 0 } },
	{ X86_SHA1NEXTErm, X86_INS_SHA1NEXTE, { 0 }, { 0 }, { X86_GRP_SHA, 0 } },
	{ X86_SHA1NEXTErr, X86_INS_SHA1NEXTE, { 0 }, { 0 }, { X86_GRP_SHA, 0 } },
	{ X86_SHA1RNDS4rmi, X86_INS_SHA1RNDS4, { 0 }, { 0 }, { X86_GRP_SHA, 0 } },
	{ X86_SHA1RNDS4rri, X86_INS_SHA1RNDS4, { 0 }, { 0 }, { X86_GRP_SHA, 0 } },
	{ X86_SHA256MSG1rm, X86_INS_SHA256MSG1, { 0 }, { 0 }, { X86_GRP_SHA, 0 } },
	{ X86_SHA256MSG1rr, X86_INS_SHA256MSG1, { 0 }, { 0 }, { X86_GRP_SHA, 0 } },
	{ X86_SHA256MSG2rm, X86_INS_SHA256MSG2, { 0 }, { 0 }, { X86_GRP_SHA, 0 } },
	{ X86_SHA256MSG2rr, X86_INS_SHA256MSG2, { 0 }, { 0 }, { X86_GRP_SHA, 0 } },
	{ X86_SHA256RNDS2rm, X86_INS_SHA256RNDS2, { X86_REG_XMM0, 0 }, { 0 }, { X86_GRP_SHA, 0 } },
	{ X86_SHA256RNDS2rr, X86_INS_SHA256RNDS2, { X86_REG_XMM0, 0 }, { 0 }, { X86_GRP_SHA, 0 } },
	{ X86_SHL16m1, X86_INS_SHL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL16mCL, X86_INS_SHL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL16mi, X86_INS_SHL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL16r1, X86_INS_SHL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL16rCL, X86_INS_SHL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL16ri, X86_INS_SHL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL32m1, X86_INS_SHL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL32mCL, X86_INS_SHL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL32mi, X86_INS_SHL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL32r1, X86_INS_SHL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL32rCL, X86_INS_SHL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL32ri, X86_INS_SHL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL64m1, X86_INS_SHL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL64mCL, X86_INS_SHL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL64mi, X86_INS_SHL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL64r1, X86_INS_SHL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL64rCL, X86_INS_SHL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL64ri, X86_INS_SHL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL8m1, X86_INS_SHL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL8mCL, X86_INS_SHL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL8mi, X86_INS_SHL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL8r1, X86_INS_SHL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL8rCL, X86_INS_SHL, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHL8ri, X86_INS_SHL, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHLD16mrCL, X86_INS_SHLD, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHLD16mri8, X86_INS_SHLD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHLD16rrCL, X86_INS_SHLD, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHLD16rri8, X86_INS_SHLD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHLD32mrCL, X86_INS_SHLD, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHLD32mri8, X86_INS_SHLD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHLD32rrCL, X86_INS_SHLD, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHLD32rri8, X86_INS_SHLD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHLD64mrCL, X86_INS_SHLD, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHLD64mri8, X86_INS_SHLD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHLD64rrCL, X86_INS_SHLD, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHLD64rri8, X86_INS_SHLD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHLX32rm, X86_INS_SHLX, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_SHLX32rr, X86_INS_SHLX, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_SHLX64rm, X86_INS_SHLX, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_SHLX64rr, X86_INS_SHLX, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_SHR16m1, X86_INS_SHR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR16mCL, X86_INS_SHR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR16mi, X86_INS_SHR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR16r1, X86_INS_SHR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR16rCL, X86_INS_SHR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR16ri, X86_INS_SHR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR32m1, X86_INS_SHR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR32mCL, X86_INS_SHR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR32mi, X86_INS_SHR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR32r1, X86_INS_SHR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR32rCL, X86_INS_SHR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR32ri, X86_INS_SHR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR64m1, X86_INS_SHR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR64mCL, X86_INS_SHR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR64mi, X86_INS_SHR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR64r1, X86_INS_SHR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR64rCL, X86_INS_SHR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR64ri, X86_INS_SHR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR8m1, X86_INS_SHR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR8mCL, X86_INS_SHR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR8mi, X86_INS_SHR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR8r1, X86_INS_SHR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR8rCL, X86_INS_SHR, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHR8ri, X86_INS_SHR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHRD16mrCL, X86_INS_SHRD, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHRD16mri8, X86_INS_SHRD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHRD16rrCL, X86_INS_SHRD, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHRD16rri8, X86_INS_SHRD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHRD32mrCL, X86_INS_SHRD, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHRD32mri8, X86_INS_SHRD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHRD32rrCL, X86_INS_SHRD, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHRD32rri8, X86_INS_SHRD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHRD64mrCL, X86_INS_SHRD, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHRD64mri8, X86_INS_SHRD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHRD64rrCL, X86_INS_SHRD, { X86_REG_CL, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHRD64rri8, X86_INS_SHRD, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SHRX32rm, X86_INS_SHRX, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_SHRX32rr, X86_INS_SHRX, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_SHRX64rm, X86_INS_SHRX, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_SHRX64rr, X86_INS_SHRX, { 0 }, { 0 }, { X86_GRP_BMI2, 0 } },
	{ X86_SHUFPDrmi, X86_INS_SHUFPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_SHUFPDrri, X86_INS_SHUFPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_SHUFPSrmi, X86_INS_SHUFPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_SHUFPSrri, X86_INS_SHUFPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_SIDT16m, X86_INS_SIDT, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_SIDTm, X86_INS_SIDT, { 0 }, { 0 }, { 0 } },
	{ X86_SIN_F, X86_INS_FSIN, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_SKINIT, X86_INS_SKINIT, { X86_REG_EAX, 0 }, { 0 }, { 0 } },
	{ X86_SLDT16m, X86_INS_SLDT, { 0 }, { 0 }, { 0 } },
	{ X86_SLDT16r, X86_INS_SLDT, { 0 }, { 0 }, { 0 } },
	{ X86_SLDT32r, X86_INS_SLDT, { 0 }, { 0 }, { 0 } },
	{ X86_SLDT64m, X86_INS_SLDT, { 0 }, { 0 }, { 0 } },
	{ X86_SLDT64r, X86_INS_SLDT, { 0 }, { 0 }, { 0 } },
	{ X86_SMSW16m, X86_INS_SMSW, { 0 }, { 0 }, { 0 } },
	{ X86_SMSW16r, X86_INS_SMSW, { 0 }, { 0 }, { 0 } },
	{ X86_SMSW32r, X86_INS_SMSW, { 0 }, { 0 }, { 0 } },
	{ X86_SMSW64r, X86_INS_SMSW, { 0 }, { 0 }, { 0 } },
	{ X86_SQRTPDm, X86_INS_SQRTPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_SQRTPDr, X86_INS_SQRTPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_SQRTPSm, X86_INS_SQRTPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_SQRTPSr, X86_INS_SQRTPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_SQRTSDm, X86_INS_SQRTSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_SQRTSDm_Int, X86_INS_SQRTSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_SQRTSDr, X86_INS_SQRTSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_SQRTSDr_Int, X86_INS_SQRTSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_SQRTSSm, X86_INS_SQRTSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_SQRTSSm_Int, X86_INS_SQRTSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_SQRTSSr, X86_INS_SQRTSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_SQRTSSr_Int, X86_INS_SQRTSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_SQRT_F, X86_INS_FSQRT, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_SS_PREFIX, X86_INS_SS, { 0 }, { 0 }, { 0 } },
	{ X86_STAC, X86_INS_STAC, { X86_REG_FLAGS, 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_STC, X86_INS_STC, { 0 }, { 0 }, { 0 } },
	{ X86_STD, X86_INS_STD, { 0 }, { 0 }, { 0 } },
	{ X86_STGI, X86_INS_STGI, { 0 }, { 0 }, { 0 } },
	{ X86_STI, X86_INS_STI, { 0 }, { 0 }, { 0 } },
	{ X86_STMXCSR, X86_INS_STMXCSR, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_STOSB, X86_INS_STOSB, { X86_REG_AL, X86_REG_EDI, X86_REG_FLAGS, 0 }, { X86_REG_EDI, 0 }, { 0 } },
	{ X86_STOSD, X86_INS_STOSD, { X86_REG_EAX, X86_REG_EDI, X86_REG_FLAGS, 0 }, { X86_REG_EDI, 0 }, { 0 } },
	{ X86_STOSQ, X86_INS_STOSQ, { X86_REG_RAX, X86_REG_RCX, X86_REG_RDI, X86_REG_FLAGS, 0 }, { X86_REG_RCX, X86_REG_RDI, 0 }, { 0 } },
	{ X86_STOSW, X86_INS_STOSW, { X86_REG_AX, X86_REG_EDI, X86_REG_FLAGS, 0 }, { X86_REG_EDI, 0 }, { 0 } },
	{ X86_STR16r, X86_INS_STR, { 0 }, { 0 }, { 0 } },
	{ X86_STR32r, X86_INS_STR, { 0 }, { 0 }, { 0 } },
	{ X86_STR64r, X86_INS_STR, { 0 }, { 0 }, { 0 } },
	{ X86_STRm, X86_INS_STR, { 0 }, { 0 }, { 0 } },
	{ X86_ST_F32m, X86_INS_FST, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_ST_F64m, X86_INS_FST, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_ST_FP32m, X86_INS_FSTP, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_ST_FP64m, X86_INS_FSTP, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_ST_FP80m, X86_INS_FSTP, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_ST_FPrr, X86_INS_FSTP, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_ST_Frr, X86_INS_FST, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_SUB16i16, X86_INS_SUB, { X86_REG_AX, 0 }, { X86_REG_AX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB16mi, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB16mi8, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB16mr, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB16ri, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB16ri8, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB16rm, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB16rr, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB16rr_REV, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB32i32, X86_INS_SUB, { X86_REG_EAX, 0 }, { X86_REG_EAX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB32mi, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB32mi8, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB32mr, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB32ri, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB32ri8, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB32rm, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB32rr, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB32rr_REV, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB64i32, X86_INS_SUB, { X86_REG_RAX, 0 }, { X86_REG_RAX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB64mi32, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB64mi8, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB64mr, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB64ri32, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB64ri8, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB64rm, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB64rr, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB64rr_REV, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB8i8, X86_INS_SUB, { X86_REG_AL, 0 }, { X86_REG_AL, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB8mi, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB8mr, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB8ri, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB8rm, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB8rr, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUB8rr_REV, X86_INS_SUB, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_SUBPDrm, X86_INS_SUBPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_SUBPDrr, X86_INS_SUBPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_SUBPSrm, X86_INS_SUBPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_SUBPSrr, X86_INS_SUBPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_SUBR_F32m, X86_INS_FSUBR, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_SUBR_F64m, X86_INS_FSUBR, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_SUBR_FI16m, X86_INS_FISUBR, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_SUBR_FI32m, X86_INS_FISUBR, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_SUBR_FPrST0, X86_INS_FSUBRP, { 0 }, { 0 }, { 0 } },
	{ X86_SUBR_FST0r, X86_INS_FSUBR, { 0 }, { 0 }, { 0 } },
	{ X86_SUBR_FrST0, X86_INS_FSUBR, { 0 }, { 0 }, { 0 } },
	{ X86_SUBSDrm, X86_INS_SUBSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_SUBSDrm_Int, X86_INS_SUBSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_SUBSDrr, X86_INS_SUBSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_SUBSDrr_Int, X86_INS_SUBSD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_SUBSSrm, X86_INS_SUBSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_SUBSSrm_Int, X86_INS_SUBSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_SUBSSrr, X86_INS_SUBSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_SUBSSrr_Int, X86_INS_SUBSS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_SUB_F32m, X86_INS_FSUB, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_SUB_F64m, X86_INS_FSUB, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_SUB_FI16m, X86_INS_FISUB, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_SUB_FI32m, X86_INS_FISUB, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_SUB_FPrST0, X86_INS_FSUBP, { 0 }, { 0 }, { 0 } },
	{ X86_SUB_FST0r, X86_INS_FSUB, { 0 }, { 0 }, { 0 } },
	{ X86_SUB_FrST0, X86_INS_FSUB, { 0 }, { 0 }, { 0 } },
	{ X86_SWAPGS, X86_INS_SWAPGS, { 0 }, { 0 }, { 0 } },
	{ X86_SYSCALL, X86_INS_SYSCALL, { 0 }, { 0 }, { 0 } },
	{ X86_SYSENTER, X86_INS_SYSENTER, { 0 }, { 0 }, { 0 } },
	{ X86_SYSEXIT, X86_INS_SYSEXIT, { 0 }, { 0 }, { 0 } },
	{ X86_SYSEXIT64, X86_INS_SYSEXIT, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_SYSRET, X86_INS_SYSRET, { 0 }, { 0 }, { 0 } },
	{ X86_SYSRET64, X86_INS_SYSRET, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_TEST16i16, X86_INS_TEST, { X86_REG_AX, 0 }, { X86_REG_AX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_TEST16mi, X86_INS_TEST, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_TEST16ri, X86_INS_TEST, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_TEST16rm, X86_INS_TEST, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_TEST16rr, X86_INS_TEST, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_TEST32i32, X86_INS_TEST, { X86_REG_EAX, 0 }, { X86_REG_EAX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_TEST32mi, X86_INS_TEST, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_TEST32ri, X86_INS_TEST, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_TEST32rm, X86_INS_TEST, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_TEST32rr, X86_INS_TEST, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_TEST64i32, X86_INS_TEST, { X86_REG_RAX, 0 }, { X86_REG_RAX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_TEST64mi32, X86_INS_TEST, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_TEST64ri32, X86_INS_TEST, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_TEST64rm, X86_INS_TEST, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_TEST64rr, X86_INS_TEST, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_TEST8i8, X86_INS_TEST, { X86_REG_AL, 0 }, { X86_REG_AL, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_TEST8mi, X86_INS_TEST, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_TEST8ri, X86_INS_TEST, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_TEST8rm, X86_INS_TEST, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_TEST8rr, X86_INS_TEST, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_TRAP, X86_INS_UD2, { 0 }, { 0 }, { 0 } },
	{ X86_TST_F, X86_INS_FTST, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_TZCNT16rm, X86_INS_TZCNT, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_TZCNT16rr, X86_INS_TZCNT, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_TZCNT32rm, X86_INS_TZCNT, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_TZCNT32rr, X86_INS_TZCNT, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_TZCNT64rm, X86_INS_TZCNT, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_TZCNT64rr, X86_INS_TZCNT, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_BMI, 0 } },
	{ X86_UCOMISDrm, X86_INS_UCOMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_UCOMISDrr, X86_INS_UCOMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_UCOMISSrm, X86_INS_UCOMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_UCOMISSrr, X86_INS_UCOMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_UCOM_FIPr, X86_INS_FUCOMPI, { X86_REG_ST0, 0 }, { X86_REG_FLAGS, X86_REG_FPSW, 0 }, { 0 } },
	{ X86_UCOM_FIr, X86_INS_FUCOMI, { X86_REG_ST0, 0 }, { X86_REG_FLAGS, X86_REG_FPSW, 0 }, { 0 } },
	{ X86_UCOM_FPPr, X86_INS_FUCOMPP, { X86_REG_ST0, 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_UCOM_FPr, X86_INS_FUCOMP, { X86_REG_ST0, 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_UCOM_Fr, X86_INS_FUCOM, { X86_REG_ST0, 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_UD2B, X86_INS_UD2B, { 0 }, { 0 }, { 0 } },
	{ X86_UNPCKHPDrm, X86_INS_UNPCKHPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_UNPCKHPDrr, X86_INS_UNPCKHPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_UNPCKHPSrm, X86_INS_UNPCKHPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_UNPCKHPSrr, X86_INS_UNPCKHPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_UNPCKLPDrm, X86_INS_UNPCKLPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_UNPCKLPDrr, X86_INS_UNPCKLPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_UNPCKLPSrm, X86_INS_UNPCKLPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_UNPCKLPSrr, X86_INS_UNPCKLPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_VADDPDYrm, X86_INS_VADDPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDPDYrr, X86_INS_VADDPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDPDZrm, X86_INS_VADDPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VADDPDZrmb, X86_INS_VADDPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VADDPDZrr, X86_INS_VADDPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VADDPDrm, X86_INS_VADDPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDPDrr, X86_INS_VADDPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDPSYrm, X86_INS_VADDPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDPSYrr, X86_INS_VADDPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDPSZrm, X86_INS_VADDPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VADDPSZrmb, X86_INS_VADDPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VADDPSZrr, X86_INS_VADDPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VADDPSrm, X86_INS_VADDPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDPSrr, X86_INS_VADDPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDSDZrm, X86_INS_VADDSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VADDSDZrr, X86_INS_VADDSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VADDSDrm, X86_INS_VADDSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDSDrm_Int, X86_INS_VADDSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDSDrr, X86_INS_VADDSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDSDrr_Int, X86_INS_VADDSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDSSZrm, X86_INS_VADDSS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VADDSSZrr, X86_INS_VADDSS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VADDSSrm, X86_INS_VADDSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDSSrm_Int, X86_INS_VADDSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDSSrr, X86_INS_VADDSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDSSrr_Int, X86_INS_VADDSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDSUBPDYrm, X86_INS_VADDSUBPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDSUBPDYrr, X86_INS_VADDSUBPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDSUBPDrm, X86_INS_VADDSUBPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDSUBPDrr, X86_INS_VADDSUBPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDSUBPSYrm, X86_INS_VADDSUBPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDSUBPSYrr, X86_INS_VADDSUBPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDSUBPSrm, X86_INS_VADDSUBPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VADDSUBPSrr, X86_INS_VADDSUBPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VAESDECLASTrm, X86_INS_VAESDECLAST, { 0 }, { 0 }, { X86_GRP_AVX, X86_GRP_AES, 0 } },
	{ X86_VAESDECLASTrr, X86_INS_VAESDECLAST, { 0 }, { 0 }, { X86_GRP_AVX, X86_GRP_AES, 0 } },
	{ X86_VAESDECrm, X86_INS_VAESDEC, { 0 }, { 0 }, { X86_GRP_AVX, X86_GRP_AES, 0 } },
	{ X86_VAESDECrr, X86_INS_VAESDEC, { 0 }, { 0 }, { X86_GRP_AVX, X86_GRP_AES, 0 } },
	{ X86_VAESENCLASTrm, X86_INS_VAESENCLAST, { 0 }, { 0 }, { X86_GRP_AVX, X86_GRP_AES, 0 } },
	{ X86_VAESENCLASTrr, X86_INS_VAESENCLAST, { 0 }, { 0 }, { X86_GRP_AVX, X86_GRP_AES, 0 } },
	{ X86_VAESENCrm, X86_INS_VAESENC, { 0 }, { 0 }, { X86_GRP_AVX, X86_GRP_AES, 0 } },
	{ X86_VAESENCrr, X86_INS_VAESENC, { 0 }, { 0 }, { X86_GRP_AVX, X86_GRP_AES, 0 } },
	{ X86_VAESIMCrm, X86_INS_VAESIMC, { 0 }, { 0 }, { X86_GRP_AVX, X86_GRP_AES, 0 } },
	{ X86_VAESIMCrr, X86_INS_VAESIMC, { 0 }, { 0 }, { X86_GRP_AVX, X86_GRP_AES, 0 } },
	{ X86_VAESKEYGENASSIST128rm, X86_INS_VAESKEYGENASSIST, { 0 }, { 0 }, { X86_GRP_AVX, X86_GRP_AES, 0 } },
	{ X86_VAESKEYGENASSIST128rr, X86_INS_VAESKEYGENASSIST, { 0 }, { 0 }, { X86_GRP_AVX, X86_GRP_AES, 0 } },
	{ X86_VALIGNDrmi, X86_INS_VALIGND, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VALIGNDrri, X86_INS_VALIGND, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VALIGNQrmi, X86_INS_VALIGNQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VALIGNQrri, X86_INS_VALIGNQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VANDNPDYrm, X86_INS_VANDNPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VANDNPDYrr, X86_INS_VANDNPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VANDNPDrm, X86_INS_VANDNPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VANDNPDrr, X86_INS_VANDNPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VANDNPSYrm, X86_INS_VANDNPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VANDNPSYrr, X86_INS_VANDNPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VANDNPSrm, X86_INS_VANDNPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VANDNPSrr, X86_INS_VANDNPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VANDPDYrm, X86_INS_VANDPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VANDPDYrr, X86_INS_VANDPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VANDPDrm, X86_INS_VANDPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VANDPDrr, X86_INS_VANDPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VANDPSYrm, X86_INS_VANDPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VANDPSYrr, X86_INS_VANDPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VANDPSrm, X86_INS_VANDPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VANDPSrr, X86_INS_VANDPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VBLENDMPDZrm, X86_INS_VBLENDMPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VBLENDMPDZrr, X86_INS_VBLENDMPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VBLENDMPSZrm, X86_INS_VBLENDMPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VBLENDMPSZrr, X86_INS_VBLENDMPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VBLENDPDYrmi, X86_INS_VBLENDPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VBLENDPDYrri, X86_INS_VBLENDPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VBLENDPDrmi, X86_INS_VBLENDPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VBLENDPDrri, X86_INS_VBLENDPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VBLENDPSYrmi, X86_INS_VBLENDPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VBLENDPSYrri, X86_INS_VBLENDPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VBLENDPSrmi, X86_INS_VBLENDPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VBLENDPSrri, X86_INS_VBLENDPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VBLENDVPDYrm, X86_INS_VBLENDVPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VBLENDVPDYrr, X86_INS_VBLENDVPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VBLENDVPDrm, X86_INS_VBLENDVPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VBLENDVPDrr, X86_INS_VBLENDVPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VBLENDVPSYrm, X86_INS_VBLENDVPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VBLENDVPSYrr, X86_INS_VBLENDVPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VBLENDVPSrm, X86_INS_VBLENDVPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VBLENDVPSrr, X86_INS_VBLENDVPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VBROADCASTF128, X86_INS_VBROADCASTF128, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VBROADCASTI128, X86_INS_VBROADCASTI128, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VBROADCASTSDYrm, X86_INS_VBROADCASTSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VBROADCASTSDYrr, X86_INS_VBROADCASTSD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VBROADCASTSDZrm, X86_INS_VBROADCASTSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VBROADCASTSDZrr, X86_INS_VBROADCASTSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VBROADCASTSSYrm, X86_INS_VBROADCASTSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VBROADCASTSSYrr, X86_INS_VBROADCASTSS, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VBROADCASTSSZrm, X86_INS_VBROADCASTSS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VBROADCASTSSZrr, X86_INS_VBROADCASTSS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VBROADCASTSSrm, X86_INS_VBROADCASTSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VBROADCASTSSrr, X86_INS_VBROADCASTSS, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VCMPPDYrmi, X86_INS_VCMPSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPPDYrmi_alt, X86_INS_VCMPPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPPDYrri, X86_INS_VCMPSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPPDYrri_alt, X86_INS_VCMPPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPPDZrmi, X86_INS_VCMPSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCMPPDZrmi_alt, X86_INS_VVCMPPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCMPPDZrri, X86_INS_VCMPSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCMPPDZrri_alt, X86_INS_VVCMPPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCMPPDrmi, X86_INS_VCMPSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPPDrmi_alt, X86_INS_VCMPPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPPDrri, X86_INS_VCMPSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPPDrri_alt, X86_INS_VCMPPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPPSYrmi, X86_INS_VCMPSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPPSYrmi_alt, X86_INS_VCMPPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPPSYrri, X86_INS_VCMPSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPPSYrri_alt, X86_INS_VCMPPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPPSZrmi, X86_INS_VCMPSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCMPPSZrmi_alt, X86_INS_VVCMPPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCMPPSZrri, X86_INS_VCMPSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCMPPSZrri_alt, X86_INS_VVCMPPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCMPPSrmi, X86_INS_VCMPSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPPSrmi_alt, X86_INS_VCMPPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPPSrri, X86_INS_VCMPSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPPSrri_alt, X86_INS_VCMPPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPSDrm, X86_INS_VCMPSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPSDrm_alt, X86_INS_VCMPSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPSDrr, X86_INS_VCMPSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPSDrr_alt, X86_INS_VCMPSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPSSrm, X86_INS_VCMPSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPSSrm_alt, X86_INS_VCMPSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPSSrr, X86_INS_VCMPSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCMPSSrr_alt, X86_INS_VCMPSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCOMISDZrm, X86_INS_VCOMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCOMISDZrr, X86_INS_VCOMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCOMISDrm, X86_INS_VCOMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCOMISDrr, X86_INS_VCOMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCOMISSZrm, X86_INS_VCOMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCOMISSZrr, X86_INS_VCOMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCOMISSrm, X86_INS_VCOMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCOMISSrr, X86_INS_VCOMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTDQ2PDYrm, X86_INS_VCVTDQ2PD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTDQ2PDYrr, X86_INS_VCVTDQ2PD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTDQ2PDZrm, X86_INS_VCVTDQ2PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTDQ2PDZrr, X86_INS_VCVTDQ2PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTDQ2PDrm, X86_INS_VCVTDQ2PD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTDQ2PDrr, X86_INS_VCVTDQ2PD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTDQ2PSYrm, X86_INS_VCVTDQ2PS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTDQ2PSYrr, X86_INS_VCVTDQ2PS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTDQ2PSZrm, X86_INS_VCVTDQ2PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTDQ2PSZrr, X86_INS_VCVTDQ2PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTDQ2PSrm, X86_INS_VCVTDQ2PS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTDQ2PSrr, X86_INS_VCVTDQ2PS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTPD2DQXrm, X86_INS_VCVTPD2DQX, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTPD2DQYrm, X86_INS_VCVTPD2DQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTPD2DQYrr, X86_INS_VCVTPD2DQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTPD2DQrr, X86_INS_VCVTPD2DQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTPD2PSXrm, X86_INS_VCVTPD2PSX, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTPD2PSYrm, X86_INS_VCVTPD2PS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTPD2PSYrr, X86_INS_VCVTPD2PS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTPD2PSZrm, X86_INS_VCVTPD2PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTPD2PSZrr, X86_INS_VCVTPD2PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTPD2PSrr, X86_INS_VCVTPD2PS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTPH2PSYrm, X86_INS_VCVTPH2PS, { 0 }, { 0 }, { X86_GRP_F16C, 0 } },
	{ X86_VCVTPH2PSYrr, X86_INS_VCVTPH2PS, { 0 }, { 0 }, { X86_GRP_F16C, 0 } },
	{ X86_VCVTPH2PSrm, X86_INS_VCVTPH2PS, { 0 }, { 0 }, { X86_GRP_F16C, 0 } },
	{ X86_VCVTPH2PSrr, X86_INS_VCVTPH2PS, { 0 }, { 0 }, { X86_GRP_F16C, 0 } },
	{ X86_VCVTPS2DQYrm, X86_INS_VCVTPS2DQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTPS2DQYrr, X86_INS_VCVTPS2DQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTPS2DQZrm, X86_INS_VCVTPS2DQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTPS2DQZrr, X86_INS_VCVTPS2DQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTPS2DQrm, X86_INS_VCVTPS2DQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTPS2DQrr, X86_INS_VCVTPS2DQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTPS2PDYrm, X86_INS_VCVTPS2PD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTPS2PDYrr, X86_INS_VCVTPS2PD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTPS2PDZrm, X86_INS_VCVTPS2PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTPS2PDZrr, X86_INS_VCVTPS2PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTPS2PDrm, X86_INS_VCVTPS2PD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTPS2PDrr, X86_INS_VCVTPS2PD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTPS2PHYmr, X86_INS_VCVTPS2PH, { 0 }, { 0 }, { X86_GRP_F16C, 0 } },
	{ X86_VCVTPS2PHYrr, X86_INS_VCVTPS2PH, { 0 }, { 0 }, { X86_GRP_F16C, 0 } },
	{ X86_VCVTPS2PHmr, X86_INS_VCVTPS2PH, { 0 }, { 0 }, { X86_GRP_F16C, 0 } },
	{ X86_VCVTPS2PHrr, X86_INS_VCVTPS2PH, { 0 }, { 0 }, { X86_GRP_F16C, 0 } },
	{ X86_VCVTSD2SI64rm, X86_INS_VCVTSD2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTSD2SI64rr, X86_INS_VCVTSD2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTSD2SIrm, X86_INS_VCVTSD2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTSD2SIrr, X86_INS_VCVTSD2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTSD2SSZrm, X86_INS_VCVTSD2SS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTSD2SSZrr, X86_INS_VCVTSD2SS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTSD2SSrm, X86_INS_VCVTSD2SS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTSD2SSrr, X86_INS_VCVTSD2SS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTSI2SD64Zrm, X86_INS_VCVTSI2SD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTSI2SD64Zrr, X86_INS_VCVTSI2SD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTSI2SD64rm, X86_INS_VCVTSI2SD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTSI2SD64rr, X86_INS_VCVTSI2SD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTSI2SDZrm, X86_INS_VCVTSI2SD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTSI2SDZrr, X86_INS_VCVTSI2SD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTSI2SDrm, X86_INS_VCVTSI2SD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTSI2SDrr, X86_INS_VCVTSI2SD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTSI2SS64Zrm, X86_INS_VCVTSI2SS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTSI2SS64Zrr, X86_INS_VCVTSI2SS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTSI2SS64rm, X86_INS_VCVTSI2SS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTSI2SS64rr, X86_INS_VCVTSI2SS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTSI2SSZrm, X86_INS_VCVTSI2SS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTSI2SSZrr, X86_INS_VCVTSI2SS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTSI2SSrm, X86_INS_VCVTSI2SS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTSI2SSrr, X86_INS_VCVTSI2SS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTSS2SDZrm, X86_INS_VCVTSS2SD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTSS2SDZrr, X86_INS_VCVTSS2SD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTSS2SDrm, X86_INS_VCVTSS2SD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTSS2SDrr, X86_INS_VCVTSS2SD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTSS2SI64rm, X86_INS_VCVTSS2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTSS2SI64rr, X86_INS_VCVTSS2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTSS2SIrm, X86_INS_VCVTSS2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTSS2SIrr, X86_INS_VCVTSS2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTTPD2DQXrm, X86_INS_VCVTTPD2DQX, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTTPD2DQYrm, X86_INS_VCVTTPD2DQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTTPD2DQYrr, X86_INS_VCVTTPD2DQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTTPD2DQZrm, X86_INS_VCVTTPD2DQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTTPD2DQZrr, X86_INS_VCVTTPD2DQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTTPD2DQrr, X86_INS_VCVTTPD2DQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTTPD2UDQZrm, X86_INS_VCVTTPD2UDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTTPD2UDQZrr, X86_INS_VCVTTPD2UDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTTPS2DQYrm, X86_INS_VCVTTPS2DQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTTPS2DQYrr, X86_INS_VCVTTPS2DQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTTPS2DQZrm, X86_INS_VCVTTPS2DQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTTPS2DQZrr, X86_INS_VCVTTPS2DQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTTPS2DQrm, X86_INS_VCVTTPS2DQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTTPS2DQrr, X86_INS_VCVTTPS2DQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTTPS2UDQZrm, X86_INS_VCVTTPS2UDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTTPS2UDQZrr, X86_INS_VCVTTPS2UDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTTSD2SI64rm, X86_INS_VCVTTSD2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTTSD2SI64rr, X86_INS_VCVTTSD2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTTSD2SIrm, X86_INS_VCVTTSD2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTTSD2SIrr, X86_INS_VCVTTSD2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTTSS2SI64rm, X86_INS_VCVTTSS2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTTSS2SI64rr, X86_INS_VCVTTSS2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTTSS2SIrm, X86_INS_VCVTTSS2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTTSS2SIrr, X86_INS_VCVTTSS2SI, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VCVTUDQ2PDZrm, X86_INS_VCVTUDQ2PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTUDQ2PDZrr, X86_INS_VCVTUDQ2PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTUDQ2PSZrm, X86_INS_VCVTUDQ2PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VCVTUDQ2PSZrr, X86_INS_VCVTUDQ2PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VDIVPDYrm, X86_INS_VDIVPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VDIVPDYrr, X86_INS_VDIVPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VDIVPDZrm, X86_INS_VDIVPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VDIVPDZrmb, X86_INS_VDIVPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VDIVPDZrr, X86_INS_VDIVPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VDIVPDrm, X86_INS_VDIVPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VDIVPDrr, X86_INS_VDIVPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VDIVPSYrm, X86_INS_VDIVPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VDIVPSYrr, X86_INS_VDIVPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VDIVPSZrm, X86_INS_VDIVPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VDIVPSZrmb, X86_INS_VDIVPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VDIVPSZrr, X86_INS_VDIVPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VDIVPSrm, X86_INS_VDIVPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VDIVPSrr, X86_INS_VDIVPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VDIVSDZrm, X86_INS_VDIVSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VDIVSDZrr, X86_INS_VDIVSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VDIVSDrm, X86_INS_VDIVSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VDIVSDrm_Int, X86_INS_VDIVSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VDIVSDrr, X86_INS_VDIVSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VDIVSDrr_Int, X86_INS_VDIVSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VDIVSSZrm, X86_INS_VDIVSS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VDIVSSZrr, X86_INS_VDIVSS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VDIVSSrm, X86_INS_VDIVSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VDIVSSrm_Int, X86_INS_VDIVSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VDIVSSrr, X86_INS_VDIVSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VDIVSSrr_Int, X86_INS_VDIVSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VDPPDrmi, X86_INS_VDPPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VDPPDrri, X86_INS_VDPPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VDPPSYrmi, X86_INS_VDPPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VDPPSYrri, X86_INS_VDPPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VDPPSrmi, X86_INS_VDPPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VDPPSrri, X86_INS_VDPPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VERRm, X86_INS_VERR, { 0 }, { 0 }, { 0 } },
	{ X86_VERRr, X86_INS_VERR, { 0 }, { 0 }, { 0 } },
	{ X86_VERWm, X86_INS_VERW, { 0 }, { 0 }, { 0 } },
	{ X86_VERWr, X86_INS_VERW, { 0 }, { 0 }, { 0 } },
	{ X86_VEXTRACTF128mr, X86_INS_VEXTRACTF128, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VEXTRACTF128rr, X86_INS_VEXTRACTF128, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VEXTRACTF32x4mr, X86_INS_VEXTRACTF32X4, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VEXTRACTF32x4rr, X86_INS_VEXTRACTF32X4, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VEXTRACTF64x4mr, X86_INS_VEXTRACTF64X4, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VEXTRACTF64x4rr, X86_INS_VEXTRACTF64X4, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VEXTRACTI128mr, X86_INS_VEXTRACTI128, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VEXTRACTI128rr, X86_INS_VEXTRACTI128, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VEXTRACTI32x4mr, X86_INS_VEXTRACTI32X4, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VEXTRACTI32x4rr, X86_INS_VEXTRACTI32X4, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VEXTRACTI64x4mr, X86_INS_VEXTRACTI64X4, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VEXTRACTI64x4rr, X86_INS_VEXTRACTI64X4, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VEXTRACTPSmr, X86_INS_VEXTRACTPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VEXTRACTPSrr, X86_INS_VEXTRACTPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VEXTRACTPSrr64, X86_INS_VEXTRACTPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VEXTRACTPSzmr, X86_INS_VEXTRACTPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VEXTRACTPSzrr, X86_INS_VEXTRACTPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADD132PDZm, X86_INS_VFMADD132PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADD132PDZmb, X86_INS_VFMADD132PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADD132PSZm, X86_INS_VFMADD132PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADD132PSZmb, X86_INS_VFMADD132PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADD213PDZm, X86_INS_VFMADD213PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADD213PDZmb, X86_INS_VFMADD213PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADD213PDZr, X86_INS_VFMADD213PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADD213PSZm, X86_INS_VFMADD213PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADD213PSZmb, X86_INS_VFMADD213PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADD213PSZr, X86_INS_VFMADD213PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADDPD4mr, X86_INS_VFMADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDPD4mrY, X86_INS_VFMADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDPD4rm, X86_INS_VFMADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDPD4rmY, X86_INS_VFMADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDPD4rr, X86_INS_VFMADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDPD4rrY, X86_INS_VFMADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDPD4rrY_REV, X86_INS_VFMADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDPD4rr_REV, X86_INS_VFMADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDPDr132m, X86_INS_VFMADD132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPDr132mY, X86_INS_VFMADD132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPDr132r, X86_INS_VFMADD132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPDr132rY, X86_INS_VFMADD132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPDr213m, X86_INS_VFMADD213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPDr213mY, X86_INS_VFMADD213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPDr213r, X86_INS_VFMADD213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPDr213rY, X86_INS_VFMADD213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPDr231m, X86_INS_VFMADD231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPDr231mY, X86_INS_VFMADD231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPDr231r, X86_INS_VFMADD231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPDr231rY, X86_INS_VFMADD231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPS4mr, X86_INS_VFMADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDPS4mrY, X86_INS_VFMADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDPS4rm, X86_INS_VFMADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDPS4rmY, X86_INS_VFMADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDPS4rr, X86_INS_VFMADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDPS4rrY, X86_INS_VFMADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDPS4rrY_REV, X86_INS_VFMADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDPS4rr_REV, X86_INS_VFMADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDPSr132m, X86_INS_VFMADD132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPSr132mY, X86_INS_VFMADD132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPSr132r, X86_INS_VFMADD132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPSr132rY, X86_INS_VFMADD132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPSr213m, X86_INS_VFMADD213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPSr213mY, X86_INS_VFMADD213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPSr213r, X86_INS_VFMADD213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPSr213rY, X86_INS_VFMADD213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPSr231m, X86_INS_VFMADD231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPSr231mY, X86_INS_VFMADD231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPSr231r, X86_INS_VFMADD231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDPSr231rY, X86_INS_VFMADD231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSD4mr, X86_INS_VFMADDSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSD4mr_Int, X86_INS_VFMADDSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSD4rm, X86_INS_VFMADDSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSD4rm_Int, X86_INS_VFMADDSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSD4rr, X86_INS_VFMADDSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSD4rr_Int, X86_INS_VFMADDSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSD4rr_REV, X86_INS_VFMADDSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSDZm, X86_INS_VFMADD213SD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADDSDZr, X86_INS_VFMADD213SD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADDSDr132m, X86_INS_VFMADD132SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSDr132r, X86_INS_VFMADD132SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSDr213m, X86_INS_VFMADD213SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSDr213m_Int, X86_INS_VFMADD213SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSDr213r, X86_INS_VFMADD213SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSDr213r_Int, X86_INS_VFMADD213SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSDr231m, X86_INS_VFMADD231SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSDr231r, X86_INS_VFMADD231SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSS4mr, X86_INS_VFMADDSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSS4mr_Int, X86_INS_VFMADDSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSS4rm, X86_INS_VFMADDSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSS4rm_Int, X86_INS_VFMADDSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSS4rr, X86_INS_VFMADDSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSS4rr_Int, X86_INS_VFMADDSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSS4rr_REV, X86_INS_VFMADDSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSSZm, X86_INS_VFMADD213SS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADDSSZr, X86_INS_VFMADD213SS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADDSSr132m, X86_INS_VFMADD132SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSSr132r, X86_INS_VFMADD132SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSSr213m, X86_INS_VFMADD213SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSSr213m_Int, X86_INS_VFMADD213SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSSr213r, X86_INS_VFMADD213SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSSr213r_Int, X86_INS_VFMADD213SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSSr231m, X86_INS_VFMADD231SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSSr231r, X86_INS_VFMADD231SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUB132PDZm, X86_INS_VFMADDSUB132PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADDSUB132PDZmb, X86_INS_VFMADDSUB132PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADDSUB132PSZm, X86_INS_VFMADDSUB132PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADDSUB132PSZmb, X86_INS_VFMADDSUB132PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADDSUB213PDZm, X86_INS_VFMADDSUB213PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADDSUB213PDZmb, X86_INS_VFMADDSUB213PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADDSUB213PDZr, X86_INS_VFMADDSUB213PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADDSUB213PSZm, X86_INS_VFMADDSUB213PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADDSUB213PSZmb, X86_INS_VFMADDSUB213PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADDSUB213PSZr, X86_INS_VFMADDSUB213PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMADDSUBPD4mr, X86_INS_VFMADDSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSUBPD4mrY, X86_INS_VFMADDSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSUBPD4rm, X86_INS_VFMADDSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSUBPD4rmY, X86_INS_VFMADDSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSUBPD4rr, X86_INS_VFMADDSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSUBPD4rrY, X86_INS_VFMADDSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSUBPD4rrY_REV, X86_INS_VFMADDSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSUBPD4rr_REV, X86_INS_VFMADDSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSUBPDr132m, X86_INS_VFMADDSUB132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPDr132mY, X86_INS_VFMADDSUB132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPDr132r, X86_INS_VFMADDSUB132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPDr132rY, X86_INS_VFMADDSUB132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPDr213m, X86_INS_VFMADDSUB213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPDr213mY, X86_INS_VFMADDSUB213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPDr213r, X86_INS_VFMADDSUB213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPDr213rY, X86_INS_VFMADDSUB213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPDr231m, X86_INS_VFMADDSUB231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPDr231mY, X86_INS_VFMADDSUB231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPDr231r, X86_INS_VFMADDSUB231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPDr231rY, X86_INS_VFMADDSUB231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPS4mr, X86_INS_VFMADDSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSUBPS4mrY, X86_INS_VFMADDSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSUBPS4rm, X86_INS_VFMADDSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSUBPS4rmY, X86_INS_VFMADDSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSUBPS4rr, X86_INS_VFMADDSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSUBPS4rrY, X86_INS_VFMADDSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSUBPS4rrY_REV, X86_INS_VFMADDSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSUBPS4rr_REV, X86_INS_VFMADDSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMADDSUBPSr132m, X86_INS_VFMADDSUB132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPSr132mY, X86_INS_VFMADDSUB132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPSr132r, X86_INS_VFMADDSUB132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPSr132rY, X86_INS_VFMADDSUB132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPSr213m, X86_INS_VFMADDSUB213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPSr213mY, X86_INS_VFMADDSUB213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPSr213r, X86_INS_VFMADDSUB213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPSr213rY, X86_INS_VFMADDSUB213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPSr231m, X86_INS_VFMADDSUB231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPSr231mY, X86_INS_VFMADDSUB231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPSr231r, X86_INS_VFMADDSUB231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMADDSUBPSr231rY, X86_INS_VFMADDSUB231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUB132PDZm, X86_INS_VFMSUB132PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUB132PDZmb, X86_INS_VFMSUB132PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUB132PSZm, X86_INS_VFMSUB132PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUB132PSZmb, X86_INS_VFMSUB132PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUB213PDZm, X86_INS_VFMSUB213PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUB213PDZmb, X86_INS_VFMSUB213PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUB213PDZr, X86_INS_VFMSUB213PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUB213PSZm, X86_INS_VFMSUB213PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUB213PSZmb, X86_INS_VFMSUB213PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUB213PSZr, X86_INS_VFMSUB213PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUBADD132PDZm, X86_INS_VFMSUBADD132PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUBADD132PDZmb, X86_INS_VFMSUBADD132PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUBADD132PSZm, X86_INS_VFMSUBADD132PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUBADD132PSZmb, X86_INS_VFMSUBADD132PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUBADD213PDZm, X86_INS_VFMSUBADD213PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUBADD213PDZmb, X86_INS_VFMSUBADD213PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUBADD213PDZr, X86_INS_VFMSUBADD213PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUBADD213PSZm, X86_INS_VFMSUBADD213PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUBADD213PSZmb, X86_INS_VFMSUBADD213PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUBADD213PSZr, X86_INS_VFMSUBADD213PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUBADDPD4mr, X86_INS_VFMSUBADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBADDPD4mrY, X86_INS_VFMSUBADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBADDPD4rm, X86_INS_VFMSUBADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBADDPD4rmY, X86_INS_VFMSUBADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBADDPD4rr, X86_INS_VFMSUBADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBADDPD4rrY, X86_INS_VFMSUBADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBADDPD4rrY_REV, X86_INS_VFMSUBADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBADDPD4rr_REV, X86_INS_VFMSUBADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBADDPDr132m, X86_INS_VFMSUBADD132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPDr132mY, X86_INS_VFMSUBADD132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPDr132r, X86_INS_VFMSUBADD132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPDr132rY, X86_INS_VFMSUBADD132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPDr213m, X86_INS_VFMSUBADD213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPDr213mY, X86_INS_VFMSUBADD213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPDr213r, X86_INS_VFMSUBADD213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPDr213rY, X86_INS_VFMSUBADD213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPDr231m, X86_INS_VFMSUBADD231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPDr231mY, X86_INS_VFMSUBADD231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPDr231r, X86_INS_VFMSUBADD231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPDr231rY, X86_INS_VFMSUBADD231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPS4mr, X86_INS_VFMSUBADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBADDPS4mrY, X86_INS_VFMSUBADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBADDPS4rm, X86_INS_VFMSUBADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBADDPS4rmY, X86_INS_VFMSUBADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBADDPS4rr, X86_INS_VFMSUBADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBADDPS4rrY, X86_INS_VFMSUBADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBADDPS4rrY_REV, X86_INS_VFMSUBADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBADDPS4rr_REV, X86_INS_VFMSUBADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBADDPSr132m, X86_INS_VFMSUBADD132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPSr132mY, X86_INS_VFMSUBADD132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPSr132r, X86_INS_VFMSUBADD132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPSr132rY, X86_INS_VFMSUBADD132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPSr213m, X86_INS_VFMSUBADD213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPSr213mY, X86_INS_VFMSUBADD213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPSr213r, X86_INS_VFMSUBADD213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPSr213rY, X86_INS_VFMSUBADD213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPSr231m, X86_INS_VFMSUBADD231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPSr231mY, X86_INS_VFMSUBADD231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPSr231r, X86_INS_VFMSUBADD231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBADDPSr231rY, X86_INS_VFMSUBADD231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPD4mr, X86_INS_VFMSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBPD4mrY, X86_INS_VFMSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBPD4rm, X86_INS_VFMSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBPD4rmY, X86_INS_VFMSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBPD4rr, X86_INS_VFMSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBPD4rrY, X86_INS_VFMSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBPD4rrY_REV, X86_INS_VFMSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBPD4rr_REV, X86_INS_VFMSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBPDr132m, X86_INS_VFMSUB132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPDr132mY, X86_INS_VFMSUB132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPDr132r, X86_INS_VFMSUB132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPDr132rY, X86_INS_VFMSUB132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPDr213m, X86_INS_VFMSUB213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPDr213mY, X86_INS_VFMSUB213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPDr213r, X86_INS_VFMSUB213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPDr213rY, X86_INS_VFMSUB213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPDr231m, X86_INS_VFMSUB231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPDr231mY, X86_INS_VFMSUB231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPDr231r, X86_INS_VFMSUB231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPDr231rY, X86_INS_VFMSUB231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPS4mr, X86_INS_VFMSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBPS4mrY, X86_INS_VFMSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBPS4rm, X86_INS_VFMSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBPS4rmY, X86_INS_VFMSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBPS4rr, X86_INS_VFMSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBPS4rrY, X86_INS_VFMSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBPS4rrY_REV, X86_INS_VFMSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBPS4rr_REV, X86_INS_VFMSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBPSr132m, X86_INS_VFMSUB132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPSr132mY, X86_INS_VFMSUB132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPSr132r, X86_INS_VFMSUB132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPSr132rY, X86_INS_VFMSUB132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPSr213m, X86_INS_VFMSUB213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPSr213mY, X86_INS_VFMSUB213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPSr213r, X86_INS_VFMSUB213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPSr213rY, X86_INS_VFMSUB213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPSr231m, X86_INS_VFMSUB231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPSr231mY, X86_INS_VFMSUB231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPSr231r, X86_INS_VFMSUB231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBPSr231rY, X86_INS_VFMSUB231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBSD4mr, X86_INS_VFMSUBSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBSD4mr_Int, X86_INS_VFMSUBSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBSD4rm, X86_INS_VFMSUBSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBSD4rm_Int, X86_INS_VFMSUBSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBSD4rr, X86_INS_VFMSUBSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBSD4rr_Int, X86_INS_VFMSUBSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBSD4rr_REV, X86_INS_VFMSUBSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBSDZm, X86_INS_VFMSUB213SD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUBSDZr, X86_INS_VFMSUB213SD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUBSDr132m, X86_INS_VFMSUB132SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBSDr132r, X86_INS_VFMSUB132SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBSDr213m, X86_INS_VFMSUB213SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBSDr213m_Int, X86_INS_VFMSUB213SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBSDr213r, X86_INS_VFMSUB213SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBSDr213r_Int, X86_INS_VFMSUB213SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBSDr231m, X86_INS_VFMSUB231SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBSDr231r, X86_INS_VFMSUB231SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBSS4mr, X86_INS_VFMSUBSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBSS4mr_Int, X86_INS_VFMSUBSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBSS4rm, X86_INS_VFMSUBSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBSS4rm_Int, X86_INS_VFMSUBSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBSS4rr, X86_INS_VFMSUBSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBSS4rr_Int, X86_INS_VFMSUBSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBSS4rr_REV, X86_INS_VFMSUBSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFMSUBSSZm, X86_INS_VFMSUB213SS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUBSSZr, X86_INS_VFMSUB213SS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFMSUBSSr132m, X86_INS_VFMSUB132SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBSSr132r, X86_INS_VFMSUB132SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBSSr213m, X86_INS_VFMSUB213SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBSSr213m_Int, X86_INS_VFMSUB213SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBSSr213r, X86_INS_VFMSUB213SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBSSr213r_Int, X86_INS_VFMSUB213SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBSSr231m, X86_INS_VFMSUB231SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFMSUBSSr231r, X86_INS_VFMSUB231SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADD132PDZm, X86_INS_VFNMADD132PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMADD132PDZmb, X86_INS_VFNMADD132PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMADD132PSZm, X86_INS_VFNMADD132PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMADD132PSZmb, X86_INS_VFNMADD132PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMADD213PDZm, X86_INS_VFNMADD213PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMADD213PDZmb, X86_INS_VFNMADD213PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMADD213PDZr, X86_INS_VFNMADD213PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMADD213PSZm, X86_INS_VFNMADD213PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMADD213PSZmb, X86_INS_VFNMADD213PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMADD213PSZr, X86_INS_VFNMADD213PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMADDPD4mr, X86_INS_VFNMADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDPD4mrY, X86_INS_VFNMADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDPD4rm, X86_INS_VFNMADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDPD4rmY, X86_INS_VFNMADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDPD4rr, X86_INS_VFNMADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDPD4rrY, X86_INS_VFNMADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDPD4rrY_REV, X86_INS_VFNMADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDPD4rr_REV, X86_INS_VFNMADDPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDPDr132m, X86_INS_VFNMADD132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPDr132mY, X86_INS_VFNMADD132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPDr132r, X86_INS_VFNMADD132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPDr132rY, X86_INS_VFNMADD132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPDr213m, X86_INS_VFNMADD213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPDr213mY, X86_INS_VFNMADD213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPDr213r, X86_INS_VFNMADD213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPDr213rY, X86_INS_VFNMADD213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPDr231m, X86_INS_VFNMADD231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPDr231mY, X86_INS_VFNMADD231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPDr231r, X86_INS_VFNMADD231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPDr231rY, X86_INS_VFNMADD231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPS4mr, X86_INS_VFNMADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDPS4mrY, X86_INS_VFNMADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDPS4rm, X86_INS_VFNMADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDPS4rmY, X86_INS_VFNMADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDPS4rr, X86_INS_VFNMADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDPS4rrY, X86_INS_VFNMADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDPS4rrY_REV, X86_INS_VFNMADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDPS4rr_REV, X86_INS_VFNMADDPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDPSr132m, X86_INS_VFNMADD132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPSr132mY, X86_INS_VFNMADD132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPSr132r, X86_INS_VFNMADD132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPSr132rY, X86_INS_VFNMADD132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPSr213m, X86_INS_VFNMADD213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPSr213mY, X86_INS_VFNMADD213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPSr213r, X86_INS_VFNMADD213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPSr213rY, X86_INS_VFNMADD213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPSr231m, X86_INS_VFNMADD231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPSr231mY, X86_INS_VFNMADD231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPSr231r, X86_INS_VFNMADD231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDPSr231rY, X86_INS_VFNMADD231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDSD4mr, X86_INS_VFNMADDSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDSD4mr_Int, X86_INS_VFNMADDSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDSD4rm, X86_INS_VFNMADDSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDSD4rm_Int, X86_INS_VFNMADDSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDSD4rr, X86_INS_VFNMADDSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDSD4rr_Int, X86_INS_VFNMADDSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDSD4rr_REV, X86_INS_VFNMADDSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDSDZm, X86_INS_VFNMADD213SD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMADDSDZr, X86_INS_VFNMADD213SD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMADDSDr132m, X86_INS_VFNMADD132SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDSDr132r, X86_INS_VFNMADD132SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDSDr213m, X86_INS_VFNMADD213SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDSDr213m_Int, X86_INS_VFNMADD213SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDSDr213r, X86_INS_VFNMADD213SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDSDr213r_Int, X86_INS_VFNMADD213SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDSDr231m, X86_INS_VFNMADD231SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDSDr231r, X86_INS_VFNMADD231SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDSS4mr, X86_INS_VFNMADDSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDSS4mr_Int, X86_INS_VFNMADDSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDSS4rm, X86_INS_VFNMADDSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDSS4rm_Int, X86_INS_VFNMADDSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDSS4rr, X86_INS_VFNMADDSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDSS4rr_Int, X86_INS_VFNMADDSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDSS4rr_REV, X86_INS_VFNMADDSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMADDSSZm, X86_INS_VFNMADD213SS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMADDSSZr, X86_INS_VFNMADD213SS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMADDSSr132m, X86_INS_VFNMADD132SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDSSr132r, X86_INS_VFNMADD132SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDSSr213m, X86_INS_VFNMADD213SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDSSr213m_Int, X86_INS_VFNMADD213SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDSSr213r, X86_INS_VFNMADD213SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDSSr213r_Int, X86_INS_VFNMADD213SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDSSr231m, X86_INS_VFNMADD231SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMADDSSr231r, X86_INS_VFNMADD231SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUB132PDZm, X86_INS_VFNMSUB132PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMSUB132PDZmb, X86_INS_VFNMSUB132PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMSUB132PSZm, X86_INS_VFNMSUB132PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMSUB132PSZmb, X86_INS_VFNMSUB132PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMSUB213PDZm, X86_INS_VFNMSUB213PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMSUB213PDZmb, X86_INS_VFNMSUB213PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMSUB213PDZr, X86_INS_VFNMSUB213PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMSUB213PSZm, X86_INS_VFNMSUB213PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMSUB213PSZmb, X86_INS_VFNMSUB213PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMSUB213PSZr, X86_INS_VFNMSUB213PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMSUBPD4mr, X86_INS_VFNMSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBPD4mrY, X86_INS_VFNMSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBPD4rm, X86_INS_VFNMSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBPD4rmY, X86_INS_VFNMSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBPD4rr, X86_INS_VFNMSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBPD4rrY, X86_INS_VFNMSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBPD4rrY_REV, X86_INS_VFNMSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBPD4rr_REV, X86_INS_VFNMSUBPD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBPDr132m, X86_INS_VFNMSUB132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPDr132mY, X86_INS_VFNMSUB132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPDr132r, X86_INS_VFNMSUB132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPDr132rY, X86_INS_VFNMSUB132PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPDr213m, X86_INS_VFNMSUB213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPDr213mY, X86_INS_VFNMSUB213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPDr213r, X86_INS_VFNMSUB213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPDr213rY, X86_INS_VFNMSUB213PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPDr231m, X86_INS_VFNMSUB231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPDr231mY, X86_INS_VFNMSUB231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPDr231r, X86_INS_VFNMSUB231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPDr231rY, X86_INS_VFNMSUB231PD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPS4mr, X86_INS_VFNMSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBPS4mrY, X86_INS_VFNMSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBPS4rm, X86_INS_VFNMSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBPS4rmY, X86_INS_VFNMSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBPS4rr, X86_INS_VFNMSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBPS4rrY, X86_INS_VFNMSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBPS4rrY_REV, X86_INS_VFNMSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBPS4rr_REV, X86_INS_VFNMSUBPS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBPSr132m, X86_INS_VFNMSUB132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPSr132mY, X86_INS_VFNMSUB132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPSr132r, X86_INS_VFNMSUB132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPSr132rY, X86_INS_VFNMSUB132PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPSr213m, X86_INS_VFNMSUB213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPSr213mY, X86_INS_VFNMSUB213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPSr213r, X86_INS_VFNMSUB213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPSr213rY, X86_INS_VFNMSUB213PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPSr231m, X86_INS_VFNMSUB231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPSr231mY, X86_INS_VFNMSUB231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPSr231r, X86_INS_VFNMSUB231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBPSr231rY, X86_INS_VFNMSUB231PS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBSD4mr, X86_INS_VFNMSUBSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBSD4mr_Int, X86_INS_VFNMSUBSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBSD4rm, X86_INS_VFNMSUBSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBSD4rm_Int, X86_INS_VFNMSUBSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBSD4rr, X86_INS_VFNMSUBSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBSD4rr_Int, X86_INS_VFNMSUBSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBSD4rr_REV, X86_INS_VFNMSUBSD, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBSDZm, X86_INS_VFNMSUB213SD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMSUBSDZr, X86_INS_VFNMSUB213SD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMSUBSDr132m, X86_INS_VFNMSUB132SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBSDr132r, X86_INS_VFNMSUB132SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBSDr213m, X86_INS_VFNMSUB213SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBSDr213m_Int, X86_INS_VFNMSUB213SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBSDr213r, X86_INS_VFNMSUB213SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBSDr213r_Int, X86_INS_VFNMSUB213SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBSDr231m, X86_INS_VFNMSUB231SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBSDr231r, X86_INS_VFNMSUB231SD, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBSS4mr, X86_INS_VFNMSUBSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBSS4mr_Int, X86_INS_VFNMSUBSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBSS4rm, X86_INS_VFNMSUBSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBSS4rm_Int, X86_INS_VFNMSUBSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBSS4rr, X86_INS_VFNMSUBSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBSS4rr_Int, X86_INS_VFNMSUBSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBSS4rr_REV, X86_INS_VFNMSUBSS, { 0 }, { 0 }, { X86_GRP_FMA4, 0 } },
	{ X86_VFNMSUBSSZm, X86_INS_VFNMSUB213SS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMSUBSSZr, X86_INS_VFNMSUB213SS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VFNMSUBSSr132m, X86_INS_VFNMSUB132SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBSSr132r, X86_INS_VFNMSUB132SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBSSr213m, X86_INS_VFNMSUB213SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBSSr213m_Int, X86_INS_VFNMSUB213SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBSSr213r, X86_INS_VFNMSUB213SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBSSr213r_Int, X86_INS_VFNMSUB213SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBSSr231m, X86_INS_VFNMSUB231SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFNMSUBSSr231r, X86_INS_VFNMSUB231SS, { 0 }, { 0 }, { X86_GRP_FMA, 0 } },
	{ X86_VFRCZPDrm, X86_INS_VFRCZPD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VFRCZPDrmY, X86_INS_VFRCZPD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VFRCZPDrr, X86_INS_VFRCZPD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VFRCZPDrrY, X86_INS_VFRCZPD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VFRCZPSrm, X86_INS_VFRCZPS, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VFRCZPSrmY, X86_INS_VFRCZPS, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VFRCZPSrr, X86_INS_VFRCZPS, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VFRCZPSrrY, X86_INS_VFRCZPS, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VFRCZSDrm, X86_INS_VFRCZSD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VFRCZSDrr, X86_INS_VFRCZSD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VFRCZSSrm, X86_INS_VFRCZSS, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VFRCZSSrr, X86_INS_VFRCZSS, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VFsANDNPDrm, X86_INS_VANDNPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VFsANDNPDrr, X86_INS_VANDNPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VFsANDNPSrm, X86_INS_VANDNPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VFsANDNPSrr, X86_INS_VANDNPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VFsANDPDrm, X86_INS_VANDPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VFsANDPDrr, X86_INS_VANDPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VFsANDPSrm, X86_INS_VANDPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VFsANDPSrr, X86_INS_VANDPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VFsORPDrm, X86_INS_VORPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VFsORPDrr, X86_INS_VORPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VFsORPSrm, X86_INS_VORPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VFsORPSrr, X86_INS_VORPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VFsXORPDrm, X86_INS_VXORPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VFsXORPDrr, X86_INS_VXORPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VFsXORPSrm, X86_INS_VXORPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VFsXORPSrr, X86_INS_VXORPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VGATHERDPDYrm, X86_INS_VGATHERDPD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VGATHERDPDZrm, X86_INS_VGATHERDPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VGATHERDPDrm, X86_INS_VGATHERDPD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VGATHERDPSYrm, X86_INS_VGATHERDPS, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VGATHERDPSZrm, X86_INS_VGATHERDPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VGATHERDPSrm, X86_INS_VGATHERDPS, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VGATHERQPDYrm, X86_INS_VGATHERQPD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VGATHERQPDZrm, X86_INS_VGATHERQPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VGATHERQPDrm, X86_INS_VGATHERQPD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VGATHERQPSYrm, X86_INS_VGATHERQPS, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VGATHERQPSZrm, X86_INS_VGATHERQPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VGATHERQPSrm, X86_INS_VGATHERQPS, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VHADDPDYrm, X86_INS_VHADDPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VHADDPDYrr, X86_INS_VHADDPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VHADDPDrm, X86_INS_VHADDPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VHADDPDrr, X86_INS_VHADDPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VHADDPSYrm, X86_INS_VHADDPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VHADDPSYrr, X86_INS_VHADDPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VHADDPSrm, X86_INS_VHADDPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VHADDPSrr, X86_INS_VHADDPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VHSUBPDYrm, X86_INS_VHSUBPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VHSUBPDYrr, X86_INS_VHSUBPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VHSUBPDrm, X86_INS_VHSUBPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VHSUBPDrr, X86_INS_VHSUBPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VHSUBPSYrm, X86_INS_VHSUBPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VHSUBPSYrr, X86_INS_VHSUBPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VHSUBPSrm, X86_INS_VHSUBPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VHSUBPSrr, X86_INS_VHSUBPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VINSERTF128rm, X86_INS_VINSERTF128, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VINSERTF128rr, X86_INS_VINSERTF128, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VINSERTF32x4rm, X86_INS_VINSERTF32X4, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VINSERTF32x4rr, X86_INS_VINSERTF32X4, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VINSERTF64x4rm, X86_INS_VINSERTF64X4, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VINSERTF64x4rr, X86_INS_VINSERTF64X4, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VINSERTI128rm, X86_INS_VINSERTI128, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VINSERTI128rr, X86_INS_VINSERTI128, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VINSERTI32x4rm, X86_INS_VINSERTI32X4, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VINSERTI32x4rr, X86_INS_VINSERTI32X4, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VINSERTI64x4rm, X86_INS_VINSERTI64X4, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VINSERTI64x4rr, X86_INS_VINSERTI64X4, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VINSERTPSrm, X86_INS_VINSERTPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VINSERTPSrr, X86_INS_VINSERTPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VINSERTPSzrm, X86_INS_VINSERTPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VINSERTPSzrr, X86_INS_VINSERTPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VLDDQUYrm, X86_INS_VLDDQU, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VLDDQUrm, X86_INS_VLDDQU, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VLDMXCSR, X86_INS_VLDMXCSR, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMASKMOVDQU, X86_INS_VMASKMOVDQU, { X86_REG_EDI, 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMASKMOVDQU64, X86_INS_VMASKMOVDQU, { X86_REG_RDI, 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMASKMOVPDYmr, X86_INS_VMASKMOVPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMASKMOVPDYrm, X86_INS_VMASKMOVPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMASKMOVPDmr, X86_INS_VMASKMOVPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMASKMOVPDrm, X86_INS_VMASKMOVPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMASKMOVPSYmr, X86_INS_VMASKMOVPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMASKMOVPSYrm, X86_INS_VMASKMOVPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMASKMOVPSmr, X86_INS_VMASKMOVPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMASKMOVPSrm, X86_INS_VMASKMOVPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXCPDYrm, X86_INS_VMAXPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXCPDYrr, X86_INS_VMAXPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXCPDrm, X86_INS_VMAXPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXCPDrr, X86_INS_VMAXPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXCPSYrm, X86_INS_VMAXPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXCPSYrr, X86_INS_VMAXPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXCPSrm, X86_INS_VMAXPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXCPSrr, X86_INS_VMAXPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXCSDrm, X86_INS_VMAXSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXCSDrr, X86_INS_VMAXSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXCSSrm, X86_INS_VMAXSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXCSSrr, X86_INS_VMAXSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXPDYrm, X86_INS_VMAXPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXPDYrr, X86_INS_VMAXPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXPDZrm, X86_INS_VMAXPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMAXPDZrmb, X86_INS_VMAXPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMAXPDZrr, X86_INS_VMAXPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMAXPDrm, X86_INS_VMAXPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXPDrr, X86_INS_VMAXPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXPSYrm, X86_INS_VMAXPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXPSYrr, X86_INS_VMAXPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXPSZrm, X86_INS_VMAXPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMAXPSZrmb, X86_INS_VMAXPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMAXPSZrr, X86_INS_VMAXPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMAXPSrm, X86_INS_VMAXPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXPSrr, X86_INS_VMAXPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXSDZrm, X86_INS_VMAXSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMAXSDZrr, X86_INS_VMAXSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMAXSDrm, X86_INS_VMAXSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXSDrm_Int, X86_INS_VMAXSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXSDrr, X86_INS_VMAXSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXSDrr_Int, X86_INS_VMAXSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXSSZrm, X86_INS_VMAXSS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMAXSSZrr, X86_INS_VMAXSS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMAXSSrm, X86_INS_VMAXSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXSSrm_Int, X86_INS_VMAXSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXSSrr, X86_INS_VMAXSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMAXSSrr_Int, X86_INS_VMAXSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMCALL, X86_INS_VMCALL, { 0 }, { 0 }, { 0 } },
	{ X86_VMCLEARm, X86_INS_VMCLEAR, { 0 }, { 0 }, { 0 } },
	{ X86_VMFUNC, X86_INS_VMFUNC, { 0 }, { 0 }, { 0 } },
	{ X86_VMINCPDYrm, X86_INS_VMINPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINCPDYrr, X86_INS_VMINPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINCPDrm, X86_INS_VMINPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINCPDrr, X86_INS_VMINPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINCPSYrm, X86_INS_VMINPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINCPSYrr, X86_INS_VMINPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINCPSrm, X86_INS_VMINPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINCPSrr, X86_INS_VMINPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINCSDrm, X86_INS_VMINSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINCSDrr, X86_INS_VMINSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINCSSrm, X86_INS_VMINSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINCSSrr, X86_INS_VMINSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINPDYrm, X86_INS_VMINPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINPDYrr, X86_INS_VMINPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINPDZrm, X86_INS_VMINPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMINPDZrmb, X86_INS_VMINPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMINPDZrr, X86_INS_VMINPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMINPDrm, X86_INS_VMINPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINPDrr, X86_INS_VMINPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINPSYrm, X86_INS_VMINPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINPSYrr, X86_INS_VMINPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINPSZrm, X86_INS_VMINPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMINPSZrmb, X86_INS_VMINPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMINPSZrr, X86_INS_VMINPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMINPSrm, X86_INS_VMINPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINPSrr, X86_INS_VMINPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINSDZrm, X86_INS_VMINSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMINSDZrr, X86_INS_VMINSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMINSDrm, X86_INS_VMINSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINSDrm_Int, X86_INS_VMINSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINSDrr, X86_INS_VMINSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINSDrr_Int, X86_INS_VMINSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINSSZrm, X86_INS_VMINSS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMINSSZrr, X86_INS_VMINSS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMINSSrm, X86_INS_VMINSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINSSrm_Int, X86_INS_VMINSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINSSrr, X86_INS_VMINSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMINSSrr_Int, X86_INS_VMINSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMLAUNCH, X86_INS_VMLAUNCH, { 0 }, { 0 }, { 0 } },
	{ X86_VMLOAD32, X86_INS_VMLOAD, { X86_REG_EAX, 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_VMLOAD64, X86_INS_VMLOAD, { X86_REG_RAX, 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_VMMCALL, X86_INS_VMMCALL, { 0 }, { 0 }, { 0 } },
	{ X86_VMOV64toPQIZrr, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOV64toPQIrr, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOV64toSDZrr, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOV64toSDrm, X86_INS_VVMOVQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOV64toSDrr, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVAPDYmr, X86_INS_VMOVAPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVAPDYrm, X86_INS_VMOVAPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVAPDYrr, X86_INS_VMOVAPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVAPDYrr_REV, X86_INS_VMOVAPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVAPDZmr, X86_INS_VMOVAPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVAPDZrm, X86_INS_VMOVAPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVAPDZrmk, X86_INS_VMOVAPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVAPDZrr, X86_INS_VMOVAPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVAPDZrrk, X86_INS_VMOVAPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVAPDmr, X86_INS_VMOVAPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVAPDrm, X86_INS_VMOVAPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVAPDrr, X86_INS_VMOVAPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVAPDrr_REV, X86_INS_VMOVAPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVAPSYmr, X86_INS_VMOVAPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVAPSYrm, X86_INS_VMOVAPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVAPSYrr, X86_INS_VMOVAPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVAPSYrr_REV, X86_INS_VMOVAPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVAPSZmr, X86_INS_VMOVAPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVAPSZrm, X86_INS_VMOVAPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVAPSZrmk, X86_INS_VMOVAPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVAPSZrr, X86_INS_VMOVAPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVAPSZrrk, X86_INS_VMOVAPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVAPSmr, X86_INS_VMOVAPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVAPSrm, X86_INS_VMOVAPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVAPSrr, X86_INS_VMOVAPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVAPSrr_REV, X86_INS_VMOVAPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDDUPYrm, X86_INS_VMOVDDUP, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDDUPYrr, X86_INS_VMOVDDUP, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDDUPZrm, X86_INS_VMOVDDUP, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVDDUPZrr, X86_INS_VMOVDDUP, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVDDUPrm, X86_INS_VMOVDDUP, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDDUPrr, X86_INS_VMOVDDUP, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDI2PDIZrm, X86_INS_VMOVD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVDI2PDIZrr, X86_INS_VMOVD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVDI2PDIrm, X86_INS_VMOVD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDI2PDIrr, X86_INS_VMOVD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDI2SSZrm, X86_INS_VMOVD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVDI2SSZrr, X86_INS_VMOVD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVDI2SSrm, X86_INS_VMOVD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDI2SSrr, X86_INS_VMOVD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDQA32mr, X86_INS_VMOVDQA32, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVDQA32rm, X86_INS_VMOVDQA32, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVDQA32rr, X86_INS_VMOVDQA32, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVDQA64mr, X86_INS_VMOVDQA64, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVDQA64rm, X86_INS_VMOVDQA64, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVDQA64rr, X86_INS_VMOVDQA64, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVDQAYmr, X86_INS_VMOVDQA, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDQAYrm, X86_INS_VMOVDQA, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDQAYrr, X86_INS_VMOVDQA, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDQAYrr_REV, X86_INS_VMOVDQA, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDQAmr, X86_INS_VMOVDQA, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDQArm, X86_INS_VMOVDQA, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDQArr, X86_INS_VMOVDQA, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDQArr_REV, X86_INS_VMOVDQA, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDQU32rm, X86_INS_VMOVDQU32, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVDQU32rmk, X86_INS_VMOVDQU32, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVDQU32rr, X86_INS_VMOVDQU32, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVDQU32rrk, X86_INS_VMOVDQU32, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVDQU64rm, X86_INS_VMOVDQU64, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVDQU64rmk, X86_INS_VMOVDQU64, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVDQU64rr, X86_INS_VMOVDQU64, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVDQU64rrk, X86_INS_VMOVDQU64, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVDQUYmr, X86_INS_VMOVDQU, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDQUYrm, X86_INS_VMOVDQU, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDQUYrr, X86_INS_VMOVDQU, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDQUYrr_REV, X86_INS_VMOVDQU, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDQUmr, X86_INS_VMOVDQU, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDQUrm, X86_INS_VMOVDQU, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDQUrr, X86_INS_VMOVDQU, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVDQUrr_REV, X86_INS_VMOVDQU, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVHLPSZrr, X86_INS_VMOVHLPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVHLPSrr, X86_INS_VMOVHLPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVHPDmr, X86_INS_VMOVHPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVHPDrm, X86_INS_VMOVHPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVHPSmr, X86_INS_VMOVHPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVHPSrm, X86_INS_VMOVHPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVLHPSZrr, X86_INS_VMOVLHPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVLHPSrr, X86_INS_VMOVLHPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVLPDmr, X86_INS_VMOVLPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVLPDrm, X86_INS_VMOVLPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVLPSmr, X86_INS_VMOVLPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVLPSrm, X86_INS_VMOVLPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVLQ128mr, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVMSKPDYr64r, X86_INS_VMOVMSKPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVMSKPDYrr32, X86_INS_VMOVMSKPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVMSKPDYrr64, X86_INS_VMOVMSKPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVMSKPDr64r, X86_INS_VMOVMSKPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVMSKPDrr32, X86_INS_VMOVMSKPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVMSKPDrr64, X86_INS_VMOVMSKPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVMSKPSYr64r, X86_INS_VMOVMSKPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVMSKPSYrr32, X86_INS_VMOVMSKPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVMSKPSYrr64, X86_INS_VMOVMSKPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVMSKPSr64r, X86_INS_VMOVMSKPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVMSKPSrr32, X86_INS_VMOVMSKPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVMSKPSrr64, X86_INS_VMOVMSKPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVNTDQAYrm, X86_INS_VMOVNTDQA, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VMOVNTDQArm, X86_INS_VMOVNTDQA, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVNTDQYmr, X86_INS_VMOVNTDQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVNTDQmr, X86_INS_VMOVNTDQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVNTPDYmr, X86_INS_VMOVNTPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVNTPDmr, X86_INS_VMOVNTPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVNTPSYmr, X86_INS_VMOVNTPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVNTPSmr, X86_INS_VMOVNTPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVPDI2DIZmr, X86_INS_VMOVD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVPDI2DIZrr, X86_INS_VMOVD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVPDI2DImr, X86_INS_VMOVD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVPDI2DIrr, X86_INS_VMOVD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVPQI2QImr, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVPQIto64Zmr, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX512, X86_GRP_MODE64, 0 } },
	{ X86_VMOVPQIto64Zrr, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX512, X86_GRP_MODE64, 0 } },
	{ X86_VMOVPQIto64rr, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVQI2PQIZrm, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVQI2PQIrm, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVQd64rr, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVQd64rr_alt, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVQs64rr, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVQxrxr, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVSDZmr, X86_INS_VMOVSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVSDZrm, X86_INS_VMOVSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVSDZrr, X86_INS_VMOVSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVSDZrr_REV, X86_INS_VMOVSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVSDmr, X86_INS_VMOVSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVSDrm, X86_INS_VMOVSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVSDrr, X86_INS_VMOVSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVSDrr_REV, X86_INS_VMOVSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVSDto64Zmr, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVSDto64Zrr, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVSDto64mr, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVSDto64rr, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVSHDUPYrm, X86_INS_VMOVSHDUP, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVSHDUPYrr, X86_INS_VMOVSHDUP, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVSHDUPrm, X86_INS_VMOVSHDUP, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVSHDUPrr, X86_INS_VMOVSHDUP, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVSLDUPYrm, X86_INS_VMOVSLDUP, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVSLDUPYrr, X86_INS_VMOVSLDUP, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVSLDUPrm, X86_INS_VMOVSLDUP, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVSLDUPrr, X86_INS_VMOVSLDUP, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVSS2DIZmr, X86_INS_VMOVD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVSS2DIZrr, X86_INS_VMOVD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVSS2DImr, X86_INS_VMOVD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVSS2DIrr, X86_INS_VMOVD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVSSZmr, X86_INS_VMOVSS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVSSZrm, X86_INS_VMOVSS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVSSZrr, X86_INS_VMOVSS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVSSZrr_REV, X86_INS_VMOVSS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVSSmr, X86_INS_VMOVSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVSSrm, X86_INS_VMOVSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVSSrr, X86_INS_VMOVSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVSSrr_REV, X86_INS_VMOVSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVUPDYmr, X86_INS_VMOVUPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVUPDYrm, X86_INS_VMOVUPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVUPDYrr, X86_INS_VMOVUPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVUPDYrr_REV, X86_INS_VMOVUPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVUPDZmr, X86_INS_VMOVUPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVUPDZrm, X86_INS_VMOVUPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVUPDZrmk, X86_INS_VMOVUPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVUPDZrr, X86_INS_VMOVUPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVUPDZrrk, X86_INS_VMOVUPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVUPDmr, X86_INS_VMOVUPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVUPDrm, X86_INS_VMOVUPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVUPDrr, X86_INS_VMOVUPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVUPDrr_REV, X86_INS_VMOVUPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVUPSYmr, X86_INS_VMOVUPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVUPSYrm, X86_INS_VMOVUPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVUPSYrr, X86_INS_VMOVUPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVUPSYrr_REV, X86_INS_VMOVUPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVUPSZmr, X86_INS_VMOVUPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVUPSZrm, X86_INS_VMOVUPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVUPSZrmk, X86_INS_VMOVUPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVUPSZrr, X86_INS_VMOVUPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVUPSZrrk, X86_INS_VMOVUPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVUPSmr, X86_INS_VMOVUPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVUPSrm, X86_INS_VMOVUPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVUPSrr, X86_INS_VMOVUPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVUPSrr_REV, X86_INS_VMOVUPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVZDI2PDIrm, X86_INS_VMOVD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVZDI2PDIrr, X86_INS_VMOVD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVZPQILo2PQIZrm, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVZPQILo2PQIZrr, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMOVZPQILo2PQIrm, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVZPQILo2PQIrr, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVZQI2PQIrm, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMOVZQI2PQIrr, X86_INS_VMOVQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMPSADBWYrmi, X86_INS_VMPSADBW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VMPSADBWYrri, X86_INS_VMPSADBW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VMPSADBWrmi, X86_INS_VMPSADBW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMPSADBWrri, X86_INS_VMPSADBW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMPTRLDm, X86_INS_VMPTRLD, { 0 }, { 0 }, { 0 } },
	{ X86_VMPTRSTm, X86_INS_VMPTRST, { 0 }, { 0 }, { 0 } },
	{ X86_VMREAD32rm, X86_INS_VMREAD, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_VMREAD32rr, X86_INS_VMREAD, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_VMREAD64rm, X86_INS_VMREAD, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_VMREAD64rr, X86_INS_VMREAD, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_VMRESUME, X86_INS_VMRESUME, { 0 }, { 0 }, { 0 } },
	{ X86_VMRUN32, X86_INS_VMRUN, { X86_REG_EAX, 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_VMRUN64, X86_INS_VMRUN, { X86_REG_RAX, 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_VMSAVE32, X86_INS_VMSAVE, { X86_REG_EAX, 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_VMSAVE64, X86_INS_VMSAVE, { X86_REG_RAX, 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_VMULPDYrm, X86_INS_VMULPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMULPDYrr, X86_INS_VMULPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMULPDZrm, X86_INS_VMULPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMULPDZrmb, X86_INS_VMULPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMULPDZrr, X86_INS_VMULPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMULPDrm, X86_INS_VMULPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMULPDrr, X86_INS_VMULPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMULPSYrm, X86_INS_VMULPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMULPSYrr, X86_INS_VMULPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMULPSZrm, X86_INS_VMULPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMULPSZrmb, X86_INS_VMULPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMULPSZrr, X86_INS_VMULPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMULPSrm, X86_INS_VMULPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMULPSrr, X86_INS_VMULPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMULSDZrm, X86_INS_VMULSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMULSDZrr, X86_INS_VMULSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMULSDrm, X86_INS_VMULSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMULSDrm_Int, X86_INS_VMULSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMULSDrr, X86_INS_VMULSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMULSDrr_Int, X86_INS_VMULSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMULSSZrm, X86_INS_VMULSS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMULSSZrr, X86_INS_VMULSS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VMULSSrm, X86_INS_VMULSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMULSSrm_Int, X86_INS_VMULSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMULSSrr, X86_INS_VMULSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMULSSrr_Int, X86_INS_VMULSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VMWRITE32rm, X86_INS_VMWRITE, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_VMWRITE32rr, X86_INS_VMWRITE, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_VMWRITE64rm, X86_INS_VMWRITE, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_VMWRITE64rr, X86_INS_VMWRITE, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_VMXOFF, X86_INS_VMXOFF, { 0 }, { 0 }, { 0 } },
	{ X86_VMXON, X86_INS_VMXON, { 0 }, { 0 }, { 0 } },
	{ X86_VORPDYrm, X86_INS_VORPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VORPDYrr, X86_INS_VORPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VORPDrm, X86_INS_VORPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VORPDrr, X86_INS_VORPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VORPSYrm, X86_INS_VORPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VORPSYrr, X86_INS_VORPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VORPSrm, X86_INS_VORPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VORPSrr, X86_INS_VORPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPABSBrm128, X86_INS_VPABSB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPABSBrm256, X86_INS_VPABSB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPABSBrr128, X86_INS_VPABSB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPABSBrr256, X86_INS_VPABSB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPABSDrm, X86_INS_VPABSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPABSDrm128, X86_INS_VPABSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPABSDrm256, X86_INS_VPABSD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPABSDrr, X86_INS_VPABSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPABSDrr128, X86_INS_VPABSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPABSDrr256, X86_INS_VPABSD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPABSQrm, X86_INS_VPABSQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPABSQrr, X86_INS_VPABSQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPABSWrm128, X86_INS_VPABSW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPABSWrm256, X86_INS_VPABSW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPABSWrr128, X86_INS_VPABSW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPABSWrr256, X86_INS_VPABSW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPACKSSDWYrm, X86_INS_VPACKSSDW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPACKSSDWYrr, X86_INS_VPACKSSDW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPACKSSDWrm, X86_INS_VPACKSSDW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPACKSSDWrr, X86_INS_VPACKSSDW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPACKSSWBYrm, X86_INS_VPACKSSWB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPACKSSWBYrr, X86_INS_VPACKSSWB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPACKSSWBrm, X86_INS_VPACKSSWB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPACKSSWBrr, X86_INS_VPACKSSWB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPACKUSDWYrm, X86_INS_VPACKUSDW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPACKUSDWYrr, X86_INS_VPACKUSDW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPACKUSDWrm, X86_INS_VPACKUSDW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPACKUSDWrr, X86_INS_VPACKUSDW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPACKUSWBYrm, X86_INS_VPACKUSWB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPACKUSWBYrr, X86_INS_VPACKUSWB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPACKUSWBrm, X86_INS_VPACKUSWB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPACKUSWBrr, X86_INS_VPACKUSWB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPADDBYrm, X86_INS_VPADDB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPADDBYrr, X86_INS_VPADDB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPADDBrm, X86_INS_VPADDB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPADDBrr, X86_INS_VPADDB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPADDDYrm, X86_INS_VPADDD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPADDDYrr, X86_INS_VPADDD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPADDDZrm, X86_INS_VPADDD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPADDDZrmb, X86_INS_VPADDD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPADDDZrr, X86_INS_VPADDD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPADDDrm, X86_INS_VPADDD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPADDDrr, X86_INS_VPADDD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPADDQYrm, X86_INS_VPADDQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPADDQYrr, X86_INS_VPADDQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPADDQZrm, X86_INS_VPADDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPADDQZrmb, X86_INS_VPADDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPADDQZrr, X86_INS_VPADDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPADDQrm, X86_INS_VPADDQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPADDQrr, X86_INS_VPADDQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPADDSBYrm, X86_INS_VPADDSB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPADDSBYrr, X86_INS_VPADDSB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPADDSBrm, X86_INS_VPADDSB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPADDSBrr, X86_INS_VPADDSB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPADDSWYrm, X86_INS_VPADDSW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPADDSWYrr, X86_INS_VPADDSW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPADDSWrm, X86_INS_VPADDSW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPADDSWrr, X86_INS_VPADDSW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPADDUSBYrm, X86_INS_VPADDUSB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPADDUSBYrr, X86_INS_VPADDUSB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPADDUSBrm, X86_INS_VPADDUSB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPADDUSBrr, X86_INS_VPADDUSB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPADDUSWYrm, X86_INS_VPADDUSW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPADDUSWYrr, X86_INS_VPADDUSW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPADDUSWrm, X86_INS_VPADDUSW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPADDUSWrr, X86_INS_VPADDUSW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPADDWYrm, X86_INS_VPADDW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPADDWYrr, X86_INS_VPADDW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPADDWrm, X86_INS_VPADDW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPADDWrr, X86_INS_VPADDW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPALIGNR128rm, X86_INS_VPALIGNR, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPALIGNR128rr, X86_INS_VPALIGNR, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPALIGNR256rm, X86_INS_VPALIGNR, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPALIGNR256rr, X86_INS_VPALIGNR, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPANDDZrm, X86_INS_VPANDD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPANDDZrmb, X86_INS_VPANDD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPANDDZrr, X86_INS_VPANDD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPANDNDZrm, X86_INS_VPANDND, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPANDNDZrmb, X86_INS_VPANDND, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPANDNDZrr, X86_INS_VPANDND, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPANDNQZrm, X86_INS_VPANDNQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPANDNQZrmb, X86_INS_VPANDNQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPANDNQZrr, X86_INS_VPANDNQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPANDNYrm, X86_INS_VPANDN, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPANDNYrr, X86_INS_VPANDN, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPANDNrm, X86_INS_VPANDN, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPANDNrr, X86_INS_VPANDN, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPANDQZrm, X86_INS_VPANDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPANDQZrmb, X86_INS_VPANDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPANDQZrr, X86_INS_VPANDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPANDYrm, X86_INS_VPAND, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPANDYrr, X86_INS_VPAND, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPANDrm, X86_INS_VPAND, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPANDrr, X86_INS_VPAND, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPAVGBYrm, X86_INS_VPAVGB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPAVGBYrr, X86_INS_VPAVGB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPAVGBrm, X86_INS_VPAVGB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPAVGBrr, X86_INS_VPAVGB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPAVGWYrm, X86_INS_VPAVGW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPAVGWYrr, X86_INS_VPAVGW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPAVGWrm, X86_INS_VPAVGW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPAVGWrr, X86_INS_VPAVGW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPBLENDDYrmi, X86_INS_VPBLENDD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBLENDDYrri, X86_INS_VPBLENDD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBLENDDrmi, X86_INS_VPBLENDD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBLENDDrri, X86_INS_VPBLENDD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBLENDMDZrm, X86_INS_VPBLENDMD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPBLENDMDZrr, X86_INS_VPBLENDMD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPBLENDMQZrm, X86_INS_VPBLENDMQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPBLENDMQZrr, X86_INS_VPBLENDMQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPBLENDVBYrm, X86_INS_VPBLENDVB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBLENDVBYrr, X86_INS_VPBLENDVB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBLENDVBrm, X86_INS_VPBLENDVB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPBLENDVBrr, X86_INS_VPBLENDVB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPBLENDWYrmi, X86_INS_VPBLENDW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBLENDWYrri, X86_INS_VPBLENDW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBLENDWrmi, X86_INS_VPBLENDW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPBLENDWrri, X86_INS_VPBLENDW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPBROADCASTBYrm, X86_INS_VPBROADCASTB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBROADCASTBYrr, X86_INS_VPBROADCASTB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBROADCASTBrm, X86_INS_VPBROADCASTB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBROADCASTBrr, X86_INS_VPBROADCASTB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBROADCASTDYrm, X86_INS_VPBROADCASTD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBROADCASTDYrr, X86_INS_VPBROADCASTD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBROADCASTDZkrm, X86_INS_VPBROADCASTD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPBROADCASTDZkrr, X86_INS_VPBROADCASTD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPBROADCASTDZrm, X86_INS_VPBROADCASTD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPBROADCASTDZrr, X86_INS_VPBROADCASTD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPBROADCASTDrZkrr, X86_INS_VPBROADCASTD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPBROADCASTDrZrr, X86_INS_VPBROADCASTD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPBROADCASTDrm, X86_INS_VPBROADCASTD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBROADCASTDrr, X86_INS_VPBROADCASTD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBROADCASTMB2Qrr, X86_INS_VPBROADCASTMB2Q, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPBROADCASTMW2Drr, X86_INS_VPBROADCASTMW2D, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPBROADCASTQYrm, X86_INS_VPBROADCASTQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBROADCASTQYrr, X86_INS_VPBROADCASTQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBROADCASTQZkrm, X86_INS_VPBROADCASTQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPBROADCASTQZkrr, X86_INS_VPBROADCASTQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPBROADCASTQZrm, X86_INS_VPBROADCASTQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPBROADCASTQZrr, X86_INS_VPBROADCASTQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPBROADCASTQrZkrr, X86_INS_VPBROADCASTQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPBROADCASTQrZrr, X86_INS_VPBROADCASTQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPBROADCASTQrm, X86_INS_VPBROADCASTQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBROADCASTQrr, X86_INS_VPBROADCASTQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBROADCASTWYrm, X86_INS_VPBROADCASTW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBROADCASTWYrr, X86_INS_VPBROADCASTW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBROADCASTWrm, X86_INS_VPBROADCASTW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPBROADCASTWrr, X86_INS_VPBROADCASTW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPCLMULQDQrm, X86_INS_VPCLMULQDQ, { 0 }, { 0 }, { X86_GRP_AVX, X86_GRP_PCLMUL, 0 } },
	{ X86_VPCLMULQDQrr, X86_INS_VPCLMULQDQ, { 0 }, { 0 }, { X86_GRP_AVX, X86_GRP_PCLMUL, 0 } },
	{ X86_VPCMOVmr, X86_INS_VPCMOV, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPCMOVmrY, X86_INS_VPCMOV, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPCMOVrm, X86_INS_VPCMOV, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPCMOVrmY, X86_INS_VPCMOV, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPCMOVrr, X86_INS_VPCMOV, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPCMOVrrY, X86_INS_VPCMOV, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPCMPDZrmi, X86_INS_VPCMPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPDZrmi_alt, X86_INS_VPCMPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPDZrri, X86_INS_VPCMPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPDZrri_alt, X86_INS_VPCMPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPEQBYrm, X86_INS_VPCMPEQB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPCMPEQBYrr, X86_INS_VPCMPEQB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPCMPEQBrm, X86_INS_VPCMPEQB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPEQBrr, X86_INS_VPCMPEQB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPEQDYrm, X86_INS_VPCMPEQD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPCMPEQDYrr, X86_INS_VPCMPEQD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPCMPEQDZrm, X86_INS_VPCMPEQD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPEQDZrr, X86_INS_VPCMPEQD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPEQDrm, X86_INS_VPCMPEQD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPEQDrr, X86_INS_VPCMPEQD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPEQQYrm, X86_INS_VPCMPEQQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPCMPEQQYrr, X86_INS_VPCMPEQQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPCMPEQQZrm, X86_INS_VPCMPEQQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPEQQZrr, X86_INS_VPCMPEQQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPEQQrm, X86_INS_VPCMPEQQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPEQQrr, X86_INS_VPCMPEQQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPEQWYrm, X86_INS_VPCMPEQW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPCMPEQWYrr, X86_INS_VPCMPEQW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPCMPEQWrm, X86_INS_VPCMPEQW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPEQWrr, X86_INS_VPCMPEQW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPESTRIrm, X86_INS_VPCMPESTRI, { X86_REG_EAX, X86_REG_EDX, 0 }, { X86_REG_ECX, X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPESTRIrr, X86_INS_VPCMPESTRI, { X86_REG_EAX, X86_REG_EDX, 0 }, { X86_REG_ECX, X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPESTRM128rm, X86_INS_VPCMPESTRM, { X86_REG_EAX, X86_REG_EDX, 0 }, { X86_REG_XMM0, X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPESTRM128rr, X86_INS_VPCMPESTRM, { X86_REG_EAX, X86_REG_EDX, 0 }, { X86_REG_XMM0, X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPGTBYrm, X86_INS_VPCMPGTB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPCMPGTBYrr, X86_INS_VPCMPGTB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPCMPGTBrm, X86_INS_VPCMPGTB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPGTBrr, X86_INS_VPCMPGTB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPGTDYrm, X86_INS_VPCMPGTD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPCMPGTDYrr, X86_INS_VPCMPGTD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPCMPGTDZrm, X86_INS_VPCMPGTD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPGTDZrr, X86_INS_VPCMPGTD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPGTDrm, X86_INS_VPCMPGTD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPGTDrr, X86_INS_VPCMPGTD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPGTQYrm, X86_INS_VPCMPGTQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPCMPGTQYrr, X86_INS_VPCMPGTQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPCMPGTQZrm, X86_INS_VPCMPGTQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPGTQZrr, X86_INS_VPCMPGTQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPGTQrm, X86_INS_VPCMPGTQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPGTQrr, X86_INS_VPCMPGTQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPGTWYrm, X86_INS_VPCMPGTW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPCMPGTWYrr, X86_INS_VPCMPGTW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPCMPGTWrm, X86_INS_VPCMPGTW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPGTWrr, X86_INS_VPCMPGTW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPISTRIrm, X86_INS_VPCMPISTRI, { 0 }, { X86_REG_ECX, X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPISTRIrr, X86_INS_VPCMPISTRI, { 0 }, { X86_REG_ECX, X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPISTRM128rm, X86_INS_VPCMPISTRM, { 0 }, { X86_REG_XMM0, X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPISTRM128rr, X86_INS_VPCMPISTRM, { 0 }, { X86_REG_XMM0, X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPCMPQZrmi, X86_INS_VPCMPQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPQZrmi_alt, X86_INS_VPCMPQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPQZrri, X86_INS_VPCMPQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPQZrri_alt, X86_INS_VPCMPQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPUDZrmi, X86_INS_VPCMPUD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPUDZrmi_alt, X86_INS_VPCMPUD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPUDZrri, X86_INS_VPCMPUD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPUDZrri_alt, X86_INS_VPCMPUD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPUQZrmi, X86_INS_VPCMPUQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPUQZrmi_alt, X86_INS_VPCMPUQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPUQZrri, X86_INS_VPCMPUQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCMPUQZrri_alt, X86_INS_VPCMPUQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPCOMBmi, X86_INS_VPCOMB, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPCOMBri, X86_INS_VPCOMB, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPCOMDmi, X86_INS_VPCOMD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPCOMDri, X86_INS_VPCOMD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPCOMQmi, X86_INS_VPCOMQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPCOMQri, X86_INS_VPCOMQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPCOMUBmi, X86_INS_VPCOMUB, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPCOMUBri, X86_INS_VPCOMUB, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPCOMUDmi, X86_INS_VPCOMUD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPCOMUDri, X86_INS_VPCOMUD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPCOMUQmi, X86_INS_VPCOMUQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPCOMUQri, X86_INS_VPCOMUQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPCOMUWmi, X86_INS_VPCOMUW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPCOMUWri, X86_INS_VPCOMUW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPCOMWmi, X86_INS_VPCOMW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPCOMWri, X86_INS_VPCOMW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPERM2F128rm, X86_INS_VPERM2F128, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPERM2F128rr, X86_INS_VPERM2F128, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPERM2I128rm, X86_INS_VPERM2I128, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPERM2I128rr, X86_INS_VPERM2I128, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPERMDYrm, X86_INS_VPERMD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPERMDYrr, X86_INS_VPERMD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPERMDZrm, X86_INS_VPERMD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMDZrr, X86_INS_VPERMD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMI2Drm, X86_INS_VPERMI2D, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMI2Drr, X86_INS_VPERMI2D, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMI2PDrm, X86_INS_VPERMI2PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMI2PDrr, X86_INS_VPERMI2PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMI2PSrm, X86_INS_VPERMI2PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMI2PSrr, X86_INS_VPERMI2PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMI2Qrm, X86_INS_VPERMI2Q, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMI2Qrr, X86_INS_VPERMI2Q, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMIL2PDmr, X86_INS_VPERMIL2PD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPERMIL2PDmrY, X86_INS_VPERMIL2PD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPERMIL2PDrm, X86_INS_VPERMIL2PD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPERMIL2PDrmY, X86_INS_VPERMIL2PD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPERMIL2PDrr, X86_INS_VPERMIL2PD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPERMIL2PDrrY, X86_INS_VPERMIL2PD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPERMIL2PSmr, X86_INS_VPERMIL2PS, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPERMIL2PSmrY, X86_INS_VPERMIL2PS, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPERMIL2PSrm, X86_INS_VPERMIL2PS, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPERMIL2PSrmY, X86_INS_VPERMIL2PS, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPERMIL2PSrr, X86_INS_VPERMIL2PS, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPERMIL2PSrrY, X86_INS_VPERMIL2PS, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPERMILPDYmi, X86_INS_VPERMILPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPERMILPDYri, X86_INS_VPERMILPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPERMILPDYrm, X86_INS_VPERMILPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPERMILPDYrr, X86_INS_VPERMILPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPERMILPDZmi, X86_INS_VPERMILPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMILPDZri, X86_INS_VPERMILPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMILPDmi, X86_INS_VPERMILPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPERMILPDri, X86_INS_VPERMILPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPERMILPDrm, X86_INS_VPERMILPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPERMILPDrr, X86_INS_VPERMILPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPERMILPSYmi, X86_INS_VPERMILPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPERMILPSYri, X86_INS_VPERMILPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPERMILPSYrm, X86_INS_VPERMILPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPERMILPSYrr, X86_INS_VPERMILPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPERMILPSZmi, X86_INS_VPERMILPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMILPSZri, X86_INS_VPERMILPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMILPSmi, X86_INS_VPERMILPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPERMILPSri, X86_INS_VPERMILPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPERMILPSrm, X86_INS_VPERMILPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPERMILPSrr, X86_INS_VPERMILPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPERMPDYmi, X86_INS_VPERMPD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPERMPDYri, X86_INS_VPERMPD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPERMPDZmi, X86_INS_VPERMPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMPDZri, X86_INS_VPERMPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMPDZrm, X86_INS_VPERMPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMPDZrr, X86_INS_VPERMPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMPSYrm, X86_INS_VPERMPS, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPERMPSYrr, X86_INS_VPERMPS, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPERMPSZrm, X86_INS_VPERMPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMPSZrr, X86_INS_VPERMPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMQYmi, X86_INS_VPERMQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPERMQYri, X86_INS_VPERMQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPERMQZmi, X86_INS_VPERMQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMQZri, X86_INS_VPERMQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMQZrm, X86_INS_VPERMQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPERMQZrr, X86_INS_VPERMQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPEXTRBmr, X86_INS_VPEXTRB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPEXTRBrr, X86_INS_VPEXTRB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPEXTRBrr64, X86_INS_VPEXTRB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPEXTRDmr, X86_INS_VPEXTRD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPEXTRDrr, X86_INS_VPEXTRD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPEXTRQmr, X86_INS_VPEXTRQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPEXTRQrr, X86_INS_VPEXTRQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPEXTRWmr, X86_INS_VPEXTRW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPEXTRWri, X86_INS_VPEXTRW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPGATHERDDYrm, X86_INS_VPGATHERDD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPGATHERDDZrm, X86_INS_VPGATHERDD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPGATHERDDrm, X86_INS_VPGATHERDD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPGATHERDQYrm, X86_INS_VPGATHERDQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPGATHERDQZrm, X86_INS_VPGATHERDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPGATHERDQrm, X86_INS_VPGATHERDQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPGATHERQDYrm, X86_INS_VPGATHERQD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPGATHERQDZrm, X86_INS_VPGATHERQD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPGATHERQDrm, X86_INS_VPGATHERQD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPGATHERQQYrm, X86_INS_VPGATHERQQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPGATHERQQZrm, X86_INS_VPGATHERQQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPGATHERQQrm, X86_INS_VPGATHERQQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPHADDBDrm, X86_INS_VPHADDBD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDBDrr, X86_INS_VPHADDBD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDBQrm, X86_INS_VPHADDBQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDBQrr, X86_INS_VPHADDBQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDBWrm, X86_INS_VPHADDBW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDBWrr, X86_INS_VPHADDBW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDDQrm, X86_INS_VPHADDDQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDDQrr, X86_INS_VPHADDDQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDDYrm, X86_INS_VPHADDD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPHADDDYrr, X86_INS_VPHADDD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPHADDDrm, X86_INS_VPHADDD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPHADDDrr, X86_INS_VPHADDD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPHADDSWrm128, X86_INS_VPHADDSW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPHADDSWrm256, X86_INS_VPHADDSW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPHADDSWrr128, X86_INS_VPHADDSW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPHADDSWrr256, X86_INS_VPHADDSW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPHADDUBDrm, X86_INS_VPHADDUBD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDUBDrr, X86_INS_VPHADDUBD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDUBQrm, X86_INS_VPHADDUBQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDUBQrr, X86_INS_VPHADDUBQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDUBWrm, X86_INS_VPHADDUBW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDUBWrr, X86_INS_VPHADDUBW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDUDQrm, X86_INS_VPHADDUDQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDUDQrr, X86_INS_VPHADDUDQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDUWDrm, X86_INS_VPHADDUWD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDUWDrr, X86_INS_VPHADDUWD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDUWQrm, X86_INS_VPHADDUWQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDUWQrr, X86_INS_VPHADDUWQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDWDrm, X86_INS_VPHADDWD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDWDrr, X86_INS_VPHADDWD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDWQrm, X86_INS_VPHADDWQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDWQrr, X86_INS_VPHADDWQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHADDWYrm, X86_INS_VPHADDW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPHADDWYrr, X86_INS_VPHADDW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPHADDWrm, X86_INS_VPHADDW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPHADDWrr, X86_INS_VPHADDW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPHMINPOSUWrm128, X86_INS_VPHMINPOSUW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPHMINPOSUWrr128, X86_INS_VPHMINPOSUW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPHSUBBWrm, X86_INS_VPHSUBBW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHSUBBWrr, X86_INS_VPHSUBBW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHSUBDQrm, X86_INS_VPHSUBDQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHSUBDQrr, X86_INS_VPHSUBDQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHSUBDYrm, X86_INS_VPHSUBD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPHSUBDYrr, X86_INS_VPHSUBD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPHSUBDrm, X86_INS_VPHSUBD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPHSUBDrr, X86_INS_VPHSUBD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPHSUBSWrm128, X86_INS_VPHSUBSW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPHSUBSWrm256, X86_INS_VPHSUBSW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPHSUBSWrr128, X86_INS_VPHSUBSW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPHSUBSWrr256, X86_INS_VPHSUBSW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPHSUBWDrm, X86_INS_VPHSUBWD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHSUBWDrr, X86_INS_VPHSUBWD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPHSUBWYrm, X86_INS_VPHSUBW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPHSUBWYrr, X86_INS_VPHSUBW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPHSUBWrm, X86_INS_VPHSUBW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPHSUBWrr, X86_INS_VPHSUBW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPINSRBrm, X86_INS_VPINSRB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPINSRBrr, X86_INS_VPINSRB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPINSRDrm, X86_INS_VPINSRD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPINSRDrr, X86_INS_VPINSRD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPINSRQrm, X86_INS_VPINSRQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPINSRQrr, X86_INS_VPINSRQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPINSRWrmi, X86_INS_VPINSRW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPINSRWrr64i, X86_INS_VPINSRW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPINSRWrri, X86_INS_VPINSRW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMACSDDrm, X86_INS_VPMACSDD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMACSDDrr, X86_INS_VPMACSDD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMACSDQHrm, X86_INS_VPMACSDQH, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMACSDQHrr, X86_INS_VPMACSDQH, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMACSDQLrm, X86_INS_VPMACSDQL, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMACSDQLrr, X86_INS_VPMACSDQL, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMACSSDDrm, X86_INS_VPMACSSDD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMACSSDDrr, X86_INS_VPMACSSDD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMACSSDQHrm, X86_INS_VPMACSSDQH, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMACSSDQHrr, X86_INS_VPMACSSDQH, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMACSSDQLrm, X86_INS_VPMACSSDQL, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMACSSDQLrr, X86_INS_VPMACSSDQL, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMACSSWDrm, X86_INS_VPMACSSWD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMACSSWDrr, X86_INS_VPMACSSWD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMACSSWWrm, X86_INS_VPMACSSWW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMACSSWWrr, X86_INS_VPMACSSWW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMACSWDrm, X86_INS_VPMACSWD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMACSWDrr, X86_INS_VPMACSWD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMACSWWrm, X86_INS_VPMACSWW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMACSWWrr, X86_INS_VPMACSWW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMADCSSWDrm, X86_INS_VPMADCSSWD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMADCSSWDrr, X86_INS_VPMADCSSWD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMADCSWDrm, X86_INS_VPMADCSWD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMADCSWDrr, X86_INS_VPMADCSWD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPMADDUBSWrm128, X86_INS_VPMADDUBSW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMADDUBSWrm256, X86_INS_VPMADDUBSW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMADDUBSWrr128, X86_INS_VPMADDUBSW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMADDUBSWrr256, X86_INS_VPMADDUBSW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMADDWDYrm, X86_INS_VPMADDWD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMADDWDYrr, X86_INS_VPMADDWD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMADDWDrm, X86_INS_VPMADDWD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMADDWDrr, X86_INS_VPMADDWD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMASKMOVDYmr, X86_INS_VPMASKMOVD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMASKMOVDYrm, X86_INS_VPMASKMOVD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMASKMOVDmr, X86_INS_VPMASKMOVD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMASKMOVDrm, X86_INS_VPMASKMOVD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMASKMOVQYmr, X86_INS_VPMASKMOVQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMASKMOVQYrm, X86_INS_VPMASKMOVQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMASKMOVQmr, X86_INS_VPMASKMOVQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMASKMOVQrm, X86_INS_VPMASKMOVQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMAXSBYrm, X86_INS_VPMAXSB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMAXSBYrr, X86_INS_VPMAXSB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMAXSBrm, X86_INS_VPMAXSB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMAXSBrr, X86_INS_VPMAXSB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMAXSDYrm, X86_INS_VPMAXSD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMAXSDYrr, X86_INS_VPMAXSD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMAXSDrm, X86_INS_VPMAXSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMAXSDrr, X86_INS_VPMAXSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMAXSWYrm, X86_INS_VPMAXSW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMAXSWYrr, X86_INS_VPMAXSW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMAXSWrm, X86_INS_VPMAXSW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMAXSWrr, X86_INS_VPMAXSW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMAXUBYrm, X86_INS_VPMAXUB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMAXUBYrr, X86_INS_VPMAXUB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMAXUBrm, X86_INS_VPMAXUB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMAXUBrr, X86_INS_VPMAXUB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMAXUDYrm, X86_INS_VPMAXUD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMAXUDYrr, X86_INS_VPMAXUD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMAXUDrm, X86_INS_VPMAXUD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMAXUDrr, X86_INS_VPMAXUD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMAXUWYrm, X86_INS_VPMAXUW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMAXUWYrr, X86_INS_VPMAXUW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMAXUWrm, X86_INS_VPMAXUW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMAXUWrr, X86_INS_VPMAXUW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMINSBYrm, X86_INS_VPMINSB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMINSBYrr, X86_INS_VPMINSB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMINSBrm, X86_INS_VPMINSB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMINSBrr, X86_INS_VPMINSB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMINSDYrm, X86_INS_VPMINSD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMINSDYrr, X86_INS_VPMINSD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMINSDrm, X86_INS_VPMINSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMINSDrr, X86_INS_VPMINSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMINSWYrm, X86_INS_VPMINSW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMINSWYrr, X86_INS_VPMINSW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMINSWrm, X86_INS_VPMINSW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMINSWrr, X86_INS_VPMINSW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMINUBYrm, X86_INS_VPMINUB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMINUBYrr, X86_INS_VPMINUB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMINUBrm, X86_INS_VPMINUB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMINUBrr, X86_INS_VPMINUB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMINUDYrm, X86_INS_VPMINUD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMINUDYrr, X86_INS_VPMINUD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMINUDrm, X86_INS_VPMINUD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMINUDrr, X86_INS_VPMINUD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMINUWYrm, X86_INS_VPMINUW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMINUWYrr, X86_INS_VPMINUW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMINUWrm, X86_INS_VPMINUW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMINUWrr, X86_INS_VPMINUW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVDBkrr, X86_INS_VPMOVDB, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVDBmr, X86_INS_VPMOVDB, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVDBrr, X86_INS_VPMOVDB, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVDWkrr, X86_INS_VPMOVDW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVDWmr, X86_INS_VPMOVDW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVDWrr, X86_INS_VPMOVDW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVMSKBYr64r, X86_INS_VPMOVMSKB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVMSKBYrr, X86_INS_VPMOVMSKB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVMSKBr64r, X86_INS_VPMOVMSKB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVMSKBrr, X86_INS_VPMOVMSKB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVQBkrr, X86_INS_VPMOVQB, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVQBmr, X86_INS_VPMOVQB, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVQBrr, X86_INS_VPMOVQB, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVQDkrr, X86_INS_VPMOVQD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVQDmr, X86_INS_VPMOVQD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVQDrr, X86_INS_VPMOVQD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVQWkrr, X86_INS_VPMOVQW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVQWmr, X86_INS_VPMOVQW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVQWrr, X86_INS_VPMOVQW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSDBkrr, X86_INS_VPMOVSDB, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSDBmr, X86_INS_VPMOVSDB, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSDBrr, X86_INS_VPMOVSDB, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSDWkrr, X86_INS_VPMOVSDW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSDWmr, X86_INS_VPMOVSDW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSDWrr, X86_INS_VPMOVSDW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSQBkrr, X86_INS_VPMOVSQB, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSQBmr, X86_INS_VPMOVSQB, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSQBrr, X86_INS_VPMOVSQB, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSQDkrr, X86_INS_VPMOVSQD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSQDmr, X86_INS_VPMOVSQD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSQDrr, X86_INS_VPMOVSQD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSQWkrr, X86_INS_VPMOVSQW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSQWmr, X86_INS_VPMOVSQW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSQWrr, X86_INS_VPMOVSQW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSXBDYrm, X86_INS_VPMOVSXBD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVSXBDYrr, X86_INS_VPMOVSXBD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVSXBDZrm, X86_INS_VPMOVSXBD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSXBDZrr, X86_INS_VPMOVSXBD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSXBDrm, X86_INS_VPMOVSXBD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVSXBDrr, X86_INS_VPMOVSXBD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVSXBQYrm, X86_INS_VPMOVSXBQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVSXBQYrr, X86_INS_VPMOVSXBQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVSXBQZrm, X86_INS_VPMOVSXBQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSXBQZrr, X86_INS_VPMOVSXBQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSXBQrm, X86_INS_VPMOVSXBQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVSXBQrr, X86_INS_VPMOVSXBQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVSXBWYrm, X86_INS_VPMOVSXBW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVSXBWYrr, X86_INS_VPMOVSXBW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVSXBWrm, X86_INS_VPMOVSXBW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVSXBWrr, X86_INS_VPMOVSXBW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVSXDQYrm, X86_INS_VPMOVSXDQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVSXDQYrr, X86_INS_VPMOVSXDQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVSXDQZrm, X86_INS_VPMOVSXDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSXDQZrr, X86_INS_VPMOVSXDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSXDQrm, X86_INS_VPMOVSXDQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVSXDQrr, X86_INS_VPMOVSXDQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVSXWDYrm, X86_INS_VPMOVSXWD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVSXWDYrr, X86_INS_VPMOVSXWD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVSXWDZrm, X86_INS_VPMOVSXWD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSXWDZrr, X86_INS_VPMOVSXWD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSXWDrm, X86_INS_VPMOVSXWD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVSXWDrr, X86_INS_VPMOVSXWD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVSXWQYrm, X86_INS_VPMOVSXWQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVSXWQYrr, X86_INS_VPMOVSXWQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVSXWQZrm, X86_INS_VPMOVSXWQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSXWQZrr, X86_INS_VPMOVSXWQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVSXWQrm, X86_INS_VPMOVSXWQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVSXWQrr, X86_INS_VPMOVSXWQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVUSDBkrr, X86_INS_VPMOVUSDB, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVUSDBmr, X86_INS_VPMOVUSDB, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVUSDBrr, X86_INS_VPMOVUSDB, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVUSDWkrr, X86_INS_VPMOVUSDW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVUSDWmr, X86_INS_VPMOVUSDW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVUSDWrr, X86_INS_VPMOVUSDW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVUSQBkrr, X86_INS_VPMOVUSQB, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVUSQBmr, X86_INS_VPMOVUSQB, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVUSQBrr, X86_INS_VPMOVUSQB, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVUSQDkrr, X86_INS_VPMOVUSQD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVUSQDmr, X86_INS_VPMOVUSQD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVUSQDrr, X86_INS_VPMOVUSQD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVUSQWkrr, X86_INS_VPMOVUSQW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVUSQWmr, X86_INS_VPMOVUSQW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVUSQWrr, X86_INS_VPMOVUSQW, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVZXBDYrm, X86_INS_VPMOVZXBD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVZXBDYrr, X86_INS_VPMOVZXBD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVZXBDZrm, X86_INS_VPMOVZXBD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVZXBDZrr, X86_INS_VPMOVZXBD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVZXBDrm, X86_INS_VPMOVZXBD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVZXBDrr, X86_INS_VPMOVZXBD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVZXBQYrm, X86_INS_VPMOVZXBQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVZXBQYrr, X86_INS_VPMOVZXBQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVZXBQZrm, X86_INS_VPMOVZXBQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVZXBQZrr, X86_INS_VPMOVZXBQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVZXBQrm, X86_INS_VPMOVZXBQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVZXBQrr, X86_INS_VPMOVZXBQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVZXBWYrm, X86_INS_VPMOVZXBW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVZXBWYrr, X86_INS_VPMOVZXBW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVZXBWrm, X86_INS_VPMOVZXBW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVZXBWrr, X86_INS_VPMOVZXBW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVZXDQYrm, X86_INS_VPMOVZXDQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVZXDQYrr, X86_INS_VPMOVZXDQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVZXDQZrm, X86_INS_VPMOVZXDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVZXDQZrr, X86_INS_VPMOVZXDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVZXDQrm, X86_INS_VPMOVZXDQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVZXDQrr, X86_INS_VPMOVZXDQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVZXWDYrm, X86_INS_VPMOVZXWD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVZXWDYrr, X86_INS_VPMOVZXWD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVZXWDZrm, X86_INS_VPMOVZXWD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVZXWDZrr, X86_INS_VPMOVZXWD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVZXWDrm, X86_INS_VPMOVZXWD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVZXWDrr, X86_INS_VPMOVZXWD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVZXWQYrm, X86_INS_VPMOVZXWQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVZXWQYrr, X86_INS_VPMOVZXWQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMOVZXWQZrm, X86_INS_VPMOVZXWQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVZXWQZrr, X86_INS_VPMOVZXWQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMOVZXWQrm, X86_INS_VPMOVZXWQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMOVZXWQrr, X86_INS_VPMOVZXWQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMULDQYrm, X86_INS_VPMULDQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMULDQYrr, X86_INS_VPMULDQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMULDQZrm, X86_INS_VPMULDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMULDQZrr, X86_INS_VPMULDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMULDQrm, X86_INS_VPMULDQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMULDQrr, X86_INS_VPMULDQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMULHRSWrm128, X86_INS_VPMULHRSW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMULHRSWrm256, X86_INS_VPMULHRSW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMULHRSWrr128, X86_INS_VPMULHRSW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMULHRSWrr256, X86_INS_VPMULHRSW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMULHUWYrm, X86_INS_VPMULHUW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMULHUWYrr, X86_INS_VPMULHUW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMULHUWrm, X86_INS_VPMULHUW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMULHUWrr, X86_INS_VPMULHUW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMULHWYrm, X86_INS_VPMULHW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMULHWYrr, X86_INS_VPMULHW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMULHWrm, X86_INS_VPMULHW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMULHWrr, X86_INS_VPMULHW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMULLDYrm, X86_INS_VPMULLD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMULLDYrr, X86_INS_VPMULLD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMULLDZrm, X86_INS_VPMULLD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMULLDZrmb, X86_INS_VPMULLD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMULLDZrr, X86_INS_VPMULLD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMULLDrm, X86_INS_VPMULLD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMULLDrr, X86_INS_VPMULLD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMULLWYrm, X86_INS_VPMULLW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMULLWYrr, X86_INS_VPMULLW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMULLWrm, X86_INS_VPMULLW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMULLWrr, X86_INS_VPMULLW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMULUDQYrm, X86_INS_VPMULUDQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMULUDQYrr, X86_INS_VPMULUDQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPMULUDQZrm, X86_INS_VPMULUDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMULUDQZrr, X86_INS_VPMULUDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPMULUDQrm, X86_INS_VPMULUDQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPMULUDQrr, X86_INS_VPMULUDQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPORDZrm, X86_INS_VPORD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPORDZrmb, X86_INS_VPORD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPORDZrr, X86_INS_VPORD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPORQZrm, X86_INS_VPORQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPORQZrmb, X86_INS_VPORQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPORQZrr, X86_INS_VPORQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPORYrm, X86_INS_VPOR, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPORYrr, X86_INS_VPOR, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPORrm, X86_INS_VPOR, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPORrr, X86_INS_VPOR, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPPERMmr, X86_INS_VPPERM, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPPERMrm, X86_INS_VPPERM, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPPERMrr, X86_INS_VPPERM, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPROTBmi, X86_INS_VPROTB, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPROTBmr, X86_INS_VPROTB, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPROTBri, X86_INS_VPROTB, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPROTBrm, X86_INS_VPROTB, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPROTBrr, X86_INS_VPROTB, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPROTDmi, X86_INS_VPROTD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPROTDmr, X86_INS_VPROTD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPROTDri, X86_INS_VPROTD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPROTDrm, X86_INS_VPROTD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPROTDrr, X86_INS_VPROTD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPROTQmi, X86_INS_VPROTQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPROTQmr, X86_INS_VPROTQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPROTQri, X86_INS_VPROTQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPROTQrm, X86_INS_VPROTQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPROTQrr, X86_INS_VPROTQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPROTWmi, X86_INS_VPROTW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPROTWmr, X86_INS_VPROTW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPROTWri, X86_INS_VPROTW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPROTWrm, X86_INS_VPROTW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPROTWrr, X86_INS_VPROTW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSADBWYrm, X86_INS_VPSADBW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSADBWYrr, X86_INS_VPSADBW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSADBWrm, X86_INS_VPSADBW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSADBWrr, X86_INS_VPSADBW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSCATTERDDZmr, X86_INS_VPSCATTERDD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSCATTERDQZmr, X86_INS_VPSCATTERDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSCATTERQDZmr, X86_INS_VPSCATTERQD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSCATTERQQZmr, X86_INS_VPSCATTERQQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSHABmr, X86_INS_VPSHAB, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHABrm, X86_INS_VPSHAB, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHABrr, X86_INS_VPSHAB, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHADmr, X86_INS_VPSHAD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHADrm, X86_INS_VPSHAD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHADrr, X86_INS_VPSHAD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHAQmr, X86_INS_VPSHAQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHAQrm, X86_INS_VPSHAQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHAQrr, X86_INS_VPSHAQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHAWmr, X86_INS_VPSHAW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHAWrm, X86_INS_VPSHAW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHAWrr, X86_INS_VPSHAW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHLBmr, X86_INS_VPSHLB, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHLBrm, X86_INS_VPSHLB, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHLBrr, X86_INS_VPSHLB, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHLDmr, X86_INS_VPSHLD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHLDrm, X86_INS_VPSHLD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHLDrr, X86_INS_VPSHLD, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHLQmr, X86_INS_VPSHLQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHLQrm, X86_INS_VPSHLQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHLQrr, X86_INS_VPSHLQ, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHLWmr, X86_INS_VPSHLW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHLWrm, X86_INS_VPSHLW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHLWrr, X86_INS_VPSHLW, { 0 }, { 0 }, { X86_GRP_XOP, 0 } },
	{ X86_VPSHUFBYrm, X86_INS_VPSHUFB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSHUFBYrr, X86_INS_VPSHUFB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSHUFBrm, X86_INS_VPSHUFB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSHUFBrr, X86_INS_VPSHUFB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSHUFDYmi, X86_INS_VPSHUFD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSHUFDYri, X86_INS_VPSHUFD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSHUFDZmi, X86_INS_VPSHUFD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSHUFDZri, X86_INS_VPSHUFD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSHUFDmi, X86_INS_VPSHUFD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSHUFDri, X86_INS_VPSHUFD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSHUFHWYmi, X86_INS_VPSHUFHW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSHUFHWYri, X86_INS_VPSHUFHW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSHUFHWmi, X86_INS_VPSHUFHW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSHUFHWri, X86_INS_VPSHUFHW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSHUFLWYmi, X86_INS_VPSHUFLW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSHUFLWYri, X86_INS_VPSHUFLW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSHUFLWmi, X86_INS_VPSHUFLW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSHUFLWri, X86_INS_VPSHUFLW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSIGNBYrm, X86_INS_VPSIGNB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSIGNBYrr, X86_INS_VPSIGNB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSIGNBrm, X86_INS_VPSIGNB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSIGNBrr, X86_INS_VPSIGNB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSIGNDYrm, X86_INS_VPSIGND, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSIGNDYrr, X86_INS_VPSIGND, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSIGNDrm, X86_INS_VPSIGND, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSIGNDrr, X86_INS_VPSIGND, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSIGNWYrm, X86_INS_VPSIGNW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSIGNWYrr, X86_INS_VPSIGNW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSIGNWrm, X86_INS_VPSIGNW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSIGNWrr, X86_INS_VPSIGNW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSLLDQYri, X86_INS_VPSLLDQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSLLDQri, X86_INS_VPSLLDQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSLLDYri, X86_INS_VPSLLD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSLLDYrm, X86_INS_VPSLLD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSLLDYrr, X86_INS_VPSLLD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSLLDZmi, X86_INS_VPSLLD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSLLDZmik, X86_INS_VPSLLD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSLLDZri, X86_INS_VPSLLD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSLLDZrik, X86_INS_VPSLLD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSLLDZrm, X86_INS_VPSLLD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSLLDZrmk, X86_INS_VPSLLD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSLLDZrr, X86_INS_VPSLLD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSLLDZrrk, X86_INS_VPSLLD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSLLDri, X86_INS_VPSLLD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSLLDrm, X86_INS_VPSLLD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSLLDrr, X86_INS_VPSLLD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSLLQYri, X86_INS_VPSLLQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSLLQYrm, X86_INS_VPSLLQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSLLQYrr, X86_INS_VPSLLQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSLLQZmi, X86_INS_VPSLLQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSLLQZmik, X86_INS_VPSLLQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSLLQZri, X86_INS_VPSLLQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSLLQZrik, X86_INS_VPSLLQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSLLQZrm, X86_INS_VPSLLQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSLLQZrmk, X86_INS_VPSLLQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSLLQZrr, X86_INS_VPSLLQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSLLQZrrk, X86_INS_VPSLLQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSLLQri, X86_INS_VPSLLQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSLLQrm, X86_INS_VPSLLQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSLLQrr, X86_INS_VPSLLQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSLLVDYrm, X86_INS_VPSLLVD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSLLVDYrr, X86_INS_VPSLLVD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSLLVDZrm, X86_INS_VPSLLVD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSLLVDZrr, X86_INS_VPSLLVD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSLLVDrm, X86_INS_VPSLLVD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSLLVDrr, X86_INS_VPSLLVD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSLLVQYrm, X86_INS_VPSLLVQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSLLVQYrr, X86_INS_VPSLLVQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSLLVQZrm, X86_INS_VPSLLVQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSLLVQZrr, X86_INS_VPSLLVQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSLLVQrm, X86_INS_VPSLLVQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSLLVQrr, X86_INS_VPSLLVQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSLLWYri, X86_INS_VPSLLW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSLLWYrm, X86_INS_VPSLLW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSLLWYrr, X86_INS_VPSLLW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSLLWri, X86_INS_VPSLLW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSLLWrm, X86_INS_VPSLLW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSLLWrr, X86_INS_VPSLLW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSRADYri, X86_INS_VPSRAD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRADYrm, X86_INS_VPSRAD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRADYrr, X86_INS_VPSRAD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRADZmi, X86_INS_VPSRAD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRADZmik, X86_INS_VPSRAD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRADZri, X86_INS_VPSRAD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRADZrik, X86_INS_VPSRAD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRADZrm, X86_INS_VPSRAD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRADZrmk, X86_INS_VPSRAD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRADZrr, X86_INS_VPSRAD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRADZrrk, X86_INS_VPSRAD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRADri, X86_INS_VPSRAD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSRADrm, X86_INS_VPSRAD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSRADrr, X86_INS_VPSRAD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSRAQZmi, X86_INS_VPSRAQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRAQZmik, X86_INS_VPSRAQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRAQZri, X86_INS_VPSRAQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRAQZrik, X86_INS_VPSRAQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRAQZrm, X86_INS_VPSRAQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRAQZrmk, X86_INS_VPSRAQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRAQZrr, X86_INS_VPSRAQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRAQZrrk, X86_INS_VPSRAQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRAVDYrm, X86_INS_VPSRAVD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRAVDYrr, X86_INS_VPSRAVD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRAVDZrm, X86_INS_VPSRAVD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRAVDZrr, X86_INS_VPSRAVD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRAVDrm, X86_INS_VPSRAVD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRAVDrr, X86_INS_VPSRAVD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRAVQZrm, X86_INS_VPSRAVQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRAVQZrr, X86_INS_VPSRAVQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRAWYri, X86_INS_VPSRAW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRAWYrm, X86_INS_VPSRAW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRAWYrr, X86_INS_VPSRAW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRAWri, X86_INS_VPSRAW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSRAWrm, X86_INS_VPSRAW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSRAWrr, X86_INS_VPSRAW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSRLDQYri, X86_INS_VPSRLDQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRLDQri, X86_INS_VPSRLDQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSRLDYri, X86_INS_VPSRLD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRLDYrm, X86_INS_VPSRLD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRLDYrr, X86_INS_VPSRLD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRLDZmi, X86_INS_VPSRLD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRLDZmik, X86_INS_VPSRLD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRLDZri, X86_INS_VPSRLD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRLDZrik, X86_INS_VPSRLD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRLDZrm, X86_INS_VPSRLD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRLDZrmk, X86_INS_VPSRLD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRLDZrr, X86_INS_VPSRLD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRLDZrrk, X86_INS_VPSRLD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRLDri, X86_INS_VPSRLD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSRLDrm, X86_INS_VPSRLD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSRLDrr, X86_INS_VPSRLD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSRLQYri, X86_INS_VPSRLQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRLQYrm, X86_INS_VPSRLQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRLQYrr, X86_INS_VPSRLQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRLQZmi, X86_INS_VPSRLQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRLQZmik, X86_INS_VPSRLQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRLQZri, X86_INS_VPSRLQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRLQZrik, X86_INS_VPSRLQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRLQZrm, X86_INS_VPSRLQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRLQZrmk, X86_INS_VPSRLQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRLQZrr, X86_INS_VPSRLQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRLQZrrk, X86_INS_VPSRLQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRLQri, X86_INS_VPSRLQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSRLQrm, X86_INS_VPSRLQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSRLQrr, X86_INS_VPSRLQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSRLVDYrm, X86_INS_VPSRLVD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRLVDYrr, X86_INS_VPSRLVD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRLVDZrm, X86_INS_VPSRLVD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRLVDZrr, X86_INS_VPSRLVD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRLVDrm, X86_INS_VPSRLVD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRLVDrr, X86_INS_VPSRLVD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRLVQYrm, X86_INS_VPSRLVQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRLVQYrr, X86_INS_VPSRLVQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRLVQZrm, X86_INS_VPSRLVQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRLVQZrr, X86_INS_VPSRLVQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSRLVQrm, X86_INS_VPSRLVQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRLVQrr, X86_INS_VPSRLVQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRLWYri, X86_INS_VPSRLW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRLWYrm, X86_INS_VPSRLW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRLWYrr, X86_INS_VPSRLW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSRLWri, X86_INS_VPSRLW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSRLWrm, X86_INS_VPSRLW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSRLWrr, X86_INS_VPSRLW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSUBBYrm, X86_INS_VPSUBB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSUBBYrr, X86_INS_VPSUBB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSUBBrm, X86_INS_VPSUBB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSUBBrr, X86_INS_VPSUBB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSUBDYrm, X86_INS_VPSUBD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSUBDYrr, X86_INS_VPSUBD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSUBDZrm, X86_INS_VPSUBD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSUBDZrmb, X86_INS_VPSUBD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSUBDZrr, X86_INS_VPSUBD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSUBDrm, X86_INS_VPSUBD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSUBDrr, X86_INS_VPSUBD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSUBQYrm, X86_INS_VPSUBQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSUBQYrr, X86_INS_VPSUBQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSUBQZrm, X86_INS_VPSUBQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSUBQZrmb, X86_INS_VPSUBQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSUBQZrr, X86_INS_VPSUBQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPSUBQrm, X86_INS_VPSUBQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSUBQrr, X86_INS_VPSUBQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSUBSBYrm, X86_INS_VPSUBSB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSUBSBYrr, X86_INS_VPSUBSB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSUBSBrm, X86_INS_VPSUBSB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSUBSBrr, X86_INS_VPSUBSB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSUBSWYrm, X86_INS_VPSUBSW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSUBSWYrr, X86_INS_VPSUBSW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSUBSWrm, X86_INS_VPSUBSW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSUBSWrr, X86_INS_VPSUBSW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSUBUSBYrm, X86_INS_VPSUBUSB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSUBUSBYrr, X86_INS_VPSUBUSB, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSUBUSBrm, X86_INS_VPSUBUSB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSUBUSBrr, X86_INS_VPSUBUSB, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSUBUSWYrm, X86_INS_VPSUBUSW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSUBUSWYrr, X86_INS_VPSUBUSW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSUBUSWrm, X86_INS_VPSUBUSW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSUBUSWrr, X86_INS_VPSUBUSW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSUBWYrm, X86_INS_VPSUBW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSUBWYrr, X86_INS_VPSUBW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPSUBWrm, X86_INS_VPSUBW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPSUBWrr, X86_INS_VPSUBW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPTESTMDZrm, X86_INS_VPTESTMD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPTESTMDZrr, X86_INS_VPTESTMD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPTESTMQZrm, X86_INS_VPTESTMQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPTESTMQZrr, X86_INS_VPTESTMQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPTESTYrm, X86_INS_VPTEST, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPTESTYrr, X86_INS_VPTEST, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPTESTrm, X86_INS_VPTEST, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPTESTrr, X86_INS_VPTEST, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPUNPCKHBWYrm, X86_INS_VPUNPCKHBW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPUNPCKHBWYrr, X86_INS_VPUNPCKHBW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPUNPCKHBWrm, X86_INS_VPUNPCKHBW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPUNPCKHBWrr, X86_INS_VPUNPCKHBW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPUNPCKHDQYrm, X86_INS_VPUNPCKHDQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPUNPCKHDQYrr, X86_INS_VPUNPCKHDQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPUNPCKHDQZrm, X86_INS_VPUNPCKHDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPUNPCKHDQZrr, X86_INS_VPUNPCKHDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPUNPCKHDQrm, X86_INS_VPUNPCKHDQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPUNPCKHDQrr, X86_INS_VPUNPCKHDQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPUNPCKHQDQYrm, X86_INS_VPUNPCKHQDQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPUNPCKHQDQYrr, X86_INS_VPUNPCKHQDQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPUNPCKHQDQZrm, X86_INS_VPUNPCKHQDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPUNPCKHQDQZrr, X86_INS_VPUNPCKHQDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPUNPCKHQDQrm, X86_INS_VPUNPCKHQDQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPUNPCKHQDQrr, X86_INS_VPUNPCKHQDQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPUNPCKHWDYrm, X86_INS_VPUNPCKHWD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPUNPCKHWDYrr, X86_INS_VPUNPCKHWD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPUNPCKHWDrm, X86_INS_VPUNPCKHWD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPUNPCKHWDrr, X86_INS_VPUNPCKHWD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPUNPCKLBWYrm, X86_INS_VPUNPCKLBW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPUNPCKLBWYrr, X86_INS_VPUNPCKLBW, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPUNPCKLBWrm, X86_INS_VPUNPCKLBW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPUNPCKLBWrr, X86_INS_VPUNPCKLBW, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPUNPCKLDQYrm, X86_INS_VPUNPCKLDQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPUNPCKLDQYrr, X86_INS_VPUNPCKLDQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPUNPCKLDQZrm, X86_INS_VPUNPCKLDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPUNPCKLDQZrr, X86_INS_VPUNPCKLDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPUNPCKLDQrm, X86_INS_VPUNPCKLDQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPUNPCKLDQrr, X86_INS_VPUNPCKLDQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPUNPCKLQDQYrm, X86_INS_VPUNPCKLQDQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPUNPCKLQDQYrr, X86_INS_VPUNPCKLQDQ, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPUNPCKLQDQZrm, X86_INS_VPUNPCKLQDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPUNPCKLQDQZrr, X86_INS_VPUNPCKLQDQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPUNPCKLQDQrm, X86_INS_VPUNPCKLQDQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPUNPCKLQDQrr, X86_INS_VPUNPCKLQDQ, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPUNPCKLWDYrm, X86_INS_VPUNPCKLWD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPUNPCKLWDYrr, X86_INS_VPUNPCKLWD, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPUNPCKLWDrm, X86_INS_VPUNPCKLWD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPUNPCKLWDrr, X86_INS_VPUNPCKLWD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPXORDZrm, X86_INS_VPXORD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPXORDZrmb, X86_INS_VPXORD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPXORDZrr, X86_INS_VPXORD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPXORQZrm, X86_INS_VPXORQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPXORQZrmb, X86_INS_VPXORQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPXORQZrr, X86_INS_VPXORQ, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VPXORYrm, X86_INS_VPXOR, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPXORYrr, X86_INS_VPXOR, { 0 }, { 0 }, { X86_GRP_AVX2, 0 } },
	{ X86_VPXORrm, X86_INS_VPXOR, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VPXORrr, X86_INS_VPXOR, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRCP14PDZm, X86_INS_VRCP14PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRCP14PDZm_Int, X86_INS_VRCP14PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRCP14PDZr, X86_INS_VRCP14PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRCP14PDZr_Int, X86_INS_VRCP14PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRCP14PSZm, X86_INS_VRCP14PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRCP14PSZm_Int, X86_INS_VRCP14PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRCP14PSZr, X86_INS_VRCP14PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRCP14PSZr_Int, X86_INS_VRCP14PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRCP14SDZm, X86_INS_VRCP14SD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRCP14SDZm_Int, X86_INS_VRCP14SD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRCP14SDZr, X86_INS_VRCP14SD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRCP14SSZm, X86_INS_VRCP14SS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRCP14SSZm_Int, X86_INS_VRCP14SS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRCP14SSZr, X86_INS_VRCP14SS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRCPPSYm, X86_INS_VRCPPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRCPPSYm_Int, X86_INS_VRCPPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRCPPSYr, X86_INS_VRCPPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRCPPSYr_Int, X86_INS_VRCPPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRCPPSm, X86_INS_VRCPPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRCPPSm_Int, X86_INS_VRCPPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRCPPSr, X86_INS_VRCPPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRCPPSr_Int, X86_INS_VRCPPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRCPSSm, X86_INS_VRCPSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRCPSSm_Int, X86_INS_VRCPSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRCPSSr, X86_INS_VRCPSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRNDSCALESDm, X86_INS_VRNDSCALESD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRNDSCALESDr, X86_INS_VRNDSCALESD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRNDSCALESDr_Int, X86_INS_VRNDSCALESD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRNDSCALESSm, X86_INS_VRNDSCALESS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRNDSCALESSr, X86_INS_VRNDSCALESS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRNDSCALESSr_Int, X86_INS_VRNDSCALESS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRNDSCALEZPDm, X86_INS_VRNDSCALEPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRNDSCALEZPDr, X86_INS_VRNDSCALEPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRNDSCALEZPSm, X86_INS_VRNDSCALEPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRNDSCALEZPSr, X86_INS_VRNDSCALEPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VROUNDPDm, X86_INS_VROUNDPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VROUNDPDr, X86_INS_VROUNDPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VROUNDPSm, X86_INS_VROUNDPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VROUNDPSr, X86_INS_VROUNDPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VROUNDSDm, X86_INS_VROUNDSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VROUNDSDr, X86_INS_VROUNDSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VROUNDSDr_Int, X86_INS_VROUNDSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VROUNDSSm, X86_INS_VROUNDSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VROUNDSSr, X86_INS_VROUNDSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VROUNDSSr_Int, X86_INS_VROUNDSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VROUNDYPDm, X86_INS_VROUNDPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VROUNDYPDr, X86_INS_VROUNDPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VROUNDYPSm, X86_INS_VROUNDPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VROUNDYPSr, X86_INS_VROUNDPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRSQRT14PDZm, X86_INS_VRSQRT14PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRSQRT14PDZm_Int, X86_INS_VRSQRT14PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRSQRT14PDZr, X86_INS_VRSQRT14PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRSQRT14PDZr_Int, X86_INS_VRSQRT14PD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRSQRT14PSZm, X86_INS_VRSQRT14PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRSQRT14PSZm_Int, X86_INS_VRSQRT14PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRSQRT14PSZr, X86_INS_VRSQRT14PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRSQRT14PSZr_Int, X86_INS_VRSQRT14PS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRSQRT14SDZm, X86_INS_VRSQRT14SD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRSQRT14SDZm_Int, X86_INS_VRSQRT14SD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRSQRT14SDZr, X86_INS_VRSQRT14SD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRSQRT14SSZm, X86_INS_VRSQRT14SS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRSQRT14SSZm_Int, X86_INS_VRSQRT14SS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRSQRT14SSZr, X86_INS_VRSQRT14SS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VRSQRTPSYm, X86_INS_VRSQRTPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRSQRTPSYm_Int, X86_INS_VRSQRTPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRSQRTPSYr, X86_INS_VRSQRTPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRSQRTPSYr_Int, X86_INS_VRSQRTPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRSQRTPSm, X86_INS_VRSQRTPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRSQRTPSm_Int, X86_INS_VRSQRTPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRSQRTPSr, X86_INS_VRSQRTPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRSQRTPSr_Int, X86_INS_VRSQRTPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRSQRTSSm, X86_INS_VRSQRTSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRSQRTSSm_Int, X86_INS_VRSQRTSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VRSQRTSSr, X86_INS_VRSQRTSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSCATTERDPDZmr, X86_INS_VSCATTERDPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSCATTERDPSZmr, X86_INS_VSCATTERDPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSCATTERQPDZmr, X86_INS_VSCATTERQPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSCATTERQPSZmr, X86_INS_VSCATTERQPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSHUFPDYrmi, X86_INS_VSHUFPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSHUFPDYrri, X86_INS_VSHUFPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSHUFPDZrmi, X86_INS_VSHUFPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSHUFPDZrri, X86_INS_VSHUFPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSHUFPDrmi, X86_INS_VSHUFPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSHUFPDrri, X86_INS_VSHUFPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSHUFPSYrmi, X86_INS_VSHUFPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSHUFPSYrri, X86_INS_VSHUFPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSHUFPSZrmi, X86_INS_VSHUFPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSHUFPSZrri, X86_INS_VSHUFPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSHUFPSrmi, X86_INS_VSHUFPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSHUFPSrri, X86_INS_VSHUFPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSQRTPDYm, X86_INS_VSQRTPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSQRTPDYr, X86_INS_VSQRTPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSQRTPDZm_Int, X86_INS_VSQRTPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSQRTPDZr_Int, X86_INS_VSQRTPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSQRTPDZrm, X86_INS_VSQRT, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSQRTPDZrr, X86_INS_VSQRT, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSQRTPDm, X86_INS_VSQRTPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSQRTPDr, X86_INS_VSQRTPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSQRTPSYm, X86_INS_VSQRTPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSQRTPSYr, X86_INS_VSQRTPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSQRTPSZm_Int, X86_INS_VSQRTPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSQRTPSZr_Int, X86_INS_VSQRTPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSQRTPSZrm, X86_INS_VSQRT, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSQRTPSZrr, X86_INS_VSQRT, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSQRTPSm, X86_INS_VSQRTPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSQRTPSr, X86_INS_VSQRTPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSQRTSDZm, X86_INS_VSQRTSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSQRTSDZm_Int, X86_INS_VSQRTSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSQRTSDZr, X86_INS_VSQRTSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSQRTSDZr_Int, X86_INS_VSQRTSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSQRTSDm, X86_INS_VSQRTSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSQRTSDm_Int, X86_INS_VSQRTSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSQRTSDr, X86_INS_VSQRTSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSQRTSSZm, X86_INS_VSQRTSS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSQRTSSZm_Int, X86_INS_VSQRTSS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSQRTSSZr, X86_INS_VSQRTSS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSQRTSSZr_Int, X86_INS_VSQRTSS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSQRTSSm, X86_INS_VSQRTSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSQRTSSm_Int, X86_INS_VSQRTSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSQRTSSr, X86_INS_VSQRTSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSTMXCSR, X86_INS_VSTMXCSR, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSUBPDYrm, X86_INS_VSUBPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSUBPDYrr, X86_INS_VSUBPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSUBPDZrm, X86_INS_VSUBPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSUBPDZrmb, X86_INS_VSUBPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSUBPDZrr, X86_INS_VSUBPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSUBPDrm, X86_INS_VSUBPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSUBPDrr, X86_INS_VSUBPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSUBPSYrm, X86_INS_VSUBPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSUBPSYrr, X86_INS_VSUBPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSUBPSZrm, X86_INS_VSUBPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSUBPSZrmb, X86_INS_VSUBPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSUBPSZrr, X86_INS_VSUBPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSUBPSrm, X86_INS_VSUBPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSUBPSrr, X86_INS_VSUBPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSUBSDZrm, X86_INS_VSUBSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSUBSDZrr, X86_INS_VSUBSD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSUBSDrm, X86_INS_VSUBSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSUBSDrm_Int, X86_INS_VSUBSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSUBSDrr, X86_INS_VSUBSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSUBSDrr_Int, X86_INS_VSUBSD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSUBSSZrm, X86_INS_VSUBSS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSUBSSZrr, X86_INS_VSUBSS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VSUBSSrm, X86_INS_VSUBSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSUBSSrm_Int, X86_INS_VSUBSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSUBSSrr, X86_INS_VSUBSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VSUBSSrr_Int, X86_INS_VSUBSS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VTESTPDYrm, X86_INS_VTESTPD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VTESTPDYrr, X86_INS_VTESTPD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VTESTPDrm, X86_INS_VTESTPD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VTESTPDrr, X86_INS_VTESTPD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VTESTPSYrm, X86_INS_VTESTPS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VTESTPSYrr, X86_INS_VTESTPS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VTESTPSrm, X86_INS_VTESTPS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VTESTPSrr, X86_INS_VTESTPS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VUCOMISDZrm, X86_INS_VUCOMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VUCOMISDZrr, X86_INS_VUCOMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VUCOMISDrm, X86_INS_VUCOMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VUCOMISDrr, X86_INS_VUCOMISD, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VUCOMISSZrm, X86_INS_VUCOMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VUCOMISSZrr, X86_INS_VUCOMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VUCOMISSrm, X86_INS_VUCOMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VUCOMISSrr, X86_INS_VUCOMISS, { 0 }, { X86_REG_FLAGS, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VUNPCKHPDYrm, X86_INS_VUNPCKHPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VUNPCKHPDYrr, X86_INS_VUNPCKHPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VUNPCKHPDZrm, X86_INS_VUNPCKHPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VUNPCKHPDZrr, X86_INS_VUNPCKHPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VUNPCKHPDrm, X86_INS_VUNPCKHPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VUNPCKHPDrr, X86_INS_VUNPCKHPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VUNPCKHPSYrm, X86_INS_VUNPCKHPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VUNPCKHPSYrr, X86_INS_VUNPCKHPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VUNPCKHPSZrm, X86_INS_VUNPCKHPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VUNPCKHPSZrr, X86_INS_VUNPCKHPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VUNPCKHPSrm, X86_INS_VUNPCKHPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VUNPCKHPSrr, X86_INS_VUNPCKHPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VUNPCKLPDYrm, X86_INS_VUNPCKLPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VUNPCKLPDYrr, X86_INS_VUNPCKLPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VUNPCKLPDZrm, X86_INS_VUNPCKLPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VUNPCKLPDZrr, X86_INS_VUNPCKLPD, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VUNPCKLPDrm, X86_INS_VUNPCKLPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VUNPCKLPDrr, X86_INS_VUNPCKLPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VUNPCKLPSYrm, X86_INS_VUNPCKLPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VUNPCKLPSYrr, X86_INS_VUNPCKLPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VUNPCKLPSZrm, X86_INS_VUNPCKLPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VUNPCKLPSZrr, X86_INS_VUNPCKLPS, { 0 }, { 0 }, { X86_GRP_AVX512, 0 } },
	{ X86_VUNPCKLPSrm, X86_INS_VUNPCKLPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VUNPCKLPSrr, X86_INS_VUNPCKLPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VXORPDYrm, X86_INS_VXORPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VXORPDYrr, X86_INS_VXORPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VXORPDrm, X86_INS_VXORPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VXORPDrr, X86_INS_VXORPD, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VXORPSYrm, X86_INS_VXORPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VXORPSYrr, X86_INS_VXORPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VXORPSrm, X86_INS_VXORPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VXORPSrr, X86_INS_VXORPS, { 0 }, { 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VZEROALL, X86_INS_VZEROALL, { 0 }, { X86_REG_YMM0, X86_REG_YMM1, X86_REG_YMM2, X86_REG_YMM3, X86_REG_YMM4, X86_REG_YMM5, X86_REG_YMM6, X86_REG_YMM7, X86_REG_YMM8, X86_REG_YMM9, X86_REG_YMM10, X86_REG_YMM11, X86_REG_YMM12, X86_REG_YMM13, X86_REG_YMM14, X86_REG_YMM15, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_VZEROUPPER, X86_INS_VZEROUPPER, { 0 }, { X86_REG_YMM0, X86_REG_YMM1, X86_REG_YMM2, X86_REG_YMM3, X86_REG_YMM4, X86_REG_YMM5, X86_REG_YMM6, X86_REG_YMM7, X86_REG_YMM8, X86_REG_YMM9, X86_REG_YMM10, X86_REG_YMM11, X86_REG_YMM12, X86_REG_YMM13, X86_REG_YMM14, X86_REG_YMM15, 0 }, { X86_GRP_AVX, 0 } },
	{ X86_W64ALLOCA, X86_INS_CALL, { X86_REG_RSP, 0 }, { X86_REG_RAX, X86_REG_R10, X86_REG_R11, X86_REG_RSP, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_WAIT, X86_INS_WAIT, { 0 }, { 0 }, { 0 } },
	{ X86_WBINVD, X86_INS_WBINVD, { 0 }, { 0 }, { 0 } },
	{ X86_WRFSBASE, X86_INS_WRFSBASE, { 0 }, { 0 }, { X86_GRP_FSGSBASE, X86_GRP_MODE64, 0 } },
	{ X86_WRFSBASE64, X86_INS_WRFSBASE, { 0 }, { 0 }, { X86_GRP_FSGSBASE, X86_GRP_MODE64, 0 } },
	{ X86_WRGSBASE, X86_INS_WRGSBASE, { 0 }, { 0 }, { X86_GRP_FSGSBASE, X86_GRP_MODE64, 0 } },
	{ X86_WRGSBASE64, X86_INS_WRGSBASE, { 0 }, { 0 }, { X86_GRP_FSGSBASE, X86_GRP_MODE64, 0 } },
	{ X86_WRMSR, X86_INS_WRMSR, { 0 }, { 0 }, { 0 } },
	{ X86_XABORT, X86_INS_XABORT, { 0 }, { 0 }, { X86_GRP_RTM, 0 } },
	{ X86_XACQUIRE_PREFIX, X86_INS_XACQUIRE, { 0 }, { 0 }, { X86_GRP_HLE, 0 } },
	{ X86_XADD16rm, X86_INS_XADD, { 0 }, { 0 }, { 0 } },
	{ X86_XADD16rr, X86_INS_XADD, { 0 }, { 0 }, { 0 } },
	{ X86_XADD32rm, X86_INS_XADD, { 0 }, { 0 }, { 0 } },
	{ X86_XADD32rr, X86_INS_XADD, { 0 }, { 0 }, { 0 } },
	{ X86_XADD64rm, X86_INS_XADD, { 0 }, { 0 }, { 0 } },
	{ X86_XADD64rr, X86_INS_XADD, { 0 }, { 0 }, { 0 } },
	{ X86_XADD8rm, X86_INS_XADD, { 0 }, { 0 }, { 0 } },
	{ X86_XADD8rr, X86_INS_XADD, { 0 }, { 0 }, { 0 } },
	{ X86_XBEGIN_4, X86_INS_XBEGIN, { 0 }, { X86_REG_EAX, 0 }, { X86_GRP_RTM, 0 } },
	{ X86_XCHG16ar, X86_INS_XCHG, { 0 }, { 0 }, { 0 } },
	{ X86_XCHG16rm, X86_INS_XCHG, { 0 }, { 0 }, { 0 } },
	{ X86_XCHG16rr, X86_INS_XCHG, { 0 }, { 0 }, { 0 } },
	{ X86_XCHG32ar, X86_INS_XCHG, { 0 }, { 0 }, { X86_GRP_MODE32, 0 } },
	{ X86_XCHG32ar64, X86_INS_XCHG, { 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_XCHG32rm, X86_INS_XCHG, { 0 }, { 0 }, { 0 } },
	{ X86_XCHG32rr, X86_INS_XCHG, { 0 }, { 0 }, { 0 } },
	{ X86_XCHG64ar, X86_INS_XCHG, { 0 }, { 0 }, { 0 } },
	{ X86_XCHG64rm, X86_INS_XCHG, { 0 }, { 0 }, { 0 } },
	{ X86_XCHG64rr, X86_INS_XCHG, { 0 }, { 0 }, { 0 } },
	{ X86_XCHG8rm, X86_INS_XCHG, { 0 }, { 0 }, { 0 } },
	{ X86_XCHG8rr, X86_INS_XCHG, { 0 }, { 0 }, { 0 } },
	{ X86_XCH_F, X86_INS_FXCH, { 0 }, { X86_REG_FPSW, 0 }, { 0 } },
	{ X86_XCRYPTCBC, X86_INS_XCRYPTCBC, { X86_REG_RBX, X86_REG_RDX, X86_REG_RSI, X86_REG_RDI, 0 }, { X86_REG_RSI, X86_REG_RDI, 0 }, { 0 } },
	{ X86_XCRYPTCFB, X86_INS_XCRYPTCFB, { X86_REG_RBX, X86_REG_RDX, X86_REG_RSI, X86_REG_RDI, 0 }, { X86_REG_RSI, X86_REG_RDI, 0 }, { 0 } },
	{ X86_XCRYPTCTR, X86_INS_XCRYPTCTR, { X86_REG_RBX, X86_REG_RDX, X86_REG_RSI, X86_REG_RDI, 0 }, { X86_REG_RSI, X86_REG_RDI, 0 }, { 0 } },
	{ X86_XCRYPTECB, X86_INS_XCRYPTECB, { X86_REG_RBX, X86_REG_RDX, X86_REG_RSI, X86_REG_RDI, 0 }, { X86_REG_RSI, X86_REG_RDI, 0 }, { 0 } },
	{ X86_XCRYPTOFB, X86_INS_XCRYPTOFB, { X86_REG_RBX, X86_REG_RDX, X86_REG_RSI, X86_REG_RDI, 0 }, { X86_REG_RSI, X86_REG_RDI, 0 }, { 0 } },
	{ X86_XEND, X86_INS_XEND, { 0 }, { 0 }, { X86_GRP_RTM, 0 } },
	{ X86_XGETBV, X86_INS_XGETBV, { X86_REG_RCX, 0 }, { X86_REG_RDX, X86_REG_RAX, 0 }, { 0 } },
	{ X86_XLAT, X86_INS_XLATB, { 0 }, { 0 }, { 0 } },
	{ X86_XOR16i16, X86_INS_XOR, { X86_REG_AX, 0 }, { X86_REG_AX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR16mi, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR16mi8, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR16mr, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR16ri, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR16ri8, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR16rm, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR16rr, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR16rr_REV, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR32i32, X86_INS_XOR, { X86_REG_EAX, 0 }, { X86_REG_EAX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR32mi, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR32mi8, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR32mr, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR32ri, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR32ri8, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR32rm, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR32rr, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR32rr_REV, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR64i32, X86_INS_XOR, { X86_REG_RAX, 0 }, { X86_REG_RAX, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR64mi32, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR64mi8, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR64mr, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR64ri32, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR64ri8, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR64rm, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR64rr, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR64rr_REV, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR8i8, X86_INS_XOR, { X86_REG_AL, 0 }, { X86_REG_AL, X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR8mi, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR8mr, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR8ri, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR8rm, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR8rr, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XOR8rr_REV, X86_INS_XOR, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
	{ X86_XORPDrm, X86_INS_XORPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_XORPDrr, X86_INS_XORPD, { 0 }, { 0 }, { X86_GRP_SSE2, 0 } },
	{ X86_XORPSrm, X86_INS_XORPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_XORPSrr, X86_INS_XORPS, { 0 }, { 0 }, { X86_GRP_SSE1, 0 } },
	{ X86_XRELEASE_PREFIX, X86_INS_XRELEASE, { 0 }, { 0 }, { X86_GRP_HLE, 0 } },
	{ X86_XRSTOR, X86_INS_XRSTOR, { X86_REG_RDX, X86_REG_RAX, 0 }, { 0 }, { 0 } },
	{ X86_XRSTOR64, X86_INS_XRSTOR64, { X86_REG_RDX, X86_REG_RAX, 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_XSAVE, X86_INS_XSAVE, { X86_REG_RDX, X86_REG_RAX, 0 }, { 0 }, { 0 } },
	{ X86_XSAVE64, X86_INS_XSAVE64, { X86_REG_RDX, X86_REG_RAX, 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_XSAVEOPT, X86_INS_XSAVEOPT, { X86_REG_RDX, X86_REG_RAX, 0 }, { 0 }, { 0 } },
	{ X86_XSAVEOPT64, X86_INS_XSAVEOPT64, { X86_REG_RDX, X86_REG_RAX, 0 }, { 0 }, { X86_GRP_MODE64, 0 } },
	{ X86_XSETBV, X86_INS_XSETBV, { X86_REG_RDX, X86_REG_RAX, X86_REG_RCX, 0 }, { 0 }, { 0 } },
	{ X86_XSHA1, X86_INS_XSHA1, { X86_REG_RAX, X86_REG_RSI, X86_REG_RDI, 0 }, { X86_REG_RAX, X86_REG_RSI, X86_REG_RDI, 0 }, { 0 } },
	{ X86_XSHA256, X86_INS_XSHA256, { X86_REG_RAX, X86_REG_RSI, X86_REG_RDI, 0 }, { X86_REG_RAX, X86_REG_RSI, X86_REG_RDI, 0 }, { 0 } },
	{ X86_XSTORE, X86_INS_XSTORE, { X86_REG_RDX, X86_REG_RDI, 0 }, { X86_REG_RAX, X86_REG_RDI, 0 }, { 0 } },
	{ X86_XTEST, X86_INS_XTEST, { 0 }, { X86_REG_FLAGS, 0 }, { 0 } },
};

void X86_get_insn_id(cs_insn *insn, unsigned int id)
{
	int i = insn_find(insns, ARR_SIZE(insns), id);
	if (i != -1) {
		insn->id = insns[i].mapid;
		memcpy(insn->regs_read, insns[i].regs_use, sizeof(insns[i].regs_use));
		memcpy(insn->regs_write, insns[i].regs_mod, sizeof(insns[i].regs_mod));
		memcpy(insn->groups, insns[i].groups, sizeof(insns[i].groups));
	}
}

unsigned int X86_get_insn_id2(unsigned int id)
{
	return insn_reverse_id(insns, ARR_SIZE(insns), id);
}

